@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006-2008 Linpro AS
+ * Copyright (c) 2006-2010 Redpill Linpro AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -26,7 +26,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: cache.h 3723 2009-02-10 14:50:53Z tfheen $
+ * $Id: cache.h 4582 2010-02-21 22:00:09Z phk $
  */
 
 /*
@@ -48,6 +48,10 @@
 #include <limits.h>
 #include <unistd.h>
 
+#if defined(HAVE_EPOLL_CTL)
+#include <sys/epoll.h>
+#endif
+
 #include "vqueue.h"
 
 #include "vsb.h"
@@ -58,8 +62,6 @@
 #include "heritage.h"
 #include "miniobj.h"
 
-#define HTTP_HDR_MAX_VAL 32
-
 enum {
 	/* Fields from the first line of HTTP proto */
 	HTTP_HDR_REQ,
@@ -69,33 +71,30 @@ enum {
 	HTTP_HDR_RESPONSE,
 	/* HTTP header lines */
 	HTTP_HDR_FIRST,
-	HTTP_HDR_MAX = HTTP_HDR_MAX_VAL
 };
 
-/* Note: intentionally not IOV_MAX unless it has to be */
-#if (IOV_MAX < (HTTP_HDR_MAX_VAL * 2))
-#  define MAX_IOVS	IOV_MAX
-#else
-#  define MAX_IOVS	(HTTP_HDR_MAX_VAL * 2)
-#endif
-
 struct cli;
 struct vsb;
 struct sess;
 struct director;
 struct object;
 struct objhead;
-struct objexp;
+struct objcore;
 struct workreq;
-struct addrinfo;
-struct esi_bit;
+struct esidata;
 struct vrt_backend;
 struct cli_proto;
 struct ban;
 struct SHA256Context;
 
+struct smp_object;
+struct smp_seg;
+
 struct lock { void *priv; };		// Opaque
 
+#define DIGEST_LEN		32
+
+
 /*--------------------------------------------------------------------*/
 
 typedef struct {
@@ -147,10 +146,12 @@ struct http {
 	int			status;
 	double			protover;
 
-	txt			hd[HTTP_HDR_MAX];
-	unsigned char		hdf[HTTP_HDR_MAX];
+	unsigned		shd;		/* Size of hd space */
+	txt			*hd;
+	unsigned char		*hdf;
 #define HDF_FILTER		(1 << 0)	/* Filtered by Connection */
-	unsigned		nhd;
+#define HDF_COPY		(1 << 1)	/* Copy this field */
+	unsigned		nhd;		/* Next free hd */
 };
 
 /*--------------------------------------------------------------------
@@ -178,11 +179,24 @@ struct acct {
 
 /*--------------------------------------------------------------------*/
 
+#define L0(n)
+#define L1(n)			int n;
+#define MAC_STAT(n, t, l, f, e)	L##l(n)
+struct dstat {
+#include "stat_field.h"
+};
+#undef MAC_STAT
+#undef L0
+#undef L1
+
+/*--------------------------------------------------------------------*/
+
 struct worker {
 	unsigned		magic;
 #define WORKER_MAGIC		0x6391adcf
 	struct objhead		*nobjhead;
-	struct object		*nobj;
+	struct objcore		*nobjcore;
+	struct dstat		stats;
 
 	double			lastused;
 
@@ -193,17 +207,38 @@ struct worker {
 
 	int			*wfd;
 	unsigned		werr;	/* valid after WRK_Flush() */
-	struct iovec		iov[MAX_IOVS];
-	int			niov;
+	struct iovec		*iov;
+	unsigned		siov;
+	unsigned		niov;
 	ssize_t			liov;
 
 	struct VCL_conf		*vcl;
-	struct srcaddr		*srcaddr;
 
 	unsigned char		*wlb, *wlp, *wle;
 	unsigned		wlr;
 
 	struct SHA256Context	*sha256ctx;
+
+	struct http_conn	htc[1];
+	struct ws		ws[1];
+	struct http		*http[3];
+	struct http		*bereq;
+	struct http		*beresp1;
+	struct http		*beresp;
+	struct http		*resp;
+
+	unsigned		cacheable;
+	double			age;
+	double			entered;
+	double			ttl;
+	double			grace;
+	unsigned		do_esi;
+
+	/* Timeouts */
+	double			connect_timeout;
+	double			first_byte_timeout;
+	double			between_bytes_timeout;
+
 };
 
 /* Work Request for worker thread ------------------------------------*/
@@ -221,16 +256,6 @@ struct workreq {
 	void			*priv;
 };
 
-/* Backend Request ---------------------------------------------------*/
-
-struct bereq {
-	unsigned		magic;
-#define BEREQ_MAGIC		0x3b6d250c
-	VTAILQ_ENTRY(bereq)	list;
-	struct ws		ws[1];
-	struct http		http[1];
-};
-
 /* Storage -----------------------------------------------------------*/
 
 struct storage {
@@ -248,52 +273,82 @@ struct storage {
 	off_t			where;
 };
 
+/* Object core structure ---------------------------------------------
+ * Objects have sideways references in the binary heap and the LRU list
+ * and we want to avoid paging in a lot of objects just to move them up
+ * or down the binheap or to move a unrelated object on the LRU list.
+ * To avoid this we use a proxy object, objcore, to hold the relevant
+ * housekeeping fields parts of an object.
+ */
+
+struct objcore {
+	unsigned		magic;
+#define OBJCORE_MAGIC		0x4d301302
+	unsigned		refcnt;
+	struct object		*obj;
+	struct objhead		*objhead;
+	double			timer_when;
+	unsigned		flags;
+#define OC_F_ONLRU		(1<<0)
+#define OC_F_BUSY		(1<<1)
+#define OC_F_PASS		(1<<2)
+#define OC_F_PERSISTENT		(1<<3)
+#define OC_F_LRUDONTMOVE	(1<<4)
+	unsigned		timer_idx;
+	VTAILQ_ENTRY(objcore)	list;
+	VLIST_ENTRY(objcore)	lru_list;
+	VTAILQ_ENTRY(objcore)	ban_list;
+	struct smp_seg		*smp_seg;
+	struct ban		*ban;
+};
+
+/*--------------------------------------------------------------------*/
+
+struct lru {
+	unsigned		magic;
+#define LRU_MAGIC		0x3fec7bb0
+	VLIST_HEAD(,objcore)	lru_head;
+	struct objcore		senteniel;
+};
+
 /* Object structure --------------------------------------------------*/
 
 struct object {
 	unsigned		magic;
 #define OBJECT_MAGIC		0x32851d42
-	unsigned		refcnt;
 	unsigned		xid;
-	struct objhead		*objhead;
 	struct storage		*objstore;
-	struct objexp		*objexp;
+	struct objcore		*objcore;
+
+	unsigned		smp_index;
 
 	struct ws		ws_o[1];
 	unsigned char		*vary;
 
-	struct ban		*ban;
-
-	unsigned		pass;
-
+	double			ban_t;
+	struct ban		*ban;	/* XXX --> objcore */
 	unsigned		response;
 
 	unsigned		cacheable;
 
-	unsigned		busy;
 	unsigned		len;
 
+	double			ttl;
 	double			age;
 	double			entered;
-	double			ttl;
 	double			grace;
-	double			prefetch;
 
 	double			last_modified;
+	double			last_lru;
 
-	struct http		http[1];
-	VTAILQ_ENTRY(object)	list;
+	struct http		*http;
 
 	VTAILQ_HEAD(, storage)	store;
 
-	VTAILQ_HEAD(, esi_bit)	esibits;
+	struct esidata		*esidata;
 
 	double			last_use;
 
-	/* Prefetch */
-	struct object		*parent;
-	struct object		*child;
-
 	int			hits;
 };
 
@@ -308,6 +363,7 @@ struct sess {
 
 	int			restarts;
 	int			esis;
+	int			disable_esi;
 
 	struct worker		*wrk;
 
@@ -320,7 +376,6 @@ struct sess {
 	/* formatted ascii client address */
 	char			*addr;
 	char			*port;
-	struct srcaddr		*srcaddr;
 
 	/* HTTP request */
 	const char		*doclose;
@@ -331,6 +386,8 @@ struct sess {
 	char			*ws_ses;	/* WS above session data */
 	char			*ws_req;	/* WS above request data */
 
+	unsigned char		digest[DIGEST_LEN];
+
 	struct http_conn	htc[1];
 
 	/* Timestamps, all on TIM_real() timescale */
@@ -339,11 +396,6 @@ struct sess {
 	double			t_resp;
 	double			t_end;
 
-	/* Timeouts */
-	double connect_timeout;
-	double first_byte_timeout;
-	double between_bytes_timeout;
-
 	/* Acceptable grace period */
 	double			grace;
 
@@ -359,8 +411,8 @@ struct sess {
 
 	struct director		*director;
 	struct vbe_conn		*vbe;
-	struct bereq		*bereq;
 	struct object		*obj;
+	struct objcore		*objcore;
 	struct objhead		*objhead;
 	struct VCL_conf		*vcl;
 
@@ -371,14 +423,11 @@ struct sess {
 	struct acct		acct;
 	struct acct		acct_req;
 
-	/* pointers to hash string components */
-	unsigned		nhashptr;
-	unsigned		ihashptr;
-	unsigned		lhashptr;
-	const char		**hashptr;
+#if defined(HAVE_EPOLL_CTL)
+	struct epoll_event ev;
+#endif
 };
 
-
 /* -------------------------------------------------------------------*/
 
 /* Backend connection */
@@ -388,24 +437,30 @@ struct vbe_conn {
 	VTAILQ_ENTRY(vbe_conn)	list;
 	struct backend		*backend;
 	int			fd;
+
+	/* Timeouts */
+	double			first_byte_timeout;
+	double			between_bytes_timeout;
 };
 
 /* Prototypes etc ----------------------------------------------------*/
 
-
 /* cache_acceptor.c */
 void vca_return_session(struct sess *sp);
 void vca_close_session(struct sess *sp, const char *why);
 void VCA_Prep(struct sess *sp);
 void VCA_Init(void);
+void VCA_Shutdown(void);
+const char *VCA_waiter_name(void);
+extern pthread_t VCA_thread;
 
 /* cache_backend.c */
 
-void VBE_GetFd(struct sess *sp);
+struct vbe_conn *VBE_GetFd(const struct director *, struct sess *sp);
+int VBE_Healthy(double now, const struct director *, uintptr_t target);
+int VBE_Healthy_sp(const struct sess *sp, const struct director *);
 void VBE_ClosedFd(struct sess *sp);
 void VBE_RecycleFd(struct sess *sp);
-struct bereq * VBE_new_bereq(void);
-void VBE_free_bereq(struct bereq **bereq);
 void VBE_AddHostHeader(const struct sess *sp);
 void VBE_Poll(void);
 
@@ -418,13 +473,19 @@ void VBP_Init(void);
 
 /* cache_ban.c */
 struct ban *BAN_New(void);
-int BAN_AddTest(struct cli *, struct ban *, const char *, const char *, const char *);
+int BAN_AddTest(struct cli *, struct ban *, const char *, const char *,
+    const char *);
 void BAN_Free(struct ban *b);
 void BAN_Insert(struct ban *b);
 void BAN_Init(void);
 void BAN_NewObj(struct object *o);
 void BAN_DestroyObj(struct object *o);
 int BAN_CheckObject(struct object *o, const struct sess *sp);
+void BAN_Reload(double t0, unsigned flags, const char *ban);
+struct ban *BAN_TailRef(void);
+void BAN_Compile(void);
+struct ban *BAN_RefBan(struct objcore *oc, double t0, const struct ban *tail);
+void BAN_Deref(struct ban **ban);
 
 /* cache_center.c [CNT] */
 void CNT_Session(struct sess *sp);
@@ -433,44 +494,48 @@ void CNT_Init(void);
 /* cache_cli.c [CLI] */
 void CLI_Init(void);
 void CLI_Run(void);
-enum cli_set_e {MASTER_CLI, PUBLIC_CLI, DEBUG_CLI};
-void CLI_AddFuncs(enum cli_set_e which, struct cli_proto *p);
+void CLI_AddFuncs(struct cli_proto *p);
 extern pthread_t cli_thread;
 #define ASSERT_CLI() do {assert(pthread_self() == cli_thread);} while (0)
 
 /* cache_expiry.c */
 void EXP_Insert(struct object *o);
+void EXP_Inject(struct objcore *oc, struct lru *lru, double ttl);
 void EXP_Init(void);
 void EXP_Rearm(const struct object *o);
-void EXP_Touch(const struct object *o, double now);
-int EXP_NukeOne(struct sess *sp);
+int EXP_Touch(const struct object *o);
+int EXP_NukeOne(const struct sess *sp, const struct lru *lru);
 
 /* cache_fetch.c */
-int Fetch(struct sess *sp);
+int FetchHdr(struct sess *sp);
+int FetchBody(struct sess *sp);
 int FetchReqBody(struct sess *sp);
 void Fetch_Init(void);
 
 /* cache_http.c */
+unsigned HTTP_estimate(unsigned nhttp);
+struct http *HTTP_create(void *p, unsigned nhttp);
 const char *http_StatusMessage(unsigned);
+unsigned http_EstimateWS(const struct http *fm, unsigned how, unsigned *nhd);
 void HTTP_Init(void);
 void http_ClrHeader(struct http *to);
 unsigned http_Write(struct worker *w, const struct http *hp, int resp);
-void http_CopyResp(struct http *to, const struct http *fm);
-void http_SetResp(struct http *to, const char *proto, const char *status,
+void http_CopyResp(const struct http *to, const struct http *fm);
+void http_SetResp(const struct http *to, const char *proto, const char *status,
     const char *response);
 void http_FilterFields(struct worker *w, int fd, struct http *to,
     const struct http *fm, unsigned how);
-void http_FilterHeader(struct sess *sp, unsigned how);
-void http_PutProtocol(struct worker *w, int fd, struct http *to,
+void http_FilterHeader(const struct sess *sp, unsigned how);
+void http_PutProtocol(struct worker *w, int fd, const struct http *to,
     const char *protocol);
 void http_PutStatus(struct worker *w, int fd, struct http *to, int status);
-void http_PutResponse(struct worker *w, int fd, struct http *to,
+void http_PutResponse(struct worker *w, int fd, const struct http *to,
     const char *response);
 void http_PrintfHeader(struct worker *w, int fd, struct http *to,
     const char *fmt, ...);
 void http_SetHeader(struct worker *w, int fd, struct http *to, const char *hdr);
-void http_SetH(struct http *to, unsigned n, const char *fm);
-void http_ForceGet(struct http *to);
+void http_SetH(const struct http *to, unsigned n, const char *fm);
+void http_ForceGet(const struct http *to);
 void http_Setup(struct http *ht, struct ws *ws);
 int http_GetHdr(const struct http *hp, const char *hdr, char **ptr);
 int http_GetHdrField(const struct http *hp, const char *hdr,
@@ -481,8 +546,8 @@ int http_HdrIs(const struct http *hp, const char *hdr, const char *val);
 int http_DissectRequest(struct sess *sp);
 int http_DissectResponse(struct worker *w, const struct http_conn *htc,
     struct http *sp);
-const char *http_DoConnection(struct http *hp);
-void http_CopyHome(struct worker *w, int fd, struct http *hp);
+const char *http_DoConnection(const struct http *hp);
+void http_CopyHome(struct worker *w, int fd, const struct http *hp);
 void http_Unset(struct http *hp, const char *hdr);
 
 /* cache_httpconn.c */
@@ -532,10 +597,10 @@ void PipeSession(struct sess *sp);
 /* cache_pool.c */
 void WRK_Init(void);
 int WRK_Queue(struct workreq *wrq);
-void WRK_QueueSession(struct sess *sp);
+int WRK_QueueSession(struct sess *sp);
+void WRK_SumStat(struct worker *w);
 
 void WRW_Reserve(struct worker *w, int *fd);
-void WRW_Release(struct worker *w);
 unsigned WRW_Flush(struct worker *w);
 unsigned WRW_FlushRelease(struct worker *w);
 unsigned WRW_Write(struct worker *w, const void *ptr, int len);
@@ -544,14 +609,16 @@ unsigned WRW_WriteH(struct worker *w, const txt *hh, const char *suf);
 void WRW_Sendfile(struct worker *w, int fd, off_t off, unsigned len);
 #endif  /* SENDFILE_WORKS */
 
+typedef void *bgthread_t(struct sess *, void *priv);
+void WRK_BgThread(pthread_t *thr, const char *name, bgthread_t *func,
+    void *priv);
+
 /* cache_session.c [SES] */
 void SES_Init(void);
-struct sess *SES_New(const struct sockaddr *addr, unsigned len);
+struct sess *SES_New(void);
+struct sess *SES_Alloc(void);
 void SES_Delete(struct sess *sp);
-void SES_RefSrcAddr(struct sess *sp);
 void SES_Charge(struct sess *sp);
-void SES_ResetBackendTimeouts(struct sess *sp);
-void SES_InheritBackendTimeouts(struct sess *sp);
 
 /* cache_shmlog.c */
 void VSL_Init(void);
@@ -587,7 +654,7 @@ void RES_BuildHttp(struct sess *sp);
 void RES_WriteObj(struct sess *sp);
 
 /* cache_vary.c */
-void VRY_Create(const struct sess *sp);
+struct vsb *VRY_Create(const struct sess *sp, const struct http *hp);
 int VRY_Match(const struct sess *sp, const unsigned char *vary);
 
 /* cache_vcl.c */
@@ -605,6 +672,7 @@ void VCL_Poll(void);
 
 void ESI_Deliver(struct sess *);
 void ESI_Destroy(struct object *);
+void ESI_Parse(struct sess *);
 
 /* cache_ws.c */
 
@@ -620,12 +688,20 @@ char *WS_Snapshot(struct ws *ws);
 unsigned WS_Free(const struct ws *ws);
 
 /* rfc2616.c */
-int RFC2616_cache_policy(const struct sess *sp, const struct http *hp);
+double RFC2616_Ttl(const struct sess *sp);
 
 /* storage_synth.c */
 struct vsb *SMS_Makesynth(struct object *obj);
 void SMS_Finish(struct object *obj);
 
+/* storage_persistent.c */
+void SMP_Fixup(struct sess *sp, const struct objhead *oh, struct objcore *oc);
+void SMP_BANchanged(const struct object *o, double t);
+void SMP_TTLchanged(const struct object *o);
+void SMP_FreeObj(const struct object *o);
+void SMP_Ready(void);
+void SMP_NewBan(double t0, const char *ban);
+
 /*
  * A normal pointer difference is signed, but we never want a negative value
  * so this little tool will make sure we don't get that.
@@ -674,3 +750,12 @@ Tadd(txt *t, const char *p, int l)
 		t->b = t->e;
 	}
 }
+
+static inline unsigned
+ObjIsBusy(const struct object *o)
+{
+	CHECK_OBJ_NOTNULL(o, OBJECT_MAGIC);
+	CHECK_OBJ_NOTNULL(o->objcore, OBJCORE_MAGIC);
+	return (o->objcore->flags & OC_F_BUSY);
+}
+
