@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006-2008 Linpro AS
+ * Copyright (c) 2006-2010 Redpill Linpro AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -26,8 +26,6 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: cache_center.c 3991 2009-03-23 12:42:15Z tfheen $
- *
  * This file contains the central state machine for pushing requests.
  *
  * We cannot just use direct calls because it is possible to kick a
@@ -59,6 +57,9 @@ DOT acceptor -> start [style=bold,color=green,weight=4]
 
 #include "config.h"
 
+#include "svnid.h"
+SVNID("$Id: cache_center.c 4605 2010-03-09 11:27:08Z phk $")
+
 #include <stdio.h>
 #include <errno.h>
 #include <math.h>
@@ -76,21 +77,21 @@ DOT acceptor -> start [style=bold,color=green,weight=4]
 #include "cli_priv.h"
 #include "cache.h"
 #include "hash_slinger.h"
+#include "stevedore.h"
+#include "vsha256.h"
 
 static unsigned xids;
 
 /*--------------------------------------------------------------------
- * AGAIN
- * We come here when we just completed a request and already have
- * received (part of) the next one.  Instead taking the detour
- * around the acceptor and then back to a worker, just stay in this
- * worker and do what it takes.
+ * WAIT
+ * Wait (briefly) until we have a full request in our htc.
  */
 
 static int
-cnt_again(struct sess *sp)
+cnt_wait(struct sess *sp)
 {
 	int i;
+	struct pollfd pfd[1];
 
 	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
 	AZ(sp->vcl);
@@ -98,14 +99,36 @@ cnt_again(struct sess *sp)
 	assert(sp->xid == 0);
 
 	i = HTC_Complete(sp->htc);
-	while (i == 0)
-		i = HTC_Rx(sp->htc);
+	if (i == 0 && params->session_linger > 0) {
+		pfd[0].fd = sp->fd;
+		pfd[0].events = POLLIN;
+		pfd[0].revents = 0;
+		i = poll(pfd, 1, params->session_linger);
+		if (i)
+			i = HTC_Rx(sp->htc);
+	}
+	if (i == 0) {
+		WSL(sp->wrk, SLT_Debug, sp->fd, "herding");
+		sp->wrk->stats.sess_herd++;
+		SES_Charge(sp);
+		sp->wrk = NULL;
+		vca_return_session(sp);
+		return (1);
+	}
 	if (i == 1) {
 		sp->step = STP_START;
-	} else {
-		vca_close_session(sp, "overflow");
-		sp->step = STP_DONE;
+		return (0);
 	}
+	if (i == -2) {
+		vca_close_session(sp, "overflow");
+		return (0);
+	}
+	if (i == -1 && Tlen(sp->htc->rxbuf) == 0 &&
+	    (errno == 0 || errno == ECONNRESET))
+		vca_close_session(sp, "EOF");
+	else
+		vca_close_session(sp, "error");
+	sp->step = STP_DONE;
 	return (0);
 }
 
@@ -155,13 +178,15 @@ cnt_deliver(struct sess *sp)
 	CHECK_OBJ_NOTNULL(sp->obj, OBJECT_MAGIC);
 	CHECK_OBJ_NOTNULL(sp->vcl, VCL_CONF_MAGIC);
 
-	AZ(sp->bereq);
-
 	sp->t_resp = TIM_real();
-	if (sp->obj->objhead != NULL) {
+	if (sp->obj->objcore != NULL) {
+		if ((sp->t_resp - sp->obj->last_lru) > params->lru_timeout &&
+		    EXP_Touch(sp->obj))
+			sp->obj->last_lru = sp->t_resp;	/* XXX: locking ? */
 		sp->obj->last_use = sp->t_resp;	/* XXX: locking ? */
-		EXP_Touch(sp->obj, sp->t_resp);
 	}
+	sp->wrk->resp = sp->wrk->http[2];
+	http_Setup(sp->wrk->resp, sp->wrk->ws);
 	RES_BuildHttp(sp);
 	VCL_deliver_method(sp);
 	switch (sp->handling) {
@@ -179,7 +204,8 @@ cnt_deliver(struct sess *sp)
 
 	RES_WriteObj(sp);
 	AZ(sp->wrk->wfd);
-	HSH_Deref(&sp->obj);
+	HSH_Deref(sp->wrk, &sp->obj);
+	sp->wrk->resp = NULL;
 	sp->step = STP_DONE;
 	return (0);
 }
@@ -198,7 +224,6 @@ static int
 cnt_done(struct sess *sp)
 {
 	double dh, dp, da;
-	struct pollfd pfd[1];
 	int i;
 
 	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
@@ -206,7 +231,6 @@ cnt_done(struct sess *sp)
 
 	AZ(sp->obj);
 	AZ(sp->vbe);
-	AZ(sp->bereq);
 	sp->director = NULL;
 	sp->restarts = 0;
 
@@ -235,48 +259,53 @@ cnt_done(struct sess *sp)
 	WSL_Flush(sp->wrk, 0);
 
 	/* If we did an ESI include, don't mess up our state */
-	if (sp->esis > 0)
+	if (sp->esis > 0) {
+		SES_Charge(sp);
 		return (1);
+	}
 
 	sp->t_req = NAN;
 
-	if (sp->fd >= 0 && sp->doclose != NULL)
+	if (sp->fd >= 0 && sp->doclose != NULL) {
+		/*
+		 * This is an orderly close of the connection; ditch nolinger
+		 * before we close, to get queued data transmitted.
+		 */
+		// XXX: not yet (void)TCP_linger(sp->fd, 0);
 		vca_close_session(sp, sp->doclose);
+	}
+
+	SES_Charge(sp);
+
 	if (sp->fd < 0) {
-		SES_Charge(sp);
-		VSL_stats->sess_closed++;
+		sp->wrk->stats.sess_closed++;
 		sp->wrk = NULL;
 		SES_Delete(sp);
 		return (1);
 	}
 
+	if (sp->wrk->stats.client_req >= params->wthread_stats_rate)
+		WRK_SumStat(sp->wrk);
 	/* Reset the workspace to the session-watermark */
 	WS_Reset(sp->ws, sp->ws_ses);
 
 	i = HTC_Reinit(sp->htc);
 	if (i == 1) {
-		VSL_stats->sess_pipeline++;
+		sp->wrk->stats.sess_pipeline++;
 		sp->step = STP_START;
 		return (0);
 	}
 	if (Tlen(sp->htc->rxbuf)) {
-		VSL_stats->sess_readahead++;
-		sp->step = STP_AGAIN;
+		sp->wrk->stats.sess_readahead++;
+		sp->step = STP_WAIT;
 		return (0);
 	}
 	if (params->session_linger > 0) {
-		pfd[0].fd = sp->fd;
-		pfd[0].events = POLLIN;
-		pfd[0].revents = 0;
-		i = poll(pfd, 1, params->session_linger);
-		if (i > 0) {
-			VSL_stats->sess_linger++;
-			sp->step = STP_AGAIN;
-			return (0);
-		}
+		sp->wrk->stats.sess_linger++;
+		sp->step = STP_WAIT;
+		return (0);
 	}
-	VSL_stats->sess_herd++;
-	SES_Charge(sp);
+	sp->wrk->stats.sess_herd++;
 	sp->wrk = NULL;
 	vca_return_session(sp);
 	return (1);
@@ -303,21 +332,24 @@ cnt_error(struct sess *sp)
 	char date[40];
 
 	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
-	AZ(sp->bereq);
 
 	w = sp->wrk;
 	if (sp->obj == NULL) {
 		HSH_Prealloc(sp);
-		sp->obj = sp->wrk->nobj;
+		sp->wrk->cacheable = 0;
+		/* XXX: 1024 is a pure guess */
+		sp->obj = STV_NewObject(sp, 1024, 0, params->http_headers);
 		sp->obj->xid = sp->xid;
 		sp->obj->entered = sp->t_req;
-		sp->wrk->nobj = NULL;
 	} else {
 		/* XXX: Null the headers ? */
 	}
 	CHECK_OBJ_NOTNULL(sp->obj, OBJECT_MAGIC);
 	h = sp->obj->http;
 
+	if (sp->err_code < 100 || sp->err_code > 999)
+		sp->err_code = 501;
+
 	http_PutProtocol(w, sp->fd, h, "HTTP/1.1");
 	http_PutStatus(w, sp->fd, h, sp->err_code);
 	TIM_format(TIM_real(), date);
@@ -332,13 +364,16 @@ cnt_error(struct sess *sp)
 		    http_StatusMessage(sp->err_code));
 	VCL_error_method(sp);
 
-	if (sp->handling == VCL_RET_RESTART) {
+	if (sp->handling == VCL_RET_RESTART &&
+	    sp->restarts <  params->max_restarts) {
 		HSH_Drop(sp);
 		sp->director = NULL;
 		sp->restarts++;
 		sp->step = STP_RECV;
 		return (0);
-	}
+	} else if (sp->handling == VCL_RET_RESTART)
+		sp->handling = VCL_RET_DELIVER;
+
 
 	/* We always close when we take this path */
 	sp->doclose = "error";
@@ -347,6 +382,7 @@ cnt_error(struct sess *sp)
 	assert(sp->handling == VCL_RET_DELIVER);
 	sp->err_code = 0;
 	sp->err_reason = NULL;
+	sp->wrk->bereq = NULL;
 	sp->step = STP_DELIVER;
 	return (0);
 }
@@ -362,7 +398,7 @@ DOT		label="fetch from backend\n(find obj.ttl)"
 DOT	]
 DOT	vcl_fetch [
 DOT		shape=record
-DOT		label="vcl_fetch()|req.\nobj.\nbereq."
+DOT		label="vcl_fetch()|req.\nobj.\nbereq.\nberesp."
 DOT	]
 DOT	fetch -> vcl_fetch [style=bold,color=blue,weight=2]
 DOT	fetch_pass [
@@ -385,63 +421,235 @@ static int
 cnt_fetch(struct sess *sp)
 {
 	int i;
+	struct http *hp, *hp2;
+	char *b;
+	unsigned handling, l, nhttp;
+	int varyl = 0;
+	struct vsb *vary = NULL;
 
 	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
 	CHECK_OBJ_NOTNULL(sp->vcl, VCL_CONF_MAGIC);
 
-	AN(sp->bereq);
 	AN(sp->director);
 	AZ(sp->vbe);
-	i = Fetch(sp);
+
+	/* sp->wrk->http[0] is (still) bereq */
+	sp->wrk->beresp = sp->wrk->http[1];
+	http_Setup(sp->wrk->beresp, sp->wrk->ws);
+
+	i = FetchHdr(sp);
+
+	/*
+	 * Save a copy before it might get mangled in VCL.  When it comes to
+	 * dealing with the body, we want to see the unadultered headers.
+	 */
+	sp->wrk->beresp1 = sp->wrk->http[2];
+	*sp->wrk->beresp1 = *sp->wrk->beresp;
+
+	if (i) {
+		if (sp->objcore != NULL) {
+			CHECK_OBJ_NOTNULL(sp->objhead, OBJHEAD_MAGIC);
+			CHECK_OBJ_NOTNULL(sp->objcore, OBJCORE_MAGIC);
+			HSH_DerefObjCore(sp);
+			AZ(sp->objhead);
+			AZ(sp->objcore);
+		}
+		AZ(sp->obj);
+		sp->wrk->bereq = NULL;
+		sp->wrk->beresp = NULL;
+		sp->wrk->beresp1 = NULL;
+		sp->err_code = 503;
+		sp->step = STP_ERROR;
+		return (0);
+	}
+
+	sp->err_code = http_GetStatus(sp->wrk->beresp);
+
+	/*
+	 * Initial cacheability determination per [RFC2616, 13.4]
+	 * We do not support ranges yet, so 206 is out.
+	 */
+	switch (sp->err_code) {
+	case 200: /* OK */
+	case 203: /* Non-Authoritative Information */
+	case 300: /* Multiple Choices */
+	case 301: /* Moved Permanently */
+	case 302: /* Moved Temporarily */
+	case 410: /* Gone */
+	case 404: /* Not Found */
+		sp->wrk->cacheable = 1;
+		break;
+	default:
+		sp->wrk->cacheable = 0;
+		break;
+	}
+
+	sp->wrk->entered = TIM_real();
+	sp->wrk->age = 0;
+	sp->wrk->ttl = RFC2616_Ttl(sp);
+
+	if (sp->objcore == NULL)
+		sp->wrk->cacheable = 0;
+
+	sp->wrk->do_esi = 0;
+	sp->wrk->grace = NAN;
+
+	VCL_fetch_method(sp);
+
+	/*
+	 * When we fetch the body, we may hit the LRU cleanup and that
+	 * will overwrite sp->handling, so we have to save our plans
+	 * here.
+	 */
+	handling = sp->handling;
+
+	if (sp->objcore == NULL) {
+		/* This is a pass from vcl_recv */
+		AZ(sp->objhead);
+		sp->wrk->cacheable = 0;
+	} else if (!sp->wrk->cacheable) {
+		if (sp->objhead != NULL)
+			HSH_DerefObjCore(sp);
+	}
+
+	/*
+	 * At this point we are either committed to flesh out the busy
+	 * object we have in the hash or we have let go of it, if we ever
+	 * had one.
+	 */
+
+	if (sp->wrk->cacheable) {
+		CHECK_OBJ_NOTNULL(sp->objhead, OBJHEAD_MAGIC);
+		CHECK_OBJ_NOTNULL(sp->objcore, OBJCORE_MAGIC);
+		vary = VRY_Create(sp, sp->wrk->beresp);
+		if (vary != NULL) {
+			varyl = vsb_len(vary);
+			assert(varyl > 0);
+		}
+	} else {
+		AZ(sp->objhead);
+		AZ(sp->objcore);
+	}
+
+	l = http_EstimateWS(sp->wrk->beresp, HTTPH_A_INS, &nhttp);
+
+	if (vary != NULL)
+		l += varyl;
+
+	/* Space for producing a Content-Length: header */
+	l += 30;
+
+	/*
+	 * XXX: If we have a Length: header, we should allocate the body
+	 * XXX: also.
+	 */
+
+	sp->obj = STV_NewObject(sp, l, sp->wrk->ttl, nhttp);
+
+	if (sp->objhead != NULL) {
+		CHECK_OBJ_NOTNULL(sp->objhead, OBJHEAD_MAGIC);
+		CHECK_OBJ_NOTNULL(sp->objcore, OBJCORE_MAGIC);
+		sp->objcore->obj = sp->obj;
+		sp->obj->objcore = sp->objcore;
+		sp->objcore->objhead = sp->objhead;
+		sp->objhead = NULL;	/* refcnt follows pointer. */
+		sp->objcore = NULL;	/* refcnt follows pointer. */
+		BAN_NewObj(sp->obj);
+	}
+
+	if (vary != NULL) {
+		sp->obj->vary =
+		    (void *)WS_Alloc(sp->obj->http->ws, varyl);
+		AN(sp->obj->vary);
+		memcpy(sp->obj->vary, vsb_data(vary), varyl);
+		vsb_delete(vary);
+		vary = NULL;
+	}
+
+	sp->obj->xid = sp->xid;
+	sp->obj->response = sp->err_code;
+	sp->obj->cacheable = sp->wrk->cacheable;
+	sp->obj->ttl = sp->wrk->ttl;
+	sp->obj->grace = sp->wrk->grace;
+	if (sp->obj->ttl == 0. && sp->obj->grace == 0.)
+		sp->obj->cacheable = 0;
+	sp->obj->age = sp->wrk->age;
+	sp->obj->entered = sp->wrk->entered;
+	WS_Assert(sp->obj->ws_o);
+
+	/* Filter into object */
+	hp = sp->wrk->beresp;
+	hp2 = sp->obj->http;
+
+	hp2->logtag = HTTP_Obj;
+	http_CopyResp(hp2, hp);
+	http_FilterFields(sp->wrk, sp->fd, hp2, hp, HTTPH_A_INS);
+	http_CopyHome(sp->wrk, sp->fd, hp2);
+
+	if (http_GetHdr(hp, H_Last_Modified, &b))
+		sp->obj->last_modified = TIM_parse(b);
+
+	i = FetchBody(sp);
 	AZ(sp->wrk->wfd);
 	AZ(sp->vbe);
 	AN(sp->director);
 
 	if (i) {
-		sp->err_code = 503;
-		sp->step = STP_ERROR;
-		VBE_free_bereq(&sp->bereq);
 		HSH_Drop(sp);
 		AZ(sp->obj);
+		sp->wrk->bereq = NULL;
+		sp->wrk->beresp = NULL;
+		sp->wrk->beresp1 = NULL;
+		sp->err_code = 503;
+		sp->step = STP_ERROR;
 		return (0);
 	}
 
-	RFC2616_cache_policy(sp, sp->obj->http);	/* XXX -> VCL */
+	if (sp->wrk->cacheable)
+		HSH_Object(sp);
 
-	sp->err_code = http_GetStatus(sp->obj->http);
-	VCL_fetch_method(sp);
+	if (sp->wrk->do_esi)
+		ESI_Parse(sp);
 
-	VBE_free_bereq(&sp->bereq);
-
-	switch (sp->handling) {
+	switch (handling) {
 	case VCL_RET_RESTART:
 		HSH_Drop(sp);
 		sp->director = NULL;
 		sp->restarts++;
+		sp->wrk->bereq = NULL;
+		sp->wrk->beresp = NULL;
+		sp->wrk->beresp1 = NULL;
 		sp->step = STP_RECV;
 		return (0);
 	case VCL_RET_PASS:
-		sp->obj->pass = 1;
+		if (sp->obj->objcore != NULL)
+			sp->obj->objcore->flags |= OC_F_PASS;
 		if (sp->obj->ttl - sp->t_req < params->default_ttl)
 			sp->obj->ttl = sp->t_req + params->default_ttl;
 		break;
 	case VCL_RET_DELIVER:
 		break;
 	case VCL_RET_ERROR:
-		sp->step = STP_ERROR;
 		HSH_Drop(sp);
+		sp->wrk->bereq = NULL;
+		sp->wrk->beresp = NULL;
+		sp->wrk->beresp1 = NULL;
+		sp->step = STP_ERROR;
 		return (0);
 	default:
 		WRONG("Illegal action in vcl_fetch{}");
 	}
 
 	sp->obj->cacheable = 1;
-	if (sp->obj->objhead != NULL) {
-		VRY_Create(sp);
+	if (sp->wrk->cacheable) {
 		EXP_Insert(sp->obj);
+		AN(sp->obj->ban);
 		HSH_Unbusy(sp);
 	}
 	sp->acct_req.fetch++;
+	sp->wrk->bereq = NULL;
+	sp->wrk->beresp = NULL;
+	sp->wrk->beresp1 = NULL;
 	sp->step = STP_DELIVER;
 	return (0);
 }
@@ -452,7 +660,6 @@ cnt_fetch(struct sess *sp)
 static int
 cnt_first(struct sess *sp)
 {
-	int i;
 
 	/*
 	 * XXX: If we don't have acceptfilters we are somewhat subject
@@ -472,26 +679,8 @@ cnt_first(struct sess *sp)
 	HTC_Init(sp->htc, sp->ws, sp->fd);
 	sp->wrk->lastused = sp->t_open;
 	sp->acct_req.sess++;
-	SES_RefSrcAddr(sp);
-	do
-		i = HTC_Rx(sp->htc);
-	while (i == 0);
 
-	switch (i) {
-	case 1:
-		sp->step = STP_START;
-		break;
-	case -1:
-		vca_close_session(sp, "error");
-		sp->step = STP_DONE;
-		break;
-	case -2:
-		vca_close_session(sp, "blast");
-		sp->step = STP_DONE;
-		break;
-	default:
-		WRONG("Illegal return from HTC_Rx");
-	}
+	sp->step = STP_WAIT;
 	return (0);
 }
 
@@ -521,19 +710,22 @@ cnt_hit(struct sess *sp)
 	CHECK_OBJ_NOTNULL(sp->obj, OBJECT_MAGIC);
 	CHECK_OBJ_NOTNULL(sp->vcl, VCL_CONF_MAGIC);
 
-	assert(!sp->obj->pass);
+	assert(!(sp->obj->objcore->flags & OC_F_PASS));
 
 	VCL_hit_method(sp);
 
 	if (sp->handling == VCL_RET_DELIVER) {
 		/* Dispose of any body part of the request */
-		FetchReqBody(sp);
+		(void)FetchReqBody(sp);
+		sp->wrk->bereq = NULL;
 		sp->step = STP_DELIVER;
 		return (0);
 	}
 
 	/* Drop our object, we won't need it */
-	HSH_Deref(&sp->obj);
+	HSH_Deref(sp->wrk, &sp->obj);
+	sp->objcore = NULL;
+	AZ(sp->objhead);
 
 	switch(sp->handling) {
 	case VCL_RET_PASS:
@@ -583,46 +775,55 @@ DOT lookup -> miss [label="no",style=bold,color=blue,weight=2]
 static int
 cnt_lookup(struct sess *sp)
 {
+	struct objcore *oc;
 	struct object *o;
+	struct objhead *oh;
 
 	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
 	CHECK_OBJ_NOTNULL(sp->vcl, VCL_CONF_MAGIC);
 
-	if (sp->obj == NULL) {
-		HSH_Prepare(sp, sp->vcl->nhashcount);
-		VCL_hash_method(sp);
-		assert(sp->handling == VCL_RET_HASH);
-	}
 
-	o = HSH_Lookup(sp);
+	oc = HSH_Lookup(sp, &oh);
 
-	if (o == NULL) {
+	if (oc == NULL) {
 		/*
 		 * We lost the session to a busy object, disembark the
 		 * worker thread.   The hash code to restart the session,
 		 * still in STP_LOOKUP, later when the busy object isn't.
+		 * NB:  Do not access sp any more !
 		 */
 		return (1);
 	}
 
-	sp->obj = o;
+	CHECK_OBJ_NOTNULL(oc, OBJCORE_MAGIC);
+	CHECK_OBJ_NOTNULL(oh, OBJHEAD_MAGIC);
 
 	/* If we inserted a new object it's a miss */
-	if (sp->obj->busy) {
-		VSL_stats->cache_miss++;
+	if (oc->flags & OC_F_BUSY) {
+		sp->wrk->stats.cache_miss++;
+
+		AZ(oc->obj);
+		sp->objhead = oh;
+		sp->objcore = oc;
 		sp->step = STP_MISS;
 		return (0);
 	}
 
-	if (sp->obj->pass) {
-		VSL_stats->cache_hitpass++;
+	o = oc->obj;
+	CHECK_OBJ_NOTNULL(o, OBJECT_MAGIC);
+	sp->obj = o;
+
+	if (oc->flags & OC_F_PASS) {
+		sp->wrk->stats.cache_hitpass++;
 		WSP(sp, SLT_HitPass, "%u", sp->obj->xid);
-		HSH_Deref(&sp->obj);
+		HSH_Deref(sp->wrk, &sp->obj);
+		sp->objcore = NULL;
+		sp->objhead = NULL;
 		sp->step = STP_PASS;
 		return (0);
 	}
 
-	VSL_stats->cache_hit++;
+	sp->wrk->stats.cache_hit++;
 	WSP(sp, SLT_Hit, "%u", sp->obj->xid);
 	sp->step = STP_HIT;
 	return (0);
@@ -656,28 +857,33 @@ cnt_miss(struct sess *sp)
 {
 
 	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
-	CHECK_OBJ_NOTNULL(sp->obj, OBJECT_MAGIC);
 	CHECK_OBJ_NOTNULL(sp->vcl, VCL_CONF_MAGIC);
 
+	AZ(sp->obj);
+	AN(sp->objcore);
+	AN(sp->objhead);
+	WS_Reset(sp->wrk->ws, NULL);
+	sp->wrk->bereq = sp->wrk->http[0];
+	http_Setup(sp->wrk->bereq, sp->wrk->ws);
 	http_FilterHeader(sp, HTTPH_R_FETCH);
+	sp->wrk->connect_timeout = 0;
+	sp->wrk->first_byte_timeout = 0;
+	sp->wrk->between_bytes_timeout = 0;
 	VCL_miss_method(sp);
-	AZ(sp->obj->cacheable);
 	switch(sp->handling) {
 	case VCL_RET_ERROR:
-		HSH_Drop(sp);
-		VBE_free_bereq(&sp->bereq);
+		HSH_DerefObjCore(sp);
 		sp->step = STP_ERROR;
 		return (0);
 	case VCL_RET_PASS:
-		HSH_Drop(sp);
-		VBE_free_bereq(&sp->bereq);
+		HSH_DerefObjCore(sp);
 		sp->step = STP_PASS;
 		return (0);
 	case VCL_RET_FETCH:
 		sp->step = STP_FETCH;
 		return (0);
 	case VCL_RET_RESTART:
-		VBE_free_bereq(&sp->bereq);
+		HSH_DerefObjCore(sp);
 		INCOMPL();
 	default:
 		WRONG("Illegal action in vcl_miss{}");
@@ -724,20 +930,21 @@ cnt_pass(struct sess *sp)
 	CHECK_OBJ_NOTNULL(sp->vcl, VCL_CONF_MAGIC);
 	AZ(sp->obj);
 
+	WS_Reset(sp->wrk->ws, NULL);
+	sp->wrk->bereq = sp->wrk->http[0];
+	http_Setup(sp->wrk->bereq, sp->wrk->ws);
 	http_FilterHeader(sp, HTTPH_R_PASS);
 
+	sp->wrk->connect_timeout = 0;
+	sp->wrk->first_byte_timeout = 0;
+	sp->wrk->between_bytes_timeout = 0;
 	VCL_pass_method(sp);
 	if (sp->handling == VCL_RET_ERROR) {
-		VBE_free_bereq(&sp->bereq);
 		sp->step = STP_ERROR;
 		return (0);
 	}
 	assert(sp->handling == VCL_RET_PASS);
 	sp->acct_req.pass++;
-	HSH_Prealloc(sp);
-	sp->obj = sp->wrk->nobj;
-	sp->wrk->nobj = NULL;
-	sp->obj->busy = 1;
 	sp->sendbody = 1;
 	sp->step = STP_FETCH;
 	return (0);
@@ -776,6 +983,9 @@ cnt_pipe(struct sess *sp)
 	CHECK_OBJ_NOTNULL(sp->vcl, VCL_CONF_MAGIC);
 
 	sp->acct_req.pipe++;
+	WS_Reset(sp->wrk->ws, NULL);
+	sp->wrk->bereq = sp->wrk->http[0];
+	http_Setup(sp->wrk->bereq, sp->wrk->ws);
 	http_FilterHeader(sp, HTTPH_R_PIPE);
 
 	VCL_pipe_method(sp);
@@ -785,8 +995,8 @@ cnt_pipe(struct sess *sp)
 	assert(sp->handling == VCL_RET_PIPE);
 
 	PipeSession(sp);
-	AZ(sp->bereq);
 	AZ(sp->wrk->wfd);
+	sp->wrk->bereq = NULL;
 	sp->step = STP_DONE;
 	return (0);
 }
@@ -812,19 +1022,22 @@ DOT recv -> hash [label="lookup",style=bold,color=green,weight=4]
 static int
 cnt_recv(struct sess *sp)
 {
+	unsigned recv_handling;
 
 	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
 	CHECK_OBJ_NOTNULL(sp->vcl, VCL_CONF_MAGIC);
 	AZ(sp->obj);
 
-	SES_ResetBackendTimeouts(sp);
-
 	/* By default we use the first backend */
 	AZ(sp->director);
 	sp->director = sp->vcl->director[0];
 	AN(sp->director);
 
+	sp->disable_esi = 0;
+
 	VCL_recv_method(sp);
+	recv_handling = sp->handling;
+
 	if (sp->restarts >= params->max_restarts) {
 		if (sp->err_code == 0)
 			sp->err_code = 503;
@@ -832,9 +1045,19 @@ cnt_recv(struct sess *sp)
 		return (0);
 	}
 
-	sp->wantbody = (strcmp(sp->http->hd[HTTP_HDR_REQ].b, "HEAD") != 0);
+	SHA256_Init(sp->wrk->sha256ctx);
+	VCL_hash_method(sp);
+	assert(sp->handling == VCL_RET_HASH);
+	SHA256_Final(sp->digest, sp->wrk->sha256ctx);
+
+	if (!strcmp(sp->http->hd[HTTP_HDR_REQ].b, "HEAD")) {
+		sp->wantbody = 0;
+		http_ForceGet(sp->http);
+	} else
+		sp->wantbody = 1;
+
 	sp->sendbody = 0;
-	switch(sp->handling) {
+	switch(recv_handling) {
 	case VCL_RET_LOOKUP:
 		/* XXX: discard req body, if any */
 		sp->step = STP_LOOKUP;
@@ -843,7 +1066,7 @@ cnt_recv(struct sess *sp)
 		if (sp->esis > 0) {
 			/* XXX: VSL something */
 			INCOMPL();
-			sp->step = STP_DONE;
+			/* sp->step = STP_DONE; */
 			return (1);
 		}
 		sp->step = STP_PIPE;
@@ -881,7 +1104,7 @@ cnt_start(struct sess *sp)
 	AZ(sp->vcl);
 
 	/* Update stats of various sorts */
-	VSL_stats->client_req++;			/* XXX not locked */
+	sp->wrk->stats.client_req++;
 	sp->t_req = TIM_real();
 	sp->wrk->lastused = sp->t_req;
 	sp->acct_req.req++;
@@ -898,6 +1121,13 @@ cnt_start(struct sess *sp)
 	http_Setup(sp->http, sp->ws);
 	done = http_DissectRequest(sp);
 
+	/* If we could not even parse the request, just close */
+	if (done < 0) {
+		sp->step = STP_DONE;
+		vca_close_session(sp, "junk");
+		return (0);
+	}
+
 	/* Catch request snapshot */
 	sp->ws_req = WS_Snapshot(sp->ws);
 
@@ -926,6 +1156,10 @@ cnt_start(struct sess *sp)
 
 		/* XXX: Don't bother with write failures for now */
 		(void)write(sp->fd, r, strlen(r));
+		/* XXX: When we do ESI includes, this is not removed
+		 * XXX: because we use http0 as our basis.  Believed
+		 * XXX: safe, but potentially confusing.
+		 */
 		http_Unset(sp->http, H_Expect);
 	}
 
@@ -982,8 +1216,11 @@ CNT_Session(struct sess *sp)
 	 * do the syscall in the worker thread.
 	 */
 	if (sp->step == STP_FIRST || sp->step == STP_START)
-		TCP_blocking(sp->fd);
+		(void)TCP_blocking(sp->fd);
 
+	/*
+	 * NB: Once done is set, we can no longer touch sp!
+	 */
 	for (done = 0; !done; ) {
 		assert(sp->wrk == w);
 		/*
@@ -993,8 +1230,8 @@ CNT_Session(struct sess *sp)
 		CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
 		CHECK_OBJ_ORNULL(sp->obj, OBJECT_MAGIC);
 		CHECK_OBJ_NOTNULL(sp->wrk, WORKER_MAGIC);
-		CHECK_OBJ_ORNULL(w->nobj, OBJECT_MAGIC);
 		CHECK_OBJ_ORNULL(w->nobjhead, OBJHEAD_MAGIC);
+		WS_Assert(w->ws);
 
 		switch (sp->step) {
 #define STEP(l,u) \
@@ -1008,7 +1245,7 @@ CNT_Session(struct sess *sp)
 		default:
 			WRONG("State engine misfire");
 		}
-		CHECK_OBJ_ORNULL(w->nobj, OBJECT_MAGIC);
+		WS_Assert(w->ws);
 		CHECK_OBJ_ORNULL(w->nobjhead, OBJHEAD_MAGIC);
 	}
 	WSL_Flush(w, 0);
@@ -1050,9 +1287,9 @@ cli_debug_srandom(struct cli *cli, const char * const *av, void *priv)
 
 static struct cli_proto debug_cmds[] = {
 	{ "debug.xid", "debug.xid",
-		"\tExamine or set XID\n", 0, 1, cli_debug_xid },
+		"\tExamine or set XID\n", 0, 1, "d", cli_debug_xid },
 	{ "debug.srandom", "debug.srandom",
-		"\tSeed the random(3) function\n", 0, 1, cli_debug_srandom },
+		"\tSeed the random(3) function\n", 0, 1, "d", cli_debug_srandom },
 	{ NULL }
 };
 
@@ -1066,7 +1303,7 @@ CNT_Init(void)
 
 	srandomdev();
 	xids = random();
-	CLI_AddFuncs(DEBUG_CLI, debug_cmds);
+	CLI_AddFuncs(debug_cmds);
 }
 
 
