@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006-2008 Linpro AS
+ * Copyright (c) 2006-2010 Redpill Linpro AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -26,20 +26,20 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: cache_http.c 3732 2009-02-11 07:13:12Z tfheen $
- *
  * HTTP request storage and manipulation
  */
 
 #include "config.h"
 
+#include "svnid.h"
+SVNID("$Id: cache_http.c 4603 2010-03-03 09:12:10Z phk $")
+
 #include <errno.h>
 #include <unistd.h>
 #include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <strings.h>
 
 #include "shmlog.h"
 #include "vct.h"
@@ -117,14 +117,49 @@ http_StatusMessage(unsigned status)
 
 /*--------------------------------------------------------------------*/
 
+unsigned
+HTTP_estimate(unsigned nhttp)
+{
+
+	/* XXX: We trust the structs to size-aligned as necessary */
+	return (sizeof (struct http) + (sizeof (txt) + 1) * nhttp);
+}
+
+struct http *
+HTTP_create(void *p, unsigned nhttp)
+{
+	struct http *hp;
+
+	hp = p;
+	hp->magic = HTTP_MAGIC;
+	hp->hd = (void*)(hp + 1);
+	hp->shd = nhttp;
+	hp->hdf = (void*)(hp->hd + nhttp);
+	return (hp);
+}
+
+/*--------------------------------------------------------------------*/
+
 void
 http_Setup(struct http *hp, struct ws *ws)
 {
+	unsigned shd;
+	txt *hd;
+	unsigned char *hdf;
 
+	/* XXX: This is not elegant, is it efficient ? */
+	shd = hp->shd;
+	hd = hp->hd;
+	hdf = hp->hdf;
 	memset(hp, 0, sizeof *hp);
+	memset(hd, 0, sizeof *hd * shd);
+	memset(hdf, 0, sizeof *hdf * shd);
 	hp->magic = HTTP_MAGIC;
 	hp->ws = ws;
 	hp->nhd = HTTP_HDR_FIRST;
+	hp->shd = shd;
+	hp->hd = hd;
+	hp->hdf = hdf;
 }
 
 /*--------------------------------------------------------------------*/
@@ -152,6 +187,11 @@ http_findhdr(const struct http *hp, unsigned l, const char *hdr)
 	unsigned u;
 
 	for (u = HTTP_HDR_FIRST; u < hp->nhd; u++) {
+		/* XXX We have to check for empty header entries
+		   because a header could have been lost in
+		   http_copyHome */
+		if (hp->hd[u].b == NULL)
+			continue;
 		Tcheck(hp->hd[u]);
 		if (hp->hd[u].e < hp->hd[u].b + l + 1)
 			continue;
@@ -244,7 +284,7 @@ http_GetHdrField(const struct http *hp, const char *hdr,
  */
 
 const char *
-http_DoConnection(struct http *hp)
+http_DoConnection(const struct http *hp)
 {
 	char *p, *q;
 	const char *ret;
@@ -327,12 +367,29 @@ http_dissect_hdrs(struct worker *w, struct http *hp, int fd, char *p, txt t)
 	hp->conds = 0;
 	r = NULL;		/* For FlexeLint */
 	for (; p < t.e; p = r) {
-		/* XXX: handle continuation lines */
-		q = strchr(p, '\n');
-		assert(q != NULL);
-		r = q + 1;
-		if (q > p && q[-1] == '\r')
-			q--;
+
+		/* Find end of next header */
+		q = r = p;
+		while (r < t.e) {
+			if (!vct_iscrlf(*r)) {
+				r++;
+				continue;
+			}
+			q = r;
+			assert(r < t.e);
+			r += vct_skipcrlf(r);
+			if (r >= t.e)
+				break;
+			/* If line does not continue: got it. */
+			if (!vct_issp(*r))
+				break;
+
+			/* Clear line continuation LWS to spaces */
+			while (vct_islws(*q))
+				*q++ = ' ';
+		}
+
+		/* Empty header = end of headers */
 		if (p == q)
 			break;
 
@@ -345,7 +402,7 @@ http_dissect_hdrs(struct worker *w, struct http *hp, int fd, char *p, txt t)
 			q--;
 		*q = '\0';
 
-		if (hp->nhd < HTTP_HDR_MAX) {
+		if (hp->nhd < hp->shd) {
 			hp->hdf[hp->nhd] = 0;
 			hp->hd[hp->nhd].b = p;
 			hp->hd[hp->nhd].e = q;
@@ -368,7 +425,7 @@ static int
 http_splitline(struct worker *w, int fd, struct http *hp,
     const struct http_conn *htc, int h1, int h2, int h3)
 {
-	char *p;
+	char *p, *q;
 
 	CHECK_OBJ_NOTNULL(htc, HTTP_CONN_MAGIC);
 	CHECK_OBJ_NOTNULL(hp, HTTP_MAGIC);
@@ -381,40 +438,47 @@ http_splitline(struct worker *w, int fd, struct http *hp,
 		continue;
 
 	/* First field cannot contain SP, CRLF or CTL */
-	hp->hd[h1].b = p;
-	for (; !vct_issp(*p); p++)
+	q = p;
+	for (; !vct_issp(*p); p++) {
 		if (vct_isctl(*p))
-			return (400);
+			return (-1);
+	}
+	hp->hd[h1].b = q;
 	hp->hd[h1].e = p;
 
 	/* Skip SP */
-	for (; vct_issp(*p); p++)
-		;
+	for (; vct_issp(*p); p++) {
+		if (vct_isctl(*p))
+			return (-1);
+	}
 
-	/* Second field cannot contain LWS */
-	hp->hd[h2].b = p;
-	for (; !vct_islws(*p); p++)
-		;
+	/* Second field cannot contain LWS or CTL */
+	q = p;
+	for (; !vct_islws(*p); p++) {
+		if (vct_isctl(*p))
+			return (-1);
+	}
+	hp->hd[h2].b = q;
 	hp->hd[h2].e = p;
 
 	if (!Tlen(hp->hd[h2]))
 		return (400);
 
 	/* Skip SP */
-	for (; vct_issp(*p); p++)
-		;
+	for (; vct_issp(*p); p++) {
+		if (vct_isctl(*p))
+			return (-1);
+	}
 
 	/* Third field is optional and cannot contain CTL */
+	q = p;
 	if (!vct_iscrlf(*p)) {
-		hp->hd[h3].b = p;
 		for (; !vct_iscrlf(*p); p++)
 			if (vct_isctl(*p))
-				return (400);
-		hp->hd[h3].e = p;
-	} else {
-		hp->hd[h3].b = p;
-		hp->hd[h3].e = p;
+				return (-1);
 	}
+	hp->hd[h3].b = q;
+	hp->hd[h3].e = p;
 
 	/* Skip CRLF */
 	p += vct_skipcrlf(p);
@@ -439,9 +503,9 @@ static void
 http_ProtoVer(struct http *hp)
 {
 
-	if (!strcmp(hp->hd[HTTP_HDR_PROTO].b, "HTTP/1.0"))
+	if (!strcasecmp(hp->hd[HTTP_HDR_PROTO].b, "HTTP/1.0"))
 		hp->protover = 1.0;
-	else if (!strcmp(hp->hd[HTTP_HDR_PROTO].b, "HTTP/1.1"))
+	else if (!strcasecmp(hp->hd[HTTP_HDR_PROTO].b, "HTTP/1.1"))
 		hp->protover = 1.1;
 	else
 		hp->protover = 0.9;
@@ -481,25 +545,35 @@ int
 http_DissectResponse(struct worker *w, const struct http_conn *htc,
     struct http *hp)
 {
-	int i;
+	int i = 0;
 
 	CHECK_OBJ_NOTNULL(htc, HTTP_CONN_MAGIC);
 	CHECK_OBJ_NOTNULL(hp, HTTP_MAGIC);
 	hp->logtag = HTTP_Rx;
 
-	i = http_splitline(w, htc->fd, hp, htc,
-	    HTTP_HDR_PROTO, HTTP_HDR_STATUS, HTTP_HDR_RESPONSE);
+	if (http_splitline(w, htc->fd, hp, htc,
+	    HTTP_HDR_PROTO, HTTP_HDR_STATUS, HTTP_HDR_RESPONSE))
+		i = 503;
 
-	if (i != 0 || memcmp(hp->hd[HTTP_HDR_PROTO].b, "HTTP/1.", 7))
-		WSLR(w, SLT_HttpGarbage, htc->fd, htc->rxbuf);
-	if (i != 0) {
-		if (hp->status == 0)
-			hp->status = i;
-	} else {
-		hp->status =
-		    strtoul(hp->hd[HTTP_HDR_STATUS].b, NULL /* XXX */, 10);
+	if (i == 0 && memcmp(hp->hd[HTTP_HDR_PROTO].b, "HTTP/1.", 7))
+		i = 503;
+
+	if (i == 0 && Tlen(hp->hd[HTTP_HDR_STATUS]) != 3)
+		i = 503;
+
+	if (i == 0) {
+		hp->status = strtoul(hp->hd[HTTP_HDR_STATUS].b, NULL, 10);
+		if (hp->status < 100 || hp->status > 999)
+			i = 503;
 	}
-	http_ProtoVer(hp);
+
+	if (i != 0) {
+		WSLR(w, SLT_HttpGarbage, htc->fd, htc->rxbuf);
+		hp->status = i;
+	} else {
+		http_ProtoVer(hp);
+	}
+
 	if (hp->hd[HTTP_HDR_RESPONSE].b == NULL ||
 	    !Tlen(hp->hd[HTTP_HDR_RESPONSE])) {
 		/* Backend didn't send a response string, use the standard */
@@ -514,10 +588,10 @@ http_DissectResponse(struct worker *w, const struct http_conn *htc,
 /*--------------------------------------------------------------------*/
 
 void
-http_SetH(struct http *to, unsigned n, const char *fm)
+http_SetH(const struct http *to, unsigned n, const char *fm)
 {
 
-	assert(n < HTTP_HDR_MAX);
+	assert(n < to->shd);
 	AN(fm);
 	to->hd[n].b = TRUST_ME(fm);
 	to->hd[n].e = strchr(to->hd[n].b, '\0');
@@ -525,55 +599,35 @@ http_SetH(struct http *to, unsigned n, const char *fm)
 }
 
 static void
-http_copyh(struct http *to, const struct http *fm, unsigned n)
+http_copyh(const struct http *to, const struct http *fm, unsigned n)
 {
 
-	assert(n < HTTP_HDR_MAX);
+	assert(n < to->shd);
 	Tcheck(fm->hd[n]);
 	to->hd[n] = fm->hd[n];
 	to->hdf[n] = fm->hdf[n];
 }
 
-static void
-http_copyreq(struct http *to, const struct http *fm, int how)
-{
-
-	CHECK_OBJ_NOTNULL(fm, HTTP_MAGIC);
-	CHECK_OBJ_NOTNULL(to, HTTP_MAGIC);
-
-	if ((how == HTTPH_R_PIPE) || (how == HTTPH_R_PASS)) {
-		http_copyh(to, fm, HTTP_HDR_REQ);
-		http_copyh(to, fm, HTTP_HDR_PROTO);
-	} else {
-		http_SetH(to, HTTP_HDR_REQ, "GET");
-		http_SetH(to, HTTP_HDR_PROTO, "HTTP/1.1");
-	}
-	http_copyh(to, fm, HTTP_HDR_URL);
-}
-
 void
-http_ForceGet(struct http *to)
+http_ForceGet(const struct http *to)
 {
 	if (strcmp(http_GetReq(to), "GET"))
 		http_SetH(to, HTTP_HDR_REQ, "GET");
 }
 
 void
-http_CopyResp(struct http *to, const struct http *fm)
+http_CopyResp(const struct http *to, const struct http *fm)
 {
 
 	CHECK_OBJ_NOTNULL(fm, HTTP_MAGIC);
 	CHECK_OBJ_NOTNULL(to, HTTP_MAGIC);
-	if (params->client_http11)
-		http_SetH(to, HTTP_HDR_PROTO, "HTTP/1.1");
-	else
-		http_copyh(to, fm, HTTP_HDR_PROTO);
+	http_SetH(to, HTTP_HDR_PROTO, "HTTP/1.1");
 	http_copyh(to, fm, HTTP_HDR_STATUS);
 	http_copyh(to, fm, HTTP_HDR_RESPONSE);
 }
 
 void
-http_SetResp(struct http *to, const char *proto, const char *status,
+http_SetResp(const struct http *to, const char *proto, const char *status,
     const char *response)
 {
 
@@ -590,9 +644,9 @@ http_copyheader(struct worker *w, int fd, struct http *to,
 
 	CHECK_OBJ_NOTNULL(fm, HTTP_MAGIC);
 	CHECK_OBJ_NOTNULL(to, HTTP_MAGIC);
-	assert(n < HTTP_HDR_MAX);
+	assert(n < fm->shd);
 	Tcheck(fm->hd[n]);
-	if (to->nhd < HTTP_HDR_MAX) {
+	if (to->nhd < to->shd) {
 		to->hd[to->nhd] = fm->hd[n];
 		to->hdf[to->nhd] = 0;
 		to->nhd++;
@@ -602,6 +656,36 @@ http_copyheader(struct worker *w, int fd, struct http *to,
 	}
 }
 
+/*--------------------------------------------------------------------
+ * Estimate how much workspace we need to Filter this header according
+ * to 'how'.
+ */
+
+unsigned
+http_EstimateWS(const struct http *fm, unsigned how, unsigned *nhd)
+{
+	unsigned u, l;
+
+	l = 0;
+	*nhd = HTTP_HDR_FIRST;
+	CHECK_OBJ_NOTNULL(fm, HTTP_MAGIC);
+	for (u = 0; u < fm->nhd; u++) {
+		if (fm->hd[u].b == NULL)
+			continue;
+		if (fm->hdf[u] & HDF_FILTER)
+			continue;
+#define HTTPH(a, b, c, d, e, f, g) \
+		if (((e) & how) && http_IsHdr(&fm->hd[u], (b))) \
+			continue;
+#include "http_headers.h"
+#undef HTTPH
+		l += Tlen(fm->hd[u]) + 1;
+		(*nhd)++;
+		// fm->hdf[u] |= HDF_COPY;
+	}
+	return (l);
+}
+
 /*--------------------------------------------------------------------*/
 
 void
@@ -615,6 +699,8 @@ http_FilterFields(struct worker *w, int fd, struct http *to,
 	to->nhd = HTTP_HDR_FIRST;
 	to->status = fm->status;
 	for (u = HTTP_HDR_FIRST; u < fm->nhd; u++) {
+		if (fm->hd[u].b == NULL)
+			continue;
 		if (fm->hdf[u] & HDF_FILTER)
 			continue;
 #define HTTPH(a, b, c, d, e, f, g) \
@@ -629,24 +715,22 @@ http_FilterFields(struct worker *w, int fd, struct http *to,
 /*--------------------------------------------------------------------*/
 
 void
-http_FilterHeader(struct sess *sp, unsigned how)
+http_FilterHeader(const struct sess *sp, unsigned how)
 {
-	struct bereq *bereq;
 	struct http *hp;
 
-	AZ(sp->bereq);
-	bereq = VBE_new_bereq();
-	AN(bereq);
-	hp = bereq->http;
+	hp = sp->wrk->bereq;
+	CHECK_OBJ_NOTNULL(hp, HTTP_MAGIC);
 	hp->logtag = HTTP_Tx;
 
-	http_copyreq(hp, sp->http, how);
+	http_copyh(hp, sp->http, HTTP_HDR_REQ);
+	http_copyh(hp, sp->http, HTTP_HDR_URL);
+	if (how == HTTPH_R_FETCH)
+		http_SetH(hp, HTTP_HDR_PROTO, "HTTP/1.1");
+	else
+		http_copyh(hp, sp->http, HTTP_HDR_PROTO);
 	http_FilterFields(sp->wrk, sp->fd, hp, sp->http, how);
 	http_PrintfHeader(sp->wrk, sp->fd, hp, "X-Varnish: %u", sp->xid);
-	http_PrintfHeader(sp->wrk, sp->fd, hp,
-	    "X-Forwarded-For: %s", sp->addr);
-
-	sp->bereq = bereq;
 }
 
 /*--------------------------------------------------------------------
@@ -655,7 +739,7 @@ http_FilterHeader(struct sess *sp, unsigned how)
  */
 
 void
-http_CopyHome(struct worker *w, int fd, struct http *hp)
+http_CopyHome(struct worker *w, int fd, const struct http *hp)
 {
 	unsigned u, l;
 	char *p;
@@ -671,10 +755,12 @@ http_CopyHome(struct worker *w, int fd, struct http *hp)
 		p = WS_Alloc(hp->ws, l + 1);
 		if (p != NULL) {
 			WSLH(w, fd, hp, u);
-			memcpy(p, hp->hd[u].b, l + 1);
+			memcpy(p, hp->hd[u].b, l + 1L);
 			hp->hd[u].b = p;
 			hp->hd[u].e = p + l;
 		} else {
+			/* XXX This leaves a slot empty */
+			VSL_stats->losthdr++;
 			WSLR(w, SLT_LostHeader, fd, hp->hd[u]);
 			hp->hd[u].b = NULL;
 			hp->hd[u].e = NULL;
@@ -701,7 +787,7 @@ http_SetHeader(struct worker *w, int fd, struct http *to, const char *hdr)
 {
 
 	CHECK_OBJ_NOTNULL(to, HTTP_MAGIC);
-	if (to->nhd >= HTTP_HDR_MAX) {
+	if (to->nhd >= to->shd) {
 		VSL_stats->losthdr++;
 		WSL(w, SLT_LostHeader, fd, "%s", hdr);
 		return;
@@ -712,7 +798,7 @@ http_SetHeader(struct worker *w, int fd, struct http *to, const char *hdr)
 /*--------------------------------------------------------------------*/
 
 static void
-http_PutField(struct worker *w, int fd, struct http *to, int field,
+http_PutField(struct worker *w, int fd, const struct http *to, int field,
     const char *string)
 {
 	char *p;
@@ -727,7 +813,7 @@ http_PutField(struct worker *w, int fd, struct http *to, int field,
 		to->hd[field].e = NULL;
 		to->hdf[field] = 0;
 	} else {
-		memcpy(p, string, l + 1);
+		memcpy(p, string, l + 1L);
 		to->hd[field].b = p;
 		to->hd[field].e = p + l;
 		to->hdf[field] = 0;
@@ -735,7 +821,7 @@ http_PutField(struct worker *w, int fd, struct http *to, int field,
 }
 
 void
-http_PutProtocol(struct worker *w, int fd, struct http *to,
+http_PutProtocol(struct worker *w, int fd, const struct http *to,
     const char *protocol)
 {
 
@@ -754,7 +840,7 @@ http_PutStatus(struct worker *w, int fd, struct http *to, int status)
 }
 
 void
-http_PutResponse(struct worker *w, int fd, struct http *to,
+http_PutResponse(struct worker *w, int fd, const struct http *to,
     const char *response)
 {
 
@@ -773,7 +859,7 @@ http_PrintfHeader(struct worker *w, int fd, struct http *to,
 	va_start(ap, fmt);
 	n = vsnprintf(to->ws->f, l, fmt, ap);
 	va_end(ap);
-	if (n + 1 >= l || to->nhd >= HTTP_HDR_MAX) {
+	if (n + 1 >= l || to->nhd >= to->shd) {
 		VSL_stats->losthdr++;
 		WSL(w, SLT_LostHeader, fd, "%s", to->ws->f);
 		WS_Release(to->ws, 0);
@@ -793,6 +879,8 @@ http_Unset(struct http *hp, const char *hdr)
 	unsigned u, v;
 
 	for (v = u = HTTP_HDR_FIRST; u < hp->nhd; u++) {
+		if (hp->hd[u].b == NULL)
+			continue;
 		if (http_IsHdr(&hp->hd[u], hdr))
 			continue;
 		if (v != u) {
@@ -829,6 +917,8 @@ http_Write(struct worker *w, const struct http *hp, int resp)
 		WSLH(w, *w->wfd, hp, HTTP_HDR_PROTO);
 	}
 	for (u = HTTP_HDR_FIRST; u < hp->nhd; u++) {
+		if (hp->hd[u].b == NULL)
+			continue;
 		AN(hp->hd[u].b);
 		AN(hp->hd[u].e);
 		l += WRW_WriteH(w, &hp->hd[u], "\r\n");
