@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006-2008 Linpro AS
+ * Copyright (c) 2006-2009 Linpro AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -26,13 +26,14 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: cache_vrt.c 4015 2009-04-01 12:21:48Z tfheen $
- *
  * Runtime support for compiled VCL programs
  */
 
 #include "config.h"
 
+#include "svnid.h"
+SVNID("$Id: cache_vrt.c 4538 2010-02-08 18:09:49Z phk $")
+
 #include <sys/types.h>
 #include <sys/socket.h>
 
@@ -53,8 +54,11 @@
 #include "hash_slinger.h"
 #include "cache_backend.h"
 
-void *vrt_magic_string_end = &vrt_magic_string_end;
-char vrt_hostname[255] = "";
+/*XXX: sort of a hack, improve the Tcl code in the compiler to avoid */
+/*lint -save -esym(818,sp) */
+
+const void * const vrt_magic_string_end = &vrt_magic_string_end;
+static char vrt_hostname[255] = "";
 
 /*--------------------------------------------------------------------*/
 
@@ -63,7 +67,7 @@ VRT_error(struct sess *sp, unsigned code, const char *reason)
 {
 
 	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
-	WSL(sp->wrk, SLT_Debug, 0, "VCL_error(%u, %s)", code, reason ? 
+	WSL(sp->wrk, SLT_Debug, 0, "VCL_error(%u, %s)", code, reason ?
 	    reason : "(null)");
 	sp->err_code = code ? code : 503;
 	sp->err_reason = reason ? reason : http_StatusMessage(sp->err_code);
@@ -102,10 +106,13 @@ vrt_selecthttp(const struct sess *sp, enum gethdr_e where)
 		hp = sp->http;
 		break;
 	case HDR_BEREQ:
-		hp = sp->bereq->http;
+		hp = sp->wrk->bereq;
+		break;
+	case HDR_BERESP:
+		hp = sp->wrk->beresp;
 		break;
 	case HDR_RESP:
-		hp = sp->http;
+		hp = sp->wrk->resp;
 		break;
 	case HDR_OBJ:
 		hp = sp->obj->http;
@@ -130,7 +137,9 @@ VRT_GetHdr(const struct sess *sp, enum gethdr_e where, const char *n)
 	return (p);
 }
 
-/*--------------------------------------------------------------------*/
+/*--------------------------------------------------------------------
+ * XXX: Optimize the single element case ?
+ */
 
 /*lint -e{818} ap,hp could be const */
 static char *
@@ -147,10 +156,11 @@ vrt_assemble_string(struct http *hp, const char *h, const char *p, va_list ap)
 		if (b + x < e)
 			memcpy(b, h, x);
 		b += x;
-		if (b + 1 < e)
-			*b++ = ' ';
+		if (b < e)
+			*b = ' ';
+		b++;
 	}
-	while (p != vrt_magic_string_end) {
+	while (p != vrt_magic_string_end && b < e) {
 		if (p == NULL)
 			p = "(null)";
 		x = strlen(p);
@@ -159,8 +169,9 @@ vrt_assemble_string(struct http *hp, const char *h, const char *p, va_list ap)
 		b += x;
 		p = va_arg(ap, const char *);
 	}
-	if (b + 1 < e)
-		*b++ = '\0';
+	if (b < e)
+		*b = '\0';
+	b++;
 	if (b > e) {
 		WS_Release(hp->ws, 0);
 		return (NULL);
@@ -242,13 +253,15 @@ VRT_r_##obj##_##hdr(const struct sess *sp)			\
 VRT_DO_HDR(req,   request,	sp->http,		HTTP_HDR_REQ)
 VRT_DO_HDR(req,   url,		sp->http,		HTTP_HDR_URL)
 VRT_DO_HDR(req,   proto,	sp->http,		HTTP_HDR_PROTO)
-VRT_DO_HDR(bereq, request,	sp->bereq->http,	HTTP_HDR_REQ)
-VRT_DO_HDR(bereq, url,		sp->bereq->http,	HTTP_HDR_URL)
-VRT_DO_HDR(bereq, proto,	sp->bereq->http,	HTTP_HDR_PROTO)
+VRT_DO_HDR(bereq, request,	sp->wrk->bereq,		HTTP_HDR_REQ)
+VRT_DO_HDR(bereq, url,		sp->wrk->bereq,		HTTP_HDR_URL)
+VRT_DO_HDR(bereq, proto,	sp->wrk->bereq,		HTTP_HDR_PROTO)
 VRT_DO_HDR(obj,   proto,	sp->obj->http,		HTTP_HDR_PROTO)
 VRT_DO_HDR(obj,   response,	sp->obj->http,		HTTP_HDR_RESPONSE)
-VRT_DO_HDR(resp,  proto,	sp->http,		HTTP_HDR_PROTO)
-VRT_DO_HDR(resp,  response,	sp->http,		HTTP_HDR_RESPONSE)
+VRT_DO_HDR(resp,  proto,	sp->wrk->resp,		HTTP_HDR_PROTO)
+VRT_DO_HDR(resp,  response,	sp->wrk->resp,		HTTP_HDR_RESPONSE)
+VRT_DO_HDR(beresp,  proto,	sp->wrk->beresp,	HTTP_HDR_PROTO)
+VRT_DO_HDR(beresp,  response,	sp->wrk->beresp,	HTTP_HDR_RESPONSE)
 
 /*--------------------------------------------------------------------*/
 
@@ -268,6 +281,60 @@ VRT_l_obj_status(const struct sess *sp, int num)
 	http_SetH(sp->obj->http, HTTP_HDR_STATUS, p);
 }
 
+/* Add an objecthead to the saintmode list for the (hopefully) relevant
+ * backend. Some double-up asserting here to avoid assert-errors when there
+ * is no object.
+ */
+void
+VRT_l_beresp_saintmode(const struct sess *sp, double a)
+{
+	struct trouble *new;
+	struct trouble *tr;
+	struct trouble *tr2;
+
+	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
+	if (!sp->vbe)
+		return;
+	CHECK_OBJ_NOTNULL(sp->vbe, VBE_CONN_MAGIC);
+	if (!sp->vbe->backend)
+		return;
+	CHECK_OBJ_NOTNULL(sp->vbe->backend, BACKEND_MAGIC);
+	if (!sp->objhead)
+		return;
+	CHECK_OBJ_NOTNULL(sp->objhead, OBJHEAD_MAGIC);
+
+	/* Setting a negative holdoff period is a mistake. Detecting this
+	 * when compiling the VCL would be better.
+	 */
+	assert(a > 0);
+
+	ALLOC_OBJ(new, TROUBLE_MAGIC);
+	AN(new);
+	new->target = (uintptr_t)sp->objhead;
+	new->timeout = sp->t_req + a;
+
+	/* Insert the new item on the list before the first item with a
+	 * timeout at a later date (ie: sort by which entry will time out
+	 * from the list
+	 */
+	Lck_Lock(&sp->vbe->backend->mtx);
+	VTAILQ_FOREACH_SAFE(tr, &sp->vbe->backend->troublelist, list, tr2) {
+		if (tr->timeout < new->timeout) {
+			VTAILQ_INSERT_BEFORE(tr, new, list);
+			new = NULL;
+			break;
+		}
+	}
+
+	/* Insert the item at the end if the list is empty or all other
+	 * items have a longer timeout.
+	 */
+	if (new)
+		VTAILQ_INSERT_TAIL(&sp->vbe->backend->troublelist, new, list);
+
+	Lck_Unlock(&sp->vbe->backend->mtx);
+}
+
 int
 VRT_r_obj_status(const struct sess *sp)
 {
@@ -276,6 +343,7 @@ VRT_r_obj_status(const struct sess *sp)
 	/* XXX: use http_GetStatus() */
 	if (sp->obj->http->status)
 		return (sp->obj->http->status);
+	AN(sp->obj->http->hd[HTTP_HDR_STATUS].b);
 	return (atoi(sp->obj->http->hd[HTTP_HDR_STATUS].b));
 }
 
@@ -285,62 +353,144 @@ VRT_l_resp_status(const struct sess *sp, int num)
 	char *p;
 
 	assert(num >= 100 && num <= 999);
-	p = WS_Alloc(sp->http->ws, 4);
+	p = WS_Alloc(sp->wrk->ws, 4);
 	if (p == NULL)
 		WSP(sp, SLT_LostHeader, "%s", "resp.status");
 	else
 		sprintf(p, "%d", num);
-	http_SetH(sp->http, HTTP_HDR_STATUS, p);
+	http_SetH(sp->wrk->resp, HTTP_HDR_STATUS, p);
 }
 
 int
 VRT_r_resp_status(const struct sess *sp)
 {
 	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
-	CHECK_OBJ_NOTNULL(sp->http, HTTP_MAGIC);
-	return (atoi(sp->http->hd[HTTP_HDR_STATUS].b));
+	CHECK_OBJ_NOTNULL(sp->wrk->resp, HTTP_MAGIC);
+	return (atoi(sp->wrk->resp->hd[HTTP_HDR_STATUS].b));
 }
 
+/*--------------------------------------------------------------------*/
+
+#define VBEREQ(dir, type,onm,field)					\
+void									\
+VRT_l_##dir##_##onm(const struct sess *sp, type a)			\
+{									\
+	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);				\
+	sp->wrk->field = a;						\
+}									\
+									\
+type									\
+VRT_r_##dir##_##onm(const struct sess *sp)				\
+{									\
+	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);				\
+	return (sp->wrk->field);					\
+}
+
+VBEREQ(beresp, unsigned, cacheable, cacheable)
+VBEREQ(beresp, double, grace, grace)
+
+/*--------------------------------------------------------------------
+ * XXX: Working relative to t_req is maybe not the right thing, we could
+ * XXX: have spent a long time talking to the backend since then.
+ * XXX: It might make sense to cache a timestamp as "current time"
+ * XXX: before vcl_recv (== t_req) and vcl_fetch.
+ * XXX: On the other hand, that might lead to inconsistent behaviour
+ * XXX: where an object expires while we are running VCL code, and
+ * XXX: and that may not be a good idea either.
+ * XXX: See also related t_req use in cache_hash.c
+ */
+
+void
+VRT_l_beresp_ttl(const struct sess *sp, double a)
+{
+
+	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
+	WSP(sp, SLT_TTL, "%u VCL %.0f %.0f", sp->xid, a, sp->t_req);
+	/*
+	 * If people set obj.ttl = 0s, they don't expect it to be cacheable
+	 * any longer, but it will still be for up to 1s - epsilon because
+	 * of the rounding to seconds.
+	 * We special case and make sure that rounding does not surprise.
+	 */
+	if (a <= 0)
+		sp->wrk->ttl = sp->t_req - 1;
+	else
+		sp->wrk->ttl = sp->t_req + a;
+}
+
+double
+VRT_r_beresp_ttl(const struct sess *sp)
+{
+	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
+	return (sp->wrk->ttl - sp->t_req);
+}
+
+void
+VRT_l_beresp_status(const struct sess *sp, int num)
+{
+	char *p;
+
+	assert(num >= 100 && num <= 999);
+	p = WS_Alloc(sp->wrk->beresp->ws, 4);
+	if (p == NULL)
+		WSP(sp, SLT_LostHeader, "%s", "obj.status");
+	else
+		sprintf(p, "%d", num);
+	http_SetH(sp->wrk->beresp, HTTP_HDR_STATUS, p);
+}
+
+int
+VRT_r_beresp_status(const struct sess *sp)
+{
+	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
+	/* XXX: use http_GetStatus() */
+	if (sp->wrk->beresp->status)
+		return (sp->wrk->beresp->status);
+	AN(sp->wrk->beresp->hd[HTTP_HDR_STATUS].b);
+	return (atoi(sp->wrk->beresp->hd[HTTP_HDR_STATUS].b));
+}
+
+
 void
 VRT_l_bereq_connect_timeout(struct sess *sp, double num)
 {
 	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
-	sp->connect_timeout = (num > 0 ? num : 0);
+	sp->wrk->connect_timeout = (num > 0 ? num : 0);
 }
 
 double
 VRT_r_bereq_connect_timeout(struct sess *sp)
 {
 	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
-	return sp->connect_timeout;
+	return (sp->wrk->connect_timeout);
 }
 
 void
 VRT_l_bereq_first_byte_timeout(struct sess *sp, double num)
 {
 	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
-	sp->first_byte_timeout = (num > 0 ? num : 0);
+	sp->wrk->first_byte_timeout = (num > 0 ? num : 0);
 }
 
 double
 VRT_r_bereq_first_byte_timeout(struct sess *sp)
 {
 	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
-	return sp->first_byte_timeout;
+	return (sp->wrk->first_byte_timeout);
 }
 
 void
 VRT_l_bereq_between_bytes_timeout(struct sess *sp, double num)
 {
 	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
-	sp->between_bytes_timeout = (num > 0 ? num : 0);
+	sp->wrk->between_bytes_timeout = (num > 0 ? num : 0);
 }
 
 double
 VRT_r_bereq_between_bytes_timeout(struct sess *sp)
 {
 	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
-	return sp->between_bytes_timeout;
+	return (sp->wrk->between_bytes_timeout);
 }
 
 /*--------------------------------------------------------------------*/
@@ -371,6 +521,8 @@ VRT_l_obj_ttl(const struct sess *sp, double a)
 
 	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
 	CHECK_OBJ_NOTNULL(sp->obj, OBJECT_MAGIC);	/* XXX */
+	if (sp->obj->objcore == NULL)
+		return;
 	WSP(sp, SLT_TTL, "%u VCL %.0f %.0f",
 	    sp->obj->xid, a, sp->t_req);
 	/*
@@ -391,6 +543,8 @@ VRT_r_obj_ttl(const struct sess *sp)
 {
 	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
 	CHECK_OBJ_NOTNULL(sp->obj, OBJECT_MAGIC);	/* XXX */
+	if (sp->obj->objcore == NULL)
+		return (0.0);
 	return (sp->obj->ttl - sp->t_req);
 }
 
@@ -422,42 +576,6 @@ VRT_r_obj_grace(const struct sess *sp)
 
 /*--------------------------------------------------------------------*/
 
-/* XXX: the VCL_info messages has unexpected fractions on the ttl */
-
-void
-VRT_l_obj_prefetch(const struct sess *sp, double a)
-{
-
-	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
-	CHECK_OBJ_NOTNULL(sp->obj, OBJECT_MAGIC);	/* XXX */
-	sp->obj->prefetch = 0.0;
-	if (a == 0.0)
-		sp->obj->prefetch = a;
-	else if (a > 0.0 && a + sp->t_req <= sp->obj->ttl)
-		sp->obj->prefetch = a + sp->t_req;
-	else if (a < 0.0 && a + sp->obj->ttl > sp->t_req)
-		sp->obj->prefetch = a;
-	else if (a > 0.0)
-		WSL(sp->wrk, SLT_VCL_info, sp->id,
-		    "XID %u: obj.prefetch (%g) after TTL (%g), ignored.",
-		    sp->obj->xid, a, sp->obj->ttl - sp->t_req);
-	else /* if (a < 0.0) */
-		WSL(sp->wrk, SLT_VCL_info, sp->id,
-		    "XID %u: obj.prefetch (%g) less than ttl (%g), ignored.",
-		    sp->obj->xid, a, sp->obj->ttl - sp->t_req);
-	EXP_Rearm(sp->obj);
-}
-
-double
-VRT_r_obj_prefetch(const struct sess *sp)
-{
-	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
-	CHECK_OBJ_NOTNULL(sp->obj, OBJECT_MAGIC);	/* XXX */
-	return (sp->obj->prefetch - sp->t_req);
-}
-
-/*--------------------------------------------------------------------*/
-
 #define VOBJ(type,onm,field)						\
 void									\
 VRT_l_obj_##onm(const struct sess *sp, type a)				\
@@ -496,6 +614,25 @@ VRT_r_req_backend(struct sess *sp)
 
 /*--------------------------------------------------------------------*/
 
+void
+VRT_l_req_esi(struct sess *sp, unsigned process_esi)
+{
+	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
+	/* only allow you to turn of esi in the main request
+	   else everything gets confused */
+	if(sp->esis == 0)
+	   sp->disable_esi = !process_esi;
+}
+
+unsigned
+VRT_r_req_esi(struct sess *sp)
+{
+	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
+	return (!sp->disable_esi);
+}
+
+/*--------------------------------------------------------------------*/
+
 int
 VRT_r_req_restarts(const struct sess *sp)
 {
@@ -573,7 +710,7 @@ VRT_r_server_identity(struct sess *sp)
 
 	if (heritage.identity[0] != '\0')
 		return heritage.identity;
-	else 
+	else
 		return heritage.name;
 }
 
@@ -584,9 +721,9 @@ VRT_r_server_hostname(struct sess *sp)
 	(void)sp;
 
 	if (vrt_hostname[0] == '\0')
-		gethostname(vrt_hostname, 255);
-	
-	return (const char*) &vrt_hostname;
+		AZ(gethostname(vrt_hostname, 255));
+
+	return (vrt_hostname);
 }
 
 int
@@ -642,26 +779,12 @@ VRT_r_obj_lastuse(const struct sess *sp)
 	return (TIM_real() - sp->obj->last_use);
 }
 
-const char *
-VRT_r_obj_hash(const struct sess *sp)
-{
-
-	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
-	if (sp->obj == NULL)
-		return (NULL);
-	CHECK_OBJ_NOTNULL(sp->obj, OBJECT_MAGIC);
-	if (sp->obj->objhead == NULL)
-		return (NULL);
-	CHECK_OBJ_NOTNULL(sp->obj->objhead, OBJHEAD_MAGIC);
-	return (sp->obj->objhead->hash);
-}
-
 unsigned
 VRT_r_req_backend_healthy(const struct sess *sp)
 {
 	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
 	CHECK_OBJ_NOTNULL(sp->director, DIRECTOR_MAGIC);
-	return (sp->director->healthy(sp));
+	return (VBE_Healthy_sp(sp, sp->director));
 }
 
 /*--------------------------------------------------------------------*/
@@ -719,6 +842,16 @@ VRT_double_string(const struct sess *sp, double num)
 	return (p);
 }
 
+char *
+VRT_time_string(const struct sess *sp, double t)
+{
+	char *p;
+
+	AN(p = WS_Alloc(sp->http->ws, TIM_FORMAT_SIZE));
+	TIM_format(t, p);
+	return p;
+}
+
 const char *
 VRT_backend_string(struct sess *sp)
 {
@@ -740,6 +873,22 @@ VRT_Rollback(struct sess *sp)
 
 /*--------------------------------------------------------------------*/
 
+void
+VRT_ESI(struct sess *sp)
+{
+
+	if (sp->cur_method != VCL_MET_FETCH) {
+		/* XXX: we should catch this at compile time */
+		WSP(sp, SLT_VCL_error,
+		    "esi can only be called from vcl_fetch");
+		return;
+	}
+
+	sp->wrk->do_esi = 1;
+}
+
+/*--------------------------------------------------------------------*/
+
 /*lint -e{818} sp could be const */
 void
 VRT_panic(struct sess *sp, const char *str, ...)
@@ -813,7 +962,7 @@ VRT_purge(struct sess *sp, char *cmds, ...)
 		a1 = va_arg(ap, char *);
 		good = 1;
 	}
-	if (!good) 
+	if (!good)
 		/* XXX: report error how ? */
 		BAN_Free(b);
 	else
@@ -823,7 +972,7 @@ VRT_purge(struct sess *sp, char *cmds, ...)
 /*--------------------------------------------------------------------*/
 
 void
-VRT_purge_string(struct sess *sp, char *str, ...)
+VRT_purge_string(struct sess *sp, const char *str, ...)
 {
 	char *p, *a1, *a2, *a3;
 	char **av;
@@ -846,22 +995,27 @@ VRT_purge_string(struct sess *sp, char *str, ...)
 	}
 	b = BAN_New();
 	good = 0;
-	for (i = 1; ; i += 3) {
-		a1 = av[i];
+	for (i = 1; ;) {
+		a1 = av[i++];
 		if (a1 == NULL)
 			break;
 		good = 0;
-		a2 = av[i + 1];
+		a2 = av[i++];
 		if (a2 == NULL)
 			break;
-		a3 = av[i + 2];
+		a3 = av[i++];
 		if (a3 == NULL)
 			break;
 		if (BAN_AddTest(NULL, b, a1, a2, a3))
 			break;
 		good = 1;
+		if (av[i] == NULL)
+			break;
+		good = 0;
+		if (strcmp(av[i++], "&&"))
+			break;
 	}
-	if (!good) 
+	if (!good)
 		/* XXX: report error how ? */
 		BAN_Free(b);
 	else
@@ -888,3 +1042,4 @@ VRT_memmove(void *dst, const void *src, unsigned len)
 	(void)memmove(dst, src, len);
 }
 
+/*lint -restore */
