@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006-2008 Linpro AS
+ * Copyright (c) 2006-2009 Linpro AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -26,14 +26,17 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: common.h 3126 2008-08-26 07:47:23Z phk $
+ * $Id: common.h 4481 2010-01-25 13:42:04Z phk $
  */
 
 struct cli;
 struct sockaddr;
 
+extern pid_t mgt_pid;
+#define ASSERT_MGT() do { assert(getpid() == mgt_pid);} while (0)
+
 /* cache_acceptor.c */
-void VCA_tweak_acceptor(struct cli *cli, const char *arg);
+void VCA_tweak_waiter(struct cli *cli, const char *arg);
 
 /* shmlog.c */
 void VSL_Panic(int *len, char **ptr);
@@ -42,6 +45,11 @@ void VSL_Panic(int *len, char **ptr);
 void VSL_MgtInit(const char *fn, unsigned size);
 extern struct varnish_stats *VSL_stats;
 
+/* varnishd.c */
+struct vsb;
+extern struct vsb *vident;
+int Symbol_Lookup(struct vsb *vsb, void *ptr);
+
 #define TRUST_ME(ptr)	((void*)(uintptr_t)(ptr))
 
 /* Really belongs in mgt.h, but storage_file chokes on both */
@@ -52,3 +60,11 @@ void mgt_child_inherit(int fd, const char *what);
 		fprintf(stderr, "Error: " __VA_ARGS__);		\
 		exit(2);					\
 	} while (0);
+
+/* A tiny helper for choosing hash/storage modules */
+struct choice {
+	const char      *name;
+	void            *ptr;
+};
+
+#define NEEDLESS_RETURN(foo)	return (foo)
