@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006-2008 Linpro AS
+ * Copyright (c) 2006-2009 Linpro AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -26,13 +26,14 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: hash_simple_list.c 3697 2009-02-09 12:13:38Z tfheen $
- *
  * This is the reference hash(/lookup) implementation
  */
 
 #include "config.h"
 
+#include "svnid.h"
+SVNID("$Id: hash_simple_list.c 4376 2009-11-27 11:06:32Z phk $")
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -90,8 +91,6 @@ hsl_lookup(const struct sess *sp, struct objhead *noh)
 	else
 		VTAILQ_INSERT_TAIL(&hsl_head, noh, hoh_list);
 
-	HSH_Copy(sp, noh);
-
 	Lck_Unlock(&hsl_mtx);
 	return (noh);
 }
