@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006-2008 Linpro AS
+ * Copyright (c) 2006-2009 Linpro AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -26,15 +26,13 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: heritage.h 3992 2009-03-23 12:50:04Z tfheen $
+ * $Id: heritage.h 4582 2010-02-21 22:00:09Z phk $
  *
  * This file contains the heritage passed when mgt forks cache
  */
 
 #include <pthread.h>
 
-#include "vqueue.h"
-
 struct listen_sock {
 	VTAILQ_ENTRY(listen_sock)	list;
 	int				sock;
@@ -82,6 +80,9 @@ struct params {
 	/* TTL used for synthesized error pages */
 	unsigned		err_ttl;
 
+	/* Maximum concurrent sessions */
+	unsigned		max_sess;
+
 	/* Worker threads and pool */
 	unsigned		wthread_min;
 	unsigned		wthread_max;
@@ -91,13 +92,15 @@ struct params {
 	unsigned		wthread_add_delay;
 	unsigned		wthread_fail_delay;
 	unsigned		wthread_purge_delay;
+	unsigned		wthread_stats_rate;
+	unsigned		wthread_stacksize;
 
 	unsigned		overflow_max;
 
 	/* Memory allocation hints */
 	unsigned		sess_workspace;
-	unsigned		obj_workspace;
 	unsigned		shm_workspace;
+	unsigned		http_headers;
 
 	unsigned		shm_reclen;
 
@@ -126,14 +129,6 @@ struct params {
 	/* Listen depth */
 	unsigned		listen_depth;
 
-	/* Srcaddr hash */
-	unsigned		srcaddr_hash;
-	unsigned		srcaddr_ttl;
-
-	/* HTTP proto behaviour */
-	unsigned		backend_http11;
-	unsigned		client_http11;
-
 	/* CLI related */
 	unsigned		cli_timeout;
 	unsigned		ping_interval;
@@ -161,7 +156,7 @@ struct params {
 
 	/* Read timeouts for backend */
 	double			first_byte_timeout;
-	double	 		between_bytes_timeout;
+	double			between_bytes_timeout;
 
 	/* How long to linger on sessions */
 	unsigned		session_linger;
@@ -178,9 +173,6 @@ struct params {
 	/* Log hash string to shm */
 	unsigned		log_hash;
 
-	/* Log hash string to shm */
-	unsigned		save_hash;
-
 	/* Log local socket address to shm */
 	unsigned		log_local_addr;
 
@@ -190,14 +182,27 @@ struct params {
 	/* Acceptable clockskew with backends */
 	unsigned		clock_skew;
 
-	/* Amount of time to sleep when running out of file
-	   descriptors.  In msecs */
-	unsigned		accept_fd_holdoff;
+	/* Acceptor pacer parameters */
+	double			acceptor_sleep_max;
+	double			acceptor_sleep_incr;
+	double			acceptor_sleep_decay;
 
 	/* Get rid of duplicate purges */
 	unsigned		purge_dups;
+
+	/* How long time does the ban lurker sleep */
+	double			ban_lurker_sleep;
+
+	/* Max size of the saintmode list. 0 == no saint mode. */
+	unsigned		saintmode_threshold;
+
+	unsigned		syslog_cli_traffic;
 };
 
+/*
+ * We declare this a volatile pointer, so that reads of parameters
+ * become atomic, leaving the CLI thread lattitude to change the values
+ */
 extern volatile struct params *params;
 extern struct heritage heritage;
 
