@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006-2008 Linpro AS
+ * Copyright (c) 2006-2009 Linpro AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -26,9 +26,11 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: mgt.h 3732 2009-02-11 07:13:12Z tfheen $
+ * $Id: mgt.h 4579 2010-02-21 20:15:44Z phk $
  */
 
+#include <vqueue.h>
+
 #include "common.h"
 #include "miniobj.h"
 
@@ -37,28 +39,37 @@
 struct cli;
 
 extern struct vev_base	*mgt_evb;
+extern unsigned		d_flag;
+extern int		exit_status;
 
 /* mgt_child.c */
-void mgt_run(int dflag, const char *T_arg);
-extern pid_t mgt_pid, child_pid;
+extern pid_t child_pid;
+void MGT_Run(void);
 void mgt_stop_child(void);
 
 /* mgt_cli.c */
 
-void mgt_cli_setup(int fdi, int fdo, int verbose, const char *ident);
+typedef void mgt_cli_close_f(void *priv);
+void mgt_cli_setup(int fdi, int fdo, int verbose, const char *ident, mgt_cli_close_f *close_func, void *priv);
 int mgt_cli_askchild(unsigned *status, char **resp, const char *fmt, ...);
 void mgt_cli_start_child(int fdi, int fdo);
 void mgt_cli_stop_child(void);
-void mgt_cli_telnet(int dflag, const char *T_arg);
+void mgt_cli_telnet(const char *T_arg);
+void mgt_cli_master(const char *M_arg);
+void mgt_cli_secret(const char *S_arg);
+void mgt_cli_close_all(void);
 
 /* mgt_param.c */
 void MCF_ParamSync(void);
 void MCF_ParamInit(struct cli *);
 void MCF_ParamSet(struct cli *, const char *param, const char *val);
+#ifdef DIAGNOSTICS
+void MCF_DumpMdoc(void);
+#endif
 
 /* mgt_vcc.c */
 void mgt_vcc_init(void);
-int mgt_vcc_default(const char *bflag, char *vcl, int Cflag);
+int mgt_vcc_default(const char *bflag, const char *f_arg, char *vcl, int Cflag);
 int mgt_push_vcls_and_start(unsigned *status, char **p);
 int mgt_has_vcl(void);
 extern char *mgt_cc_cmd;
@@ -75,4 +86,3 @@ extern char *mgt_cc_cmd;
 		fprintf(stderr, fmt "\n", __VA_ARGS__);	\
 		syslog(pri, fmt, __VA_ARGS__);		\
 	} while (0)
-
