@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006-2008 Linpro AS
+ * Copyright (c) 2006-2009 Linpro AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -26,13 +26,14 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: mgt_child.c 3599 2009-02-05 10:13:52Z tfheen $
- *
  * The mechanics of handling the child process
  */
 
 #include "config.h"
 
+#include "svnid.h"
+SVNID("$Id: mgt_child.c 4576 2010-02-17 19:07:21Z phk $")
+
 #include <unistd.h>
 #include <stdio.h>
 #include <string.h>
@@ -45,6 +46,10 @@
 #include <sys/types.h>
 #include <sys/wait.h>
 
+#ifdef HAVE_PRIV_H
+#include <priv.h>
+#endif
+
 #ifndef HAVE_SETPROCTITLE
 #include "compat/setproctitle.h"
 #endif
@@ -53,8 +58,8 @@
 #include <sys/prctl.h>
 #endif
 
-#include "heritage.h"
 #include "mgt.h"
+#include "heritage.h"
 #include "cli.h"
 #include "cli_priv.h"
 #include "mgt_cli.h"
@@ -64,7 +69,6 @@
 #include "vss.h"
 #include "vbm.h"
 
-pid_t		mgt_pid;
 pid_t		child_pid = -1;
 
 static struct vbitmap	*fd_map;
@@ -89,7 +93,6 @@ static const char * const ch_state[] = {
 	[CH_DIED] =	"died, (restarting)",
 };
 
-struct vev_base		*mgt_evb;
 static struct vev	*ev_poker;
 static struct vev	*ev_listen;
 static struct vlu	*vlu;
@@ -133,7 +136,7 @@ child_line(void *priv, const char *p)
 {
 	(void)priv;
 
-	REPORT(LOG_NOTICE, "Child (%d) said %s", child_pid, p);
+	REPORT(LOG_NOTICE, "Child (%jd) said %s", (intmax_t)child_pid, p);
 	return (0);
 }
 
@@ -170,8 +173,8 @@ child_poker(const struct vev *e, int what)
 	if (!mgt_cli_askchild(NULL, NULL, "ping\n"))
 		return (0);
 	REPORT(LOG_ERR,
-	    "Child (%d) not responding to ping, killing it.",
-	    child_pid);
+	    "Child (%jd) not responding to ping, killing it.",
+	    (intmax_t)child_pid);
 	if (params->diag_bitmap & 0x1000)
 		(void)kill(child_pid, SIGKILL);
 	else
@@ -203,7 +206,7 @@ open_sockets(void)
 		 * closes before we call accept(2) and nobody else are in
 		 * the listen queue to release us.
 		 */
-		TCP_nonblocking(ls->sock);
+		(void)TCP_nonblocking(ls->sock);
 		(void)TCP_filter_http(ls->sock);
 		good++;
 	}
@@ -229,6 +232,37 @@ close_sockets(void)
 
 /*--------------------------------------------------------------------*/
 
+/* Waive all privileges in the child, it does not need any */
+
+static inline void
+waive_privileges(void)
+{
+
+#ifdef HAVE_SETPPRIV
+	priv_set_t *empty;
+
+	if (!(empty = priv_allocset())) {
+		perror("priv_allocset_failed");
+		return;
+	}
+	priv_emptyset(empty);
+
+#define SETPPRIV(which, set)				       \
+	if (setppriv(PRIV_SET, which, set))		       \
+		perror("Waiving privileges failed on " #which)
+
+	SETPPRIV(PRIV_LIMIT, empty);
+	SETPPRIV(PRIV_INHERITABLE, empty);
+	SETPPRIV(PRIV_PERMITTED, empty); /* implies PRIV_EFFECTIVE */
+
+	priv_freeset(empty);
+#else
+	return;
+#endif
+}
+
+/*--------------------------------------------------------------------*/
+
 static void
 start_child(struct cli *cli)
 {
@@ -236,7 +270,7 @@ start_child(struct cli *cli)
 	unsigned u;
 	char *p;
 	struct vev *e;
-	int i, cp[2];
+	int i, j, cp[2];
 
 	if (child_state != CH_STOPPED && child_state != CH_DIED)
 		return;
@@ -304,7 +338,8 @@ start_child(struct cli *cli)
 		/* Close anything we shouldn't know about */
 		closelog();
 		printf("Closed fds:");
-		for (i = STDERR_FILENO + 1; i < getdtablesize(); i++) {
+		j = getdtablesize();
+		for (i = STDERR_FILENO + 1; i < j; i++) {
 			if (vbit_test(fd_map, i))
 				continue;
 			if (close(i) == 0)
@@ -312,6 +347,8 @@ start_child(struct cli *cli)
 		}
 		printf("\n");
 
+		waive_privileges();
+
 		setproctitle("Varnish-Chld %s", heritage.name);
 
 		(void)signal(SIGINT, SIG_DFL);
@@ -320,7 +357,7 @@ start_child(struct cli *cli)
 
 		exit(1);
 	}
-	REPORT(LOG_NOTICE, "child (%d) Started", pid);
+	REPORT(LOG_NOTICE, "child (%jd) Started", (intmax_t)pid);
 
 	/* Close stuff the child got */
 	closex(&heritage.std_fd);
@@ -345,7 +382,6 @@ start_child(struct cli *cli)
 	e->callback = child_listener;
 	AZ(vev_add(mgt_evb, e));
 	ev_listen = e;
-
 	AZ(ev_poker);
 	if (params->ping_interval > 0) {
 		e = vev_new();
@@ -362,11 +398,10 @@ start_child(struct cli *cli)
 	if (mgt_push_vcls_and_start(&u, &p)) {
 		REPORT(LOG_ERR, "Pushing vcls failed: %s", p);
 		free(p);
-		/* Pick up any stuff lingering on stdout/stderr */
-		(void)child_listener(NULL, EV_RD);
-		exit(2);
-	}
-	child_state = CH_RUNNING;
+		child_state = CH_RUNNING;
+		mgt_stop_child();
+	} else
+		child_state = CH_RUNNING;
 }
 
 /*--------------------------------------------------------------------*/
@@ -405,7 +440,7 @@ mgt_report_panic(pid_t r)
 	VSL_Panic(&l, &p);
 	if (*p == '\0')
 		return;
-	REPORT(LOG_ERR, "Child (%d) Panic message: %s", r, p);
+	REPORT(LOG_ERR, "Child (%jd) Panic message: %s", (intmax_t)r, p);
 }
 
 /*--------------------------------------------------------------------*/
@@ -433,13 +468,19 @@ mgt_sigchld(const struct vev *e, int what)
 	vsb = vsb_newauto();
 	XXXAN(vsb);
 	vsb_printf(vsb, "Child (%d) %s", r, status ? "died" : "ended");
-	if (!WIFEXITED(status) && WEXITSTATUS(status))
+	if (!WIFEXITED(status) && WEXITSTATUS(status)) {
 		vsb_printf(vsb, " status=%d", WEXITSTATUS(status));
-	if (WIFSIGNALED(status))
+		exit_status |= 0x20;
+	}
+	if (WIFSIGNALED(status)) {
 		vsb_printf(vsb, " signal=%d", WTERMSIG(status));
+		exit_status |= 0x40;
+	}
 #ifdef WCOREDUMP
-	if (WCOREDUMP(status))
+	if (WCOREDUMP(status)) {
 		vsb_printf(vsb, " (core dumped)");
+		exit_status |= 0x80;
+	}
 #endif
 	vsb_finish(vsb);
 	AZ(vsb_overflowed(vsb));
@@ -499,23 +540,12 @@ mgt_sigint(const struct vev *e, int what)
  */
 
 void
-mgt_run(int dflag, const char *T_arg)
+MGT_Run(void)
 {
 	struct sigaction sac;
 	struct vev *e;
 	int i;
 
-	mgt_pid = getpid();
-
-	mgt_evb = vev_new_base();
-	XXXAN(mgt_evb);
-
-	if (dflag)
-		mgt_cli_setup(0, 1, 1, "debug");
-
-	if (T_arg)
-		mgt_cli_telnet(dflag, T_arg);
-
 	e = vev_new();
 	XXXAN(e);
 	e->sig = SIGTERM;
@@ -547,22 +577,19 @@ mgt_run(int dflag, const char *T_arg)
 	AZ(sigaction(SIGPIPE, &sac, NULL));
 	AZ(sigaction(SIGHUP, &sac, NULL));
 
-	if (!dflag && !mgt_has_vcl())
+	if (!d_flag && !mgt_has_vcl())
 		REPORT0(LOG_ERR, "No VCL loaded yet");
-	else if (!dflag) {
+	else if (!d_flag) {
 		start_child(NULL);
 		if (child_state == CH_STOPPED)
 			exit(2);
-	} else
-		fprintf(stderr,
-		    "Debugging mode, enter \"start\" to start child\n");
+	}
 
 	i = vev_schedule(mgt_evb);
 	if (i != 0)
 		REPORT(LOG_ERR, "vev_schedule() = %d", i);
 
 	REPORT0(LOG_ERR, "manager dies");
-	exit(2);
 }
 
 /*--------------------------------------------------------------------*/
