@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006-2008 Linpro AS
+ * Copyright (c) 2006-2010 Redpill Linpro AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -25,19 +25,19 @@
  * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
- *
- * $Id: mgt_param.c 3739 2009-02-11 07:41:41Z tfheen $
  */
 
 #include "config.h"
 
+#include "svnid.h"
+SVNID("$Id: mgt_param.c 4557 2010-02-12 11:58:43Z phk $")
+
 #include <sys/types.h>
 #include <sys/stat.h>
 
 #include <grp.h>
 #include <limits.h>
 #include <pwd.h>
-#include <regex.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -68,7 +68,7 @@ mcf_findpar(const char *name)
 	int i;
 
 	for (i = 0; i < nparspec; i++)
-		if (!strcmp(parspec[i]->name, name)) 
+		if (!strcmp(parspec[i]->name, name))
 			return (parspec[i]);
 	return (NULL);
 }
@@ -92,25 +92,6 @@ tweak_generic_timeout(struct cli *cli, volatile unsigned *dst, const char *arg)
 		cli_out(cli, "%u", *dst);
 }
 
-static void
-tweak_generic_timeout_double(struct cli *cli, volatile double *dst, const char *arg)
-{
-	double u;
-
-	if (arg != NULL) {
-		u = strtod(arg, NULL);
-		if (u < 0) {
-			cli_out(cli,
-			    "Timeout must be greater or equal to zero\n");
-			cli_result(cli, CLIS_PARAM);
-			return;
-		}
-		*dst = u;
-	} else
-		cli_out(cli, "%f", *dst);
-}
-
-
 /*--------------------------------------------------------------------*/
 
 void
@@ -123,13 +104,65 @@ tweak_timeout(struct cli *cli, const struct parspec *par, const char *arg)
 }
 
 static void
-tweak_timeout_double(struct cli *cli, const struct parspec *par, const char *arg)
+tweak_timeout_double(struct cli *cli, const struct parspec *par,
+    const char *arg)
 {
 	volatile double *dest;
+	double u;
 
 	dest = par->priv;
-	tweak_generic_timeout_double(cli, dest, arg);
+	if (arg != NULL) {
+		u = strtod(arg, NULL);
+		if (u < par->min) {
+			cli_out(cli,
+			    "Timeout must be greater or equal to %.g\n",
+				 par->min);
+			cli_result(cli, CLIS_PARAM);
+			return;
+		}
+		if (u > par->max) {
+			cli_out(cli,
+			    "Timeout must be less than or equal to %.g\n",
+				 par->max);
+			cli_result(cli, CLIS_PARAM);
+			return;
+		}
+		*dest = u;
+	} else
+		cli_out(cli, "%.6f", *dest);
 }
+
+/*--------------------------------------------------------------------*/
+
+static void
+tweak_generic_double(struct cli *cli, const struct parspec *par,
+    const char *arg)
+{
+	volatile double *dest;
+	double u;
+
+	dest = par->priv;
+	if (arg != NULL) {
+		u = strtod(arg, NULL);
+		if (u < par->min) {
+			cli_out(cli,
+			    "Must be greater or equal to %.g\n",
+				 par->min);
+			cli_result(cli, CLIS_PARAM);
+			return;
+		}
+		if (u > par->max) {
+			cli_out(cli,
+			    "Must be less than or equal to %.g\n",
+				 par->max);
+			cli_result(cli, CLIS_PARAM);
+			return;
+		}
+		*dest = u;
+	} else
+		cli_out(cli, "%f", *dest);
+}
+
 /*--------------------------------------------------------------------*/
 
 static void
@@ -211,7 +244,7 @@ tweak_uint(struct cli *cli, const struct parspec *par, const char *arg)
 	volatile unsigned *dest;
 
 	dest = par->priv;
-	tweak_generic_uint(cli, dest, arg, par->umin, par->umax);
+	tweak_generic_uint(cli, dest, arg, (uint)par->min, (uint)par->max);
 }
 
 /*--------------------------------------------------------------------
@@ -413,12 +446,12 @@ tweak_cc_command(struct cli *cli, const struct parspec *par, const char *arg)
 /*--------------------------------------------------------------------*/
 
 static void
-tweak_acceptor(struct cli *cli, const struct parspec *par, const char *arg)
+tweak_waiter(struct cli *cli, const struct parspec *par, const char *arg)
 {
 
 	/* XXX should have tweak_generic_string */
 	(void)par;
-	VCA_tweak_acceptor(cli, arg);
+	VCA_tweak_waiter(cli, arg);
 }
 
 /*--------------------------------------------------------------------*/
@@ -481,7 +514,7 @@ static const struct parspec input_parspec[] = {
 		"Objects already cached will not be affected by changes "
 		"made until they are fetched from the backend again.\n"
 		"To force an immediate effect at the expense of a total "
-		"flush of the cache use \"url.purge .\"",
+		"flush of the cache use \"purge.url .\"",
 		0,
 		"120", "seconds" },
 	{ "sess_workspace", tweak_uint, &master.sess_workspace, 1024, UINT_MAX,
@@ -490,15 +523,14 @@ static const struct parspec input_parspec[] = {
 		"header and any edits done to it in the VCL code.\n"
 		"Minimum is 1024 bytes.",
 		DELAYED_EFFECT,
-		"16384", "bytes" },
-	{ "obj_workspace", tweak_uint, &master.obj_workspace, 1024, UINT_MAX,
-		"Bytes of HTTP protocol workspace allocated for objects. "
-		"This space must be big enough for the entire HTTP protocol "
-		"header and any edits done to it in the VCL code while it "
-		"is cached.\n"
-		"Minimum is 1024 bytes.",
-		DELAYED_EFFECT,
-		"8192", "bytes" },
+		"65536",
+		"bytes" },
+	{ "http_headers", tweak_uint, &master.http_headers, 32, UINT_MAX,
+		"Maximum number of HTTP headers we will deal with.\n"
+		"This space is preallocated in sessions and workthreads only "
+		"objects allocate only space for the headers they store.\n",
+		0,
+		"64", "header lines" },
 	{ "shm_workspace", tweak_uint, &master.shm_workspace, 4096, UINT_MAX,
 		"Bytes of shmlog workspace allocated for worker threads. "
 		"If too big, it wastes some ram, if too small it causes "
@@ -543,7 +575,7 @@ static const struct parspec input_parspec[] = {
 		0,
 		"on", "bool" },
 	{ "fetch_chunksize",
-		tweak_uint, &master.fetch_chunksize, 4, UINT_MAX / 1024,
+		tweak_uint, &master.fetch_chunksize, 4, UINT_MAX / 1024.,
 		"The default chunksize used by fetcher. "
 		"This should be bigger than the majority of objects with "
 		"short TTLs.\n"
@@ -576,33 +608,11 @@ static const struct parspec input_parspec[] = {
 		"Listen queue depth.",
 		MUST_RESTART,
 		"1024", "connections" },
-	{ "srcaddr_hash", tweak_uint, &master.srcaddr_hash, 63, UINT_MAX,
-		"Number of source address hash buckets.\n"
-		"Powers of two are bad, prime numbers are good.",
-		EXPERIMENTAL | MUST_RESTART,
-		"1049", "buckets" },
-	{ "srcaddr_ttl", tweak_uint, &master.srcaddr_ttl, 0, UINT_MAX,
-		"Lifetime of srcaddr entries.\n"
-		"Zero will disable srcaddr accounting entirely.",
-		EXPERIMENTAL,
-		"30", "seconds" },
-	{ "backend_http11", tweak_bool, &master.backend_http11, 0, 0,
-		"Force all backend requests to be HTTP/1.1.\n"
-		"By default we copy the protocol version from the "
-		"incoming client request.",
-		EXPERIMENTAL,
-		"on", "bool" },
-	{ "client_http11", tweak_bool, &master.client_http11, 0, 0,
-		"Force all client responses to be HTTP/1.1.\n"
-		"By default we copy the protocol version from the "
-		"backend response.",
-		EXPERIMENTAL,
-		"off", "bool" },
 	{ "cli_timeout", tweak_timeout, &master.cli_timeout, 0, 0,
 		"Timeout for the childs replies to CLI requests from "
 		"the master.",
 		0,
-		"5", "seconds" },
+		"10", "seconds" },
 	{ "ping_interval", tweak_uint, &master.ping_interval, 0, UINT_MAX,
 		"Interval between pings from parent to child.\n"
 		"Zero will disable pinging entirely, which makes "
@@ -662,26 +672,46 @@ static const struct parspec input_parspec[] = {
 		&master.first_byte_timeout,0, UINT_MAX,
 		"Default timeout for receiving first byte from backend. "
 		"We only wait for this many seconds for the first "
-		"byte before giving up. A value of 0 means it will never time out. "
+		"byte before giving up. A value of 0 means it will never time "
+		"out. "
 		"VCL can override this default value for each backend and "
 		"backend request. This parameter does not apply to pipe.",
 		0,
 		"60", "s" },
 	{ "between_bytes_timeout", tweak_timeout_double,
 		&master.between_bytes_timeout,0, UINT_MAX,
-		"Default timeout between bytes when receiving data from backend. "
+		"Default timeout between bytes when receiving data from "
+		"backend. "
 		"We only wait for this many seconds between bytes "
 		"before giving up. A value of 0 means it will never time out. "
-		"VCL can override this default value for each backend request and "
-		"backend request. This parameter does not apply to pipe.",
+		"VCL can override this default value for each backend request "
+		"and backend request. This parameter does not apply to pipe.",
 		0,
 		"60", "s" },
-	{ "accept_fd_holdoff", tweak_timeout,
-		&master.accept_fd_holdoff, 0,  3600*1000,
-		"If we run out of file descriptors, the accept thread will "
-		"sleep.  This parameter control for how long it will sleep.",
+	{ "acceptor_sleep_max", tweak_timeout_double,
+		&master.acceptor_sleep_max, 0,  10,
+		"If we run out of resources, such as file descriptors or "
+		"worker threads, the acceptor will sleep between accepts.\n"
+		"This parameter limits how long it can sleep between "
+		"attempts to accept new connections.",
 		EXPERIMENTAL,
-		"50", "ms" },
+		"0.050", "s" },
+	{ "acceptor_sleep_incr", tweak_timeout_double,
+		&master.acceptor_sleep_incr, 0,  1,
+		"If we run out of resources, such as file descriptors or "
+		"worker threads, the acceptor will sleep between accepts.\n"
+		"This parameter control how much longer we sleep, each time "
+		"we fail to accept a new connection.",
+		EXPERIMENTAL,
+		"0.001", "s" },
+	{ "acceptor_sleep_decay", tweak_generic_double,
+		&master.acceptor_sleep_decay, 0,  1,
+		"If we run out of resources, such as file descriptors or "
+		"worker threads, the acceptor will sleep between accepts.\n"
+		"This parameter (multiplicatively) reduce the sleep duration "
+		"for each succesfull accept. (ie: 0.9 = reduce by 10%)",
+		EXPERIMENTAL,
+		"0.900", "" },
 	{ "clock_skew", tweak_uint, &master.clock_skew, 0, UINT_MAX,
 		"How much clockskew we are willing to accept between the "
 		"backend and our own clock.",
@@ -692,6 +722,15 @@ static const struct parspec input_parspec[] = {
 		"have both IPv4 and IPv6 addresses.",
 		0,
 		"off", "bool" },
+	{ "session_max", tweak_uint,
+		&master.max_sess, 1000, UINT_MAX,
+		"Maximum number of sessions we will allocate "
+		"before just dropping connections.\n"
+		"This is mostly an anti-DoS measure, and setting it plenty "
+		"high should not hurt, as long as you have the memory for "
+		"it.\n",
+		0,
+		"100000", "sessions" },
 	{ "session_linger", tweak_uint,
 		&master.session_linger,0, UINT_MAX,
 		"How long time the workerthread lingers on the session "
@@ -701,9 +740,9 @@ static const struct parspec input_parspec[] = {
 		"completing.\n"
 		"Setting this too high results in worker threads not doing "
 		"anything for their keep, setting it too low just means that "
-		"more sessions take a detour around the acceptor.",
+		"more sessions take a detour around the waiter.",
 		EXPERIMENTAL,
-		"0", "ms" },
+		"50", "ms" },
 	{ "cli_buffer", tweak_uint, &master.cli_buffer, 4096, UINT_MAX,
 		"Size of buffer for CLI input."
 		"\nYou may need to increase this if you have big VCL files "
@@ -715,19 +754,13 @@ static const struct parspec input_parspec[] = {
 		"Log the hash string to shared memory log.\n",
 		0,
 		"off", "bool" },
-	{ "purge_hash", tweak_bool, &master.save_hash, 0, 0,
-		"Enable purge.hash command.\n"
-		"NB: this increases storage requirement per object "
-		"by the length of the hash string.\n",
-		MUST_RESTART,
-		"on", "bool" },
 	{ "log_local_address", tweak_bool, &master.log_local_addr, 0, 0,
 		"Log the local address on the TCP connection in the "
 		"SessionOpen shared memory record.\n",
 		0,
 		"off", "bool" },
-	{ "acceptor", tweak_acceptor, NULL, 0, 0,
-		"Select the acceptor kernel interface.\n",
+	{ "waiter", tweak_waiter, NULL, 0, 0,
+		"Select the waiter kernel interface.\n",
 		EXPERIMENTAL | MUST_RESTART,
 		"default", NULL },
 	{ "diag_bitmap", tweak_diag_bitmap, 0, 0, 0,
@@ -746,6 +779,8 @@ static const struct parspec input_parspec[] = {
 		"  0x00008000 - panic to abort2().\n"
 #endif
 		"  0x00010000 - synchronize shmlog.\n"
+		"  0x00020000 - synchronous start of persistence.\n"
+		"  0x80000000 - do edge-detection on digest.\n"
 		"Use 0x notation and do the bitor in your head :-)\n",
 		0,
 		"0", "bitmap" },
@@ -756,7 +791,26 @@ static const struct parspec input_parspec[] = {
 	{ "purge_dups", tweak_bool, &master.purge_dups, 0, 0,
 		"Detect and eliminate duplicate purges.\n",
 		0,
-		"off", "bool" },
+		"on", "bool" },
+	{ "syslog_cli_traffic", tweak_bool, &master.syslog_cli_traffic, 0, 0,
+		"Log all CLI traffic to syslog(LOG_INFO).\n",
+		0,
+		"on", "bool" },
+	{ "ban_lurker_sleep", tweak_timeout_double,
+		&master.ban_lurker_sleep, 0, UINT_MAX,
+		"How long time does the ban lurker thread sleeps between "
+		"successfull attempts to push the last item up the purge "
+		" list.  It always sleeps a second when nothing can be done.\n"
+		"A value of zero disables the ban lurker.",
+		0,
+		"0.0", "s" },
+	{ "saintmode_threshold", tweak_uint,
+		&master.saintmode_threshold, 0, UINT_MAX,
+		"The maximum number of objects held off by saint mode before "
+		"no further will be made to the backend until one times out.  "
+		"A value of 0 disables saintmode.",
+		EXPERIMENTAL,
+		"10", "objects" },
 	{ NULL, NULL, NULL }
 };
 
@@ -911,7 +965,7 @@ MCF_AddParams(const struct parspec *ps)
 			margin = strlen(pp->name) + 1;
 		n++;
 	}
-	parspec = realloc(parspec, (nparspec + n + 1) * sizeof *parspec);
+	parspec = realloc(parspec, (1L + nparspec + n) * sizeof *parspec);
 	XXXAN(parspec);
 	for (pp = ps; pp->name != NULL; pp++)
 		parspec[nparspec++] = pp;
@@ -955,3 +1009,66 @@ MCF_ParamInit(struct cli *cli)
 
 	params = &master;
 }
+
+/*--------------------------------------------------------------------*/
+
+#ifdef DIAGNOSTICS
+
+void
+MCF_DumpMdoc(void)
+{
+	const struct parspec *pp;
+	const char *p, *q;
+	int i;
+
+	printf(".Bl -tag -width 4n\n");
+	for (i = 0; i < nparspec; i++) {
+		pp = parspec[i];
+		printf(".It Va %s\n", pp->name);
+		if (pp->units != NULL && *pp->units != '\0')
+			printf("Units:\n.Dv %s\n.br\n", pp->units);
+		printf("Default:\n.Dv %s\n.br\n", pp->def);
+		/*
+		 * XXX: we should mark the params with one/two flags
+		 * XXX: that say if ->min/->max are valid, so we
+		 * XXX: can emit those also in help texts.
+		 */
+		if (pp->flags) {
+			printf("Flags:\n.Dv \"");
+			q = "";
+			if (pp->flags & DELAYED_EFFECT) {
+				printf("%sdelayed", q);
+				q = ", ";
+			}
+			if (pp->flags & MUST_RESTART) {
+				printf("%smust_restart", q);
+				q = ", ";
+			}
+			if (pp->flags & MUST_RELOAD) {
+				printf("%smust_reload", q);
+				q = ", ";
+			}
+			if (pp->flags & EXPERIMENTAL) {
+				printf("%sexperimental", q);
+				q = ", ";
+			}
+			printf("\"\n.br\n");
+		}
+		printf(".Pp\n");
+		for (p = pp->descr; *p; p++) {
+			if (*p == '\n' && p[1] =='\0')
+				break;
+			if (*p == '\n' && p[1] =='\n') {
+				printf("\n.Pp\n");
+				p++;
+			} else if (*p == '\n') {
+				printf("\n.br\n");
+			} else {
+				printf("%c", *p);
+			}
+		}
+		printf("\n.Pp\n");
+	}
+	printf(".El\n");
+}
+#endif /* DIAGNOSTICS */
