@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006-2008 Linpro AS
+ * Copyright (c) 2006-2009 Linpro AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -26,13 +26,14 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: mgt_vcc.c 3732 2009-02-11 07:13:12Z tfheen $
- *
  * VCL compiler stuff
  */
 
 #include "config.h"
 
+#include "svnid.h"
+SVNID("$Id: mgt_vcc.c 4479 2010-01-25 11:36:43Z phk $")
+
 #include <sys/types.h>
 #include <sys/wait.h>
 
@@ -43,13 +44,8 @@
 #include <string.h>
 #include <unistd.h>
 
-#ifndef HAVE_ASPRINTF
-#include "compat/asprintf.h"
-#endif
 #include "vsb.h"
 
-#include "vqueue.h"
-
 #include "libvcl.h"
 #include "cli.h"
 #include "cli_priv.h"
@@ -328,13 +324,16 @@ mgt_vcc_delbyname(const char *name)
 /*--------------------------------------------------------------------*/
 
 int
-mgt_vcc_default(const char *b_arg, char *vcl, int C_flag)
+mgt_vcc_default(const char *b_arg, const char *f_arg, char *vcl, int C_flag)
 {
 	char *addr, *port;
 	char *vf;
 	struct vsb *sb;
 	struct vclprog *vp;
+	char buf[BUFSIZ];
 
+	/* XXX: annotate vcl with -b/-f arg so people know where it came from */
+	(void)f_arg;
 	if (b_arg != NULL) {
 		AZ(vcl);
 		/*
@@ -358,15 +357,17 @@ mgt_vcc_default(const char *b_arg, char *vcl, int C_flag)
 			return (1);
 		}
 
-		asprintf(&vcl,
+		bprintf(buf,
 		    "backend default {\n"
 		    "    .host = \"%s\";\n"
 		    "    .port = \"%s\";\n"
 		    "}\n", addr, port ? port : "http");
 		free(addr);
 		free(port);
+		vcl = strdup(buf);
 		AN(vcl);
 	}
+	strcpy(buf, "boot");
 
 	vf = mgt_VccCompile(&sb, vcl, C_flag);
 	free(vcl);
@@ -374,7 +375,7 @@ mgt_vcc_default(const char *b_arg, char *vcl, int C_flag)
 		fprintf(stderr, "%s", vsb_data(sb));
 	vsb_delete(sb);
 	if (C_flag) {
-		if (vf != NULL) 
+		if (vf != NULL)
 			AZ(unlink(vf));
 		return (0);
 	}
@@ -382,7 +383,7 @@ mgt_vcc_default(const char *b_arg, char *vcl, int C_flag)
 		fprintf(stderr, "\nVCL compilation failed\n");
 		return (1);
 	}
-	vp = mgt_vcc_add("boot", vf);
+	vp = mgt_vcc_add(buf, vf);
 	vp->active = 1;
 	return (0);
 }
@@ -405,13 +406,13 @@ mgt_push_vcls_and_start(unsigned *status, char **p)
 
 	VTAILQ_FOREACH(vp, &vclhead, list) {
 		if (mgt_cli_askchild(status, p,
-		    "vcl.load %s %s\n", vp->name, vp->fname))
+		    "vcl.load \"%s\" %s\n", vp->name, vp->fname))
 			return (1);
 		free(*p);
 		if (!vp->active)
 			continue;
 		if (mgt_cli_askchild(status, p,
-		    "vcl.use %s\n", vp->name))
+		    "vcl.use \"%s\"\n", vp->name))
 			return (1);
 		free(*p);
 	}
