@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006-2008 Linpro AS
+ * Copyright (c) 2006-2009 Linpro AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -26,7 +26,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: mgt_param.c 3440 2008-11-25 13:39:15Z phk $
+ * $Id: vparam.h 4599 2010-03-01 11:20:21Z phk $
  */
 
 struct parspec;
@@ -37,8 +37,8 @@ struct parspec {
 	const char	*name;
 	tweak_t		*func;
 	volatile void	*priv;
-	unsigned	umin;
-	unsigned	umax;
+	double		min;
+	double		max;
 	const char	*descr;
 	int		 flags;
 #define DELAYED_EFFECT 1
