@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2009 Linpro AS
+ * Copyright (c) 2008-2010 Redpill Linpro AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -24,20 +24,24 @@
  * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
- *
- * $Id: vtc_http.c 4004 2009-03-27 12:25:11Z tfheen $
  */
 
 #include "config.h"
 
+#include "svnid.h"
+SVNID("$Id: vtc_http.c 4635 2010-03-24 09:21:26Z phk $")
+
 #include <poll.h>
 #include <stdio.h>
 #include <poll.h>
+#include <pthread.h>
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
 #include <unistd.h>
-#include <ctype.h>
+
+#include <sys/types.h>
+#include <sys/socket.h>
 
 #include "libvarnish.h"
 #include "vct.h"
@@ -52,7 +56,7 @@ struct http {
 	unsigned		magic;
 #define HTTP_MAGIC		0x2f02169c
 	int			fd;
-	int			client;
+	int			sfd;
 	int			timeout;
 	struct vtclog		*vl;
 
@@ -68,8 +72,23 @@ struct http {
 	char			*resp[MAX_HDR];
 };
 
+#define ONLY_CLIENT(hp, av)						\
+	do {								\
+		if (hp->sfd >= 0)					\
+			vtc_log(hp->vl, 0,				\
+			    "\"%s\" only possible in client", av[0]);	\
+	} while (0)
+
+#define ONLY_SERVER(hp, av)						\
+	do {								\
+		if (hp->sfd < 0)					\
+			vtc_log(hp->vl, 0,				\
+			    "\"%s\" only possible in server", av[0]);	\
+	} while (0)
+
+
 /* XXX: we may want to vary this */
-static const char *nl = "\r\n";
+static const char * const nl = "\r\n";
 
 /**********************************************************************
  * Generate a synthetic body
@@ -119,7 +138,8 @@ http_write(const struct http *hp, int lvl, const char *pfx)
 	AZ(vsb_overflowed(hp->vsb));
 	vtc_dump(hp->vl, lvl, pfx, vsb_data(hp->vsb));
 	l = write(hp->fd, vsb_data(hp->vsb), vsb_len(hp->vsb));
-	assert(l == vsb_len(hp->vsb));
+	if (l != vsb_len(hp->vsb))
+		vtc_log(hp->vl, 0, "Write failed: %s", strerror(errno));
 }
 
 /**********************************************************************
@@ -203,27 +223,22 @@ cmd_http_expect(CMD_ARGS)
 	cmp = av[1];
 	rhs = cmd_var_resolve(hp, av[2]);
 	if (!strcmp(cmp, "==")) {
-		if (strcmp(lhs, rhs)) {
+		if (strcmp(lhs, rhs))
 			vtc_log(hp->vl, 0, "EXPECT %s (%s) %s %s (%s) failed",
 			    av[0], lhs, av[1], av[2], rhs);
-			exit (1);
-		} else {
+		else
 			vtc_log(hp->vl, 4, "EXPECT %s (%s) %s %s (%s) match",
 			    av[0], lhs, av[1], av[2], rhs);
-		}
 	} else if (!strcmp(cmp, "!=")) {
-		if (!strcmp(lhs, rhs)) {
+		if (!strcmp(lhs, rhs))
 			vtc_log(hp->vl, 0, "EXPECT %s (%s) %s %s (%s) failed",
 			    av[0], lhs, av[1], av[2], rhs);
-			exit (1);
-		} else {
+		else
 			vtc_log(hp->vl, 4, "EXPECT %s (%s) %s %s (%s) match",
 			    av[0], lhs, av[1], av[2], rhs);
-		}
 	} else {
 		vtc_log(hp->vl, 0, "EXPECT %s (%s) %s %s (%s) not implemented",
 		    av[0], lhs, av[1], av[2], rhs);
-		exit (1);
 	}
 }
 
@@ -321,12 +336,17 @@ http_rxchar_eof(struct http *hp, int n)
 		pfd[0].events = POLLIN;
 		pfd[0].revents = 0;
 		i = poll(pfd, 1, hp->timeout);
+		if (i <= 0)
+			vtc_log(hp->vl, 0, "HTTP rx failed (poll: %s)",
+			    strerror(errno));
 		assert(i > 0);
-		assert(hp->prxbuf < hp->nrxbuf);
+		assert(hp->prxbuf + n < hp->nrxbuf);
 		i = read(hp->fd, hp->rxbuf + hp->prxbuf, n);
 		if (i == 0)
 			return (i);
-		assert(i > 0);
+		if (i <= 0)
+			vtc_log(hp->vl, 0, "HTTP rx failed (read: %s)",
+			    strerror(errno));
 		hp->prxbuf += i;
 		hp->rxbuf[hp->prxbuf] = '\0';
 		n -= i;
@@ -340,6 +360,8 @@ http_rxchar(struct http *hp, int n)
 	int i;
 
 	i = http_rxchar_eof(hp, n);
+	if (i <= 0)
+		vtc_log(hp->vl, 0, "HTTP rx failed (%s)", strerror(errno));
 	assert(i > 0);
 }
 
@@ -358,7 +380,7 @@ http_swallow_body(struct http *hp, char * const *hh, int body)
 	p = http_find_header(hh, "content-length");
 	if (p != NULL) {
 		l = strtoul(p, NULL, 0);
-		hp->body = q = hp->rxbuf + hp->prxbuf;
+		hp->body = hp->rxbuf + hp->prxbuf;
 		http_rxchar(hp, l);
 		vtc_dump(hp->vl, 4, "body", hp->body);
 		sprintf(hp->bodylen, "%d", l);
@@ -396,7 +418,7 @@ http_swallow_body(struct http *hp, char * const *hh, int body)
 		return;
 	}
 	if (body) {
-		hp->body = q = hp->rxbuf + hp->prxbuf;
+		hp->body = hp->rxbuf + hp->prxbuf;
 		do  {
 			i = http_rxchar_eof(hp, 1);
 			ll += i;
@@ -422,7 +444,6 @@ http_rxhdr(struct http *hp)
 	while (1) {
 		http_rxchar(hp, 1);
 		p = hp->rxbuf + hp->prxbuf - 1;
-		i = 0;
 		for (i = 0; p > hp->rxbuf; p--) {
 			if (*p != '\n')
 				break;
@@ -450,14 +471,12 @@ cmd_http_rxresp(CMD_ARGS)
 	(void)cmd;
 	(void)vl;
 	CAST_OBJ_NOTNULL(hp, priv, HTTP_MAGIC);
-	AN(hp->client);
+	ONLY_CLIENT(hp, av);
 	assert(!strcmp(av[0], "rxresp"));
 	av++;
 
-	for(; *av != NULL; av++) {
-		fprintf(stderr, "Unknown http rxresp spec: %s\n", *av);
-		exit (1);
-	}
+	for(; *av != NULL; av++)
+		vtc_log(hp->vl, 0, "Unknown http rxresp spec: %s\n", *av);
 	vtc_log(hp->vl, 3, "rxresp");
 	http_rxhdr(hp);
 	http_splitheader(hp, 0);
@@ -486,7 +505,7 @@ cmd_http_txresp(CMD_ARGS)
 	(void)cmd;
 	(void)vl;
 	CAST_OBJ_NOTNULL(hp, priv, HTTP_MAGIC);
-	AZ(hp->client);
+	ONLY_SERVER(hp, av);
 	assert(!strcmp(av[0], "txresp"));
 	av++;
 
@@ -520,6 +539,7 @@ cmd_http_txresp(CMD_ARGS)
 		if (!strcmp(*av, "-body")) {
 			AZ(body);
 			REPLACE(body, av[1]);
+			AN(body);
 			av++;
 			bodylen = strlen(body);
 			for (b = body; *b != '\0'; b++) {
@@ -540,10 +560,8 @@ cmd_http_txresp(CMD_ARGS)
 		} else
 			break;
 	}
-	if (*av != NULL) {
-		fprintf(stderr, "Unknown http txresp spec: %s\n", *av);
-		exit (1);
-	}
+	if (*av != NULL)
+		vtc_log(hp->vl, 0, "Unknown http txresp spec: %s\n", *av);
 	if (body != NULL)
 		vsb_printf(hp->vsb, "Content-Length: %d%s", bodylen, nl);
 	vsb_cat(hp->vsb, nl);
@@ -564,14 +582,12 @@ cmd_http_rxreq(CMD_ARGS)
 	(void)cmd;
 	(void)vl;
 	CAST_OBJ_NOTNULL(hp, priv, HTTP_MAGIC);
-	AZ(hp->client);
+	ONLY_SERVER(hp, av);
 	assert(!strcmp(av[0], "rxreq"));
 	av++;
 
-	for(; *av != NULL; av++) {
-		fprintf(stderr, "Unknown http rxreq spec: %s\n", *av);
-		exit (1);
-	}
+	for(; *av != NULL; av++)
+		vtc_log(hp->vl, 0, "Unknown http rxreq spec: %s\n", *av);
 	vtc_log(hp->vl, 3, "rxreq");
 	http_rxhdr(hp);
 	http_splitheader(hp, 1);
@@ -587,14 +603,12 @@ cmd_http_rxhdrs(CMD_ARGS)
 	(void)cmd;
 	(void)vl;
 	CAST_OBJ_NOTNULL(hp, priv, HTTP_MAGIC);
-	AZ(hp->client);
+	ONLY_SERVER(hp, av);
 	assert(!strcmp(av[0], "rxhdrs"));
 	av++;
 
-	for(; *av != NULL; av++) {
-		fprintf(stderr, "Unknown http rxreq spec: %s\n", *av);
-		exit (1);
-	}
+	for(; *av != NULL; av++)
+		vtc_log(hp->vl, 0, "Unknown http rxreq spec: %s\n", *av);
 	vtc_log(hp->vl, 3, "rxhdrs");
 	http_rxhdr(hp);
 	http_splitheader(hp, 1);
@@ -608,14 +622,12 @@ cmd_http_rxbody(CMD_ARGS)
 	(void)cmd;
 	(void)vl;
 	CAST_OBJ_NOTNULL(hp, priv, HTTP_MAGIC);
-	AZ(hp->client);
+	ONLY_SERVER(hp, av);
 	assert(!strcmp(av[0], "rxbody"));
 	av++;
 
-	for(; *av != NULL; av++) {
-		fprintf(stderr, "Unknown http rxreq spec: %s\n", *av);
-		exit (1);
-	}
+	for(; *av != NULL; av++)
+		vtc_log(hp->vl, 0, "Unknown http rxreq spec: %s\n", *av);
 	vtc_log(hp->vl, 3, "rxbody");
 	http_swallow_body(hp, hp->req, 0);
 	vtc_log(hp->vl, 4, "bodylen = %s", hp->bodylen);
@@ -637,7 +649,7 @@ cmd_http_txreq(CMD_ARGS)
 	(void)cmd;
 	(void)vl;
 	CAST_OBJ_NOTNULL(hp, priv, HTTP_MAGIC);
-	AN(hp->client);
+	ONLY_CLIENT(hp, av);
 	assert(!strcmp(av[0], "txreq"));
 	av++;
 
@@ -676,10 +688,8 @@ cmd_http_txreq(CMD_ARGS)
 		} else
 			break;
 	}
-	if (*av != NULL) {
-		fprintf(stderr, "Unknown http txreq spec: %s\n", *av);
-		exit (1);
-	}
+	if (*av != NULL)
+		vtc_log(hp->vl, 0, "Unknown http txreq spec: %s\n", *av);
 	if (body != NULL)
 		vsb_printf(hp->vsb, "Content-Length: %d%s", strlen(body), nl);
 	vsb_cat(hp->vsb, nl);
@@ -744,46 +754,137 @@ cmd_http_timeout(CMD_ARGS)
 	CAST_OBJ_NOTNULL(hp, priv, HTTP_MAGIC);
 	AN(av[1]);
 	AZ(av[2]);
-	hp->timeout = strtod(av[1], NULL) * 1000.0;
+	hp->timeout = (int)(strtod(av[1], NULL) * 1000.0);
+}
+
+/**********************************************************************
+ * expect other end to close (server only)
+ */
+
+static void
+cmd_http_expect_close(CMD_ARGS)
+{
+	struct http *hp;
+	struct pollfd fds[1];
+	char c;
+	int i;
+
+	(void)cmd;
+	(void)vl;
+	CAST_OBJ_NOTNULL(hp, priv, HTTP_MAGIC);
+	AZ(av[1]);
+	assert(hp->sfd >= 0);
+
+	vtc_log(vl, 4, "Expecting close (fd = %d)", hp->fd);
+	while (1) {
+		fds[0].fd = hp->fd;
+		fds[0].events = POLLIN | POLLHUP | POLLERR;
+		fds[0].revents = 0;
+		i = poll(fds, 1, 1000);
+		if (i == 0)
+			vtc_log(vl, 0, "Expected close: timeout");
+		if (i != 1 || !(fds[0].revents & POLLIN))
+			vtc_log(vl, 0,
+			    "Expected close: poll = %d, revents = 0x%x",
+			    i, fds[0].revents);
+		i = read(hp->fd, &c, 1);
+		if (i == 0)
+			break;
+		if (i == 1 && vct_islws(c))
+			continue;
+		vtc_log(vl, 0,
+		    "Expecting close: read = %d, c = 0x%02x", i, c);
+	}
+	vtc_log(vl, 4, "fd=%d EOF, as expected", hp->fd);
+}
+
+/**********************************************************************
+ * close and accept a new connection  (server only)
+ */
+
+static void
+cmd_http_accept(CMD_ARGS)
+{
+	struct http *hp;
+
+	(void)cmd;
+	(void)vl;
+	CAST_OBJ_NOTNULL(hp, priv, HTTP_MAGIC);
+	AZ(av[1]);
+	assert(hp->sfd >= 0);
+	TCP_close(&hp->fd);
+	vtc_log(vl, 4, "Accepting");
+	hp->fd = accept(hp->sfd, NULL, NULL);
+	if (hp->fd < 0)
+		vtc_log(vl, 0, "Accepted failed: %s", strerror(errno));
+	vtc_log(vl, 3, "Accepted socket fd is %d", hp->fd);
+}
+
+/**********************************************************************
+ * loop operator
+ */
+
+static void
+cmd_http_loop(CMD_ARGS)
+{
+	struct http *hp;
+	unsigned n, m;
+	char *s;
+
+	CAST_OBJ_NOTNULL(hp, priv, HTTP_MAGIC);
+	AN(av[1]);
+	AN(av[2]);
+	AZ(av[3]);
+	n = strtod(av[1], NULL);
+	for (m = 1 ; m <= n; m++) {
+		vtc_log(vl, 4, "Loop #%u", m);
+		s = strdup(av[2]);
+		AN(s);
+		parse_string(s, cmd, hp, vl);
+	}
 }
 
 /**********************************************************************
  * Execute HTTP specifications
  */
 
-static struct cmds http_cmds[] = {
-	{ "timeout",	cmd_http_timeout },
-	{ "txreq",	cmd_http_txreq },
+static const struct cmds http_cmds[] = {
+	{ "timeout",		cmd_http_timeout },
+	{ "txreq",		cmd_http_txreq },
 
-	{ "rxreq",	cmd_http_rxreq },
-	{ "rxhdrs",	cmd_http_rxhdrs },
-	{ "rxbody",	cmd_http_rxbody },
+	{ "rxreq",		cmd_http_rxreq },
+	{ "rxhdrs",		cmd_http_rxhdrs },
+	{ "rxbody",		cmd_http_rxbody },
 
-	{ "txresp",	cmd_http_txresp },
-	{ "rxresp",	cmd_http_rxresp },
-	{ "expect",	cmd_http_expect },
-	{ "send",	cmd_http_send },
-	{ "chunked",	cmd_http_chunked },
-	{ "delay",	cmd_delay },
-	{ "sema",	cmd_sema },
-	{ NULL,		NULL }
+	{ "txresp",		cmd_http_txresp },
+	{ "rxresp",		cmd_http_rxresp },
+	{ "expect",		cmd_http_expect },
+	{ "send",		cmd_http_send },
+	{ "chunked",		cmd_http_chunked },
+	{ "delay",		cmd_delay },
+	{ "sema",		cmd_sema },
+	{ "expect_close",	cmd_http_expect_close },
+	{ "accept",		cmd_http_accept },
+	{ "loop",		cmd_http_loop },
+	{ NULL,			NULL }
 };
 
 void
-http_process(struct vtclog *vl, const char *spec, int sock, int client)
+http_process(struct vtclog *vl, const char *spec, int sock, int sfd)
 {
 	struct http *hp;
 	char *s, *q;
 
+	(void)sfd;
 	ALLOC_OBJ(hp, HTTP_MAGIC);
 	AN(hp);
 	hp->fd = sock;
-	hp->vl = vl;
-	hp->client = client;
 	hp->timeout = 3000;
-	hp->nrxbuf = 8192;
+	hp->nrxbuf = 640*1024;
 	hp->vsb = vsb_newauto();
 	hp->rxbuf = malloc(hp->nrxbuf);		/* XXX */
+	hp->sfd = sfd;
+	hp->vl = vl;
 	AN(hp->rxbuf);
 	AN(hp->vsb);
 
