@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2009 Linpro AS
+ * Copyright (c) 2008-2010 Redpill Linpro AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -24,12 +24,13 @@
  * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
- *
- * $Id: vtc_server.c 3958 2009-03-19 12:50:02Z tfheen $
  */
 
 #include "config.h"
 
+#include "svnid.h"
+SVNID("$Id: vtc_server.c 4635 2010-03-24 09:21:26Z phk $")
+
 #include <stdio.h>
 #include <errno.h>
 #include <stdlib.h>
@@ -54,6 +55,7 @@ struct server {
 	char			*name;
 	struct vtclog		*vl;
 	VTAILQ_ENTRY(server)	list;
+	char			run;
 
 	unsigned		repeat;
 	char			*spec;
@@ -64,6 +66,8 @@ struct server {
 	struct vss_addr		**vss_addr;
 	char			*addr;
 	char			*port;
+	char			aaddr[32];
+	char			aport[32];
 
 	pthread_t		tp;
 };
@@ -90,20 +94,24 @@ server_thread(void *priv)
 
 	vl = vtc_logopen(s->name);
 
-	vtc_log(vl, 2, "Started on %s", s->listen);
+	vtc_log(vl, 2, "Started on %s:%s", s->aaddr, s->aport);
 	for (i = 0; i < s->repeat; i++) {
 		if (s->repeat > 1)
 			vtc_log(vl, 3, "Iteration %d", i);
 		addr = (void*)&addr_s;
 		l = sizeof addr_s;
 		fd = accept(s->sock, addr, &l);
-		vtc_log(vl, 3, "Accepted socket fd is %d", fd);
-		http_process(vl, s->spec, fd, 0);
+		if (fd < 0)
+			vtc_log(vl, 0, "Accepted failed: %s", strerror(errno));
+		http_process(vl, s->spec, fd, s->sock);
 		vtc_log(vl, 3, "shutting fd %d", fd);
 		assert((shutdown(fd, SHUT_WR) == 0)
 		    || errno == ENOTCONN || errno == ECONNRESET);
 		TCP_close(&fd);
 	}
+	macro_def(s->vl, s->name, "addr", NULL);
+	macro_def(s->vl, s->name, "port", NULL);
+	macro_def(s->vl, s->name, "sock", NULL);
 	vtc_log(vl, 2, "Ending");
 	return (NULL);
 }
@@ -126,7 +134,7 @@ server_new(const char *name)
 	if (*s->name != 's')
 		vtc_log(s->vl, 0, "Server name must start with 's'");
 
-	REPLACE(s->listen, "127.0.0.1:9080");
+	REPLACE(s->listen, "127.0.0.1:0");
 	AZ(VSS_parse(s->listen, &s->addr, &s->port));
 	s->repeat = 1;
 	s->depth = 1;
@@ -164,18 +172,24 @@ server_start(struct server *s)
 	vtc_log(s->vl, 2, "Starting server");
 	if (s->sock < 0) {
 		naddr = VSS_resolve(s->addr, s->port, &s->vss_addr);
-		if (naddr != 1) {
+		if (naddr != 1)
 			vtc_log(s->vl, 0,
 			    "Server s listen address not unique"
 			    " \"%s\" resolves to (%d) sockets",
 			    s->listen, naddr);
-			exit (1);
-		}
 		s->sock = VSS_listen(s->vss_addr[0], s->depth);
 		assert(s->sock >= 0);
+		TCP_myname(s->sock, s->aaddr, sizeof s->aaddr,
+		    s->aport, sizeof s->aport);
+		macro_def(s->vl, s->name, "addr", "%s", s->aaddr);
+		macro_def(s->vl, s->name, "port", "%s", s->aport);
+		macro_def(s->vl, s->name, "sock", "%s:%s", s->aaddr, s->aport);
+		/* Record the actual port, and reuse it on subsequent starts */
+		if (!strcmp(s->port, "0"))
+			REPLACE(s->port, s->aport);
 	}
-	vtc_log(s->vl, 3, "listen on %s (fd %d)",
-	    s->listen, s->sock);
+	vtc_log(s->vl, 1, "Listen on %s:%s", s->addr, s->port);
+	s->run = 1;
 	AZ(pthread_create(&s->tp, NULL, server_thread, s));
 }
 
@@ -191,14 +205,13 @@ server_wait(struct server *s)
 	CHECK_OBJ_NOTNULL(s, SERVER_MAGIC);
 	vtc_log(s->vl, 2, "Waiting for server");
 	AZ(pthread_join(s->tp, &res));
-	if (res != NULL) {
+	if (res != NULL && !vtc_stop)
 		vtc_log(s->vl, 0, "Server returned \"%p\"",
 		    (char *)res);
-		exit (1);
-	}
 	s->tp = 0;
 	TCP_close(&s->sock);
 	s->sock = -1;
+	s->run = 0;
 }
 
 /**********************************************************************
@@ -213,9 +226,7 @@ cmd_server_genvcl(struct vsb *vsb)
 	VTAILQ_FOREACH(s, &servers, list) {
 		vsb_printf(vsb,
 		    "backend %s { .host = \"%s\"; .port = \"%s\"; }\n",
-		    s->name,
-		    s->addr == NULL ? "127.0.0.1" : s->addr,
-		    s->port);
+		    s->name, s->aaddr, s->aport);
 	}
 }
 
@@ -236,9 +247,12 @@ cmd_server(CMD_ARGS)
 	if (av == NULL) {
 		/* Reset and free */
 		VTAILQ_FOREACH_SAFE(s, &servers, list, s2) {
+			CHECK_OBJ_NOTNULL(s, SERVER_MAGIC);
 			VTAILQ_REMOVE(&servers, s, list);
-			if (s->sock >= 0)
+			if (s->run) {
+				(void)pthread_cancel(s->tp);
 				server_wait(s);
+			}
 			server_delete(s);
 		}
 		return;
@@ -252,9 +266,26 @@ cmd_server(CMD_ARGS)
 			break;
 	if (s == NULL)
 		s = server_new(av[0]);
+	CHECK_OBJ_NOTNULL(s, SERVER_MAGIC);
 	av++;
 
 	for (; *av != NULL; av++) {
+		if (vtc_error)
+			break;
+		if (!strcmp(*av, "-wait")) {
+			if (!s->run)
+				vtc_log(s->vl, 0, "Server not -started");
+			server_wait(s);
+			continue;
+		}
+		/*
+		 * We do an implict -wait if people muck about with a
+		 * running server.
+		 */
+		if (s->run)
+			server_wait(s);
+
+		assert(s->run == 0);
 		if (!strcmp(*av, "-repeat")) {
 			s->repeat = atoi(av[1]);
 			av++;
@@ -270,14 +301,8 @@ cmd_server(CMD_ARGS)
 			server_start(s);
 			continue;
 		}
-		if (!strcmp(*av, "-wait")) {
-			server_wait(s);
-			continue;
-		}
-		if (**av == '-') {
+		if (**av == '-')
 			vtc_log(s->vl, 0, "Unknown server argument: %s", *av);
-			exit (1);
-		}
 		s->spec = *av;
 	}
 }
