@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2009 Linpro AS
+ * Copyright (c) 2008-2010 Redpill Linpro AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -24,19 +24,22 @@
  * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
- *
- * $Id: vtc_varnish.c 3958 2009-03-19 12:50:02Z tfheen $
  */
 
 #include "config.h"
 
+#include "svnid.h"
+SVNID("$Id: vtc_varnish.c 4595 2010-02-26 19:16:39Z phk $")
+
 #include <stdio.h>
 
 #include <ctype.h>
+#include <fcntl.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <string.h>
 #include <signal.h>
+#include <poll.h>
 #include <pthread.h>
 #include <inttypes.h>
 
@@ -65,16 +68,15 @@ struct varnish {
 
 	struct varnish_stats	*stats;
 
-	const char		*args;
+	struct vsb		*args;
 	int			fds[4];
 	pid_t			pid;
-	const char		*telnet;
-	const char		*accept;
 
 	pthread_t		tp;
 
 	int			cli_fd;
 	int			vcl_nbr;
+	char			*workdir;
 };
 
 static VTAILQ_HEAD(, varnish)	varnishes =
@@ -91,15 +93,22 @@ varnish_ask_cli(const struct varnish *v, const char *cmd, char **repl)
 	unsigned retval;
 	char *r;
 
-	vtc_dump(v->vl, 4, "CLI TX", cmd);
-	i = write(v->cli_fd, cmd, strlen(cmd));
-	assert(i == strlen(cmd));
-	i = write(v->cli_fd, "\n", 1);
-	assert(i == 1);
-	i = cli_readres(v->cli_fd, &retval, &r, 10.0);
+	if (cmd != NULL) {
+		vtc_dump(v->vl, 4, "CLI TX", cmd);
+		i = write(v->cli_fd, cmd, strlen(cmd));
+		assert(i == strlen(cmd));
+		i = write(v->cli_fd, "\n", 1);
+		assert(i == 1);
+	}
+	i = cli_readres(v->cli_fd, &retval, &r, 20.0);
+	if (i != 0) {
+		vtc_log(v->vl, 0, "CLI failed (%s) = %d %u %s",
+		    cmd, i, retval, r);
+		return ((enum cli_status_e)retval);
+	}
 	assert(i == 0);
+	vtc_log(v->vl, 3, "CLI RX  %u", retval);
 	vtc_dump(v->vl, 4, "CLI RX", r);
-	vtc_log(v->vl, 3, "CLI STATUS %u", retval);
 	if (repl != NULL)
 		*repl = r;
 	else
@@ -137,23 +146,34 @@ static struct varnish *
 varnish_new(const char *name)
 {
 	struct varnish *v;
+	char buf[1024];
 
 	AN(name);
 	ALLOC_OBJ(v, VARNISH_MAGIC);
 	AN(v);
 	REPLACE(v->name, name);
+
+	bprintf(buf, "%s/%s", vtc_tmpdir, name);
+	v->workdir = strdup(buf);
+	AN(v->workdir);
+
+	bprintf(buf, "rm -rf %s ; mkdir -p %s ; echo ' %ld' > %s/_S",
+	    v->workdir, v->workdir, random(), v->workdir);
+	AZ(system(buf));
+
 	v->vl = vtc_logopen(name);
 	AN(v->vl);
+
 	v->vl1 = vtc_logopen(name);
 	AN(v->vl1);
+
 	if (*v->name != 'v')
 		vtc_log(v->vl, 0, "Varnish name must start with 'v'");
 
-	v->args = "";
-	v->telnet = "127.0.0.1:9001";
-	v->accept = "127.0.0.1:9081";
+	v->args = vsb_newauto();
 	v->cli_fd = -1;
 	VTAILQ_INSERT_TAIL(&varnishes, v, list);
+
 	return (v);
 }
 
@@ -168,7 +188,17 @@ varnish_delete(struct varnish *v)
 	CHECK_OBJ_NOTNULL(v, VARNISH_MAGIC);
 	vtc_logclose(v->vl);
 	free(v->name);
-	/* XXX: MEMLEAK */
+	free(v->workdir);
+	VSL_Close();
+
+	/*
+	 * We do not delete the workdir, it may contain stuff people
+	 * want (coredumps, shmlog/stats etc), and trying to divine
+	 * "may want" is just too much trouble.  Leave it around and
+	 * nuke it at the start of the next test-run.
+	 */
+
+	/* XXX: MEMLEAK (?) */
 	FREE_OBJ(v);
 }
 
@@ -181,15 +211,26 @@ varnish_thread(void *priv)
 {
 	struct varnish *v;
 	char buf[BUFSIZ];
+	struct pollfd *fds, fd;
 	int i;
 
 	CAST_OBJ_NOTNULL(v, priv, VARNISH_MAGIC);
+	TCP_nonblocking(v->fds[0]);
 	while (1) {
+		fds = &fd;
+		memset(fds, 0, sizeof fds);
+		fds->fd = v->fds[0];
+		fds->events = POLLIN;
+		i = poll(fds, 1, 1000);
+		if (i == 0)
+			continue;
+		if (fds->revents & (POLLERR|POLLHUP))
+			break;
 		i = read(v->fds[0], buf, sizeof buf - 1);
 		if (i <= 0)
 			break;
 		buf[i] = '\0';
-		vtc_dump(v->vl1, 4, "debug", buf);
+		vtc_dump(v->vl1, 3, "debug", buf);
 	}
 	return (NULL);
 }
@@ -202,16 +243,33 @@ static void
 varnish_launch(struct varnish *v)
 {
 	struct vsb *vsb;
-	int i;
+	int i, nfd, nap;
+	struct vss_addr **ap;
+	char abuf[128], pbuf[128];
+	struct pollfd fd;
+	enum cli_status_e u;
+	char *r;
 
+	/* Create listener socket */
+	nap = VSS_resolve("127.0.0.1", "0", &ap);
+	AN(nap);
+	v->cli_fd = VSS_listen(ap[0], 1);
+	TCP_myname(v->cli_fd, abuf, sizeof abuf, pbuf, sizeof pbuf);
+
+	vsb_finish(v->args);
+	AZ(vsb_overflowed(v->args));
 	vtc_log(v->vl, 2, "Launch");
 	vsb = vsb_newauto();
 	AN(vsb);
 	vsb_printf(vsb, "cd ../varnishd &&");
-	vsb_printf(vsb, " ./varnishd -d -d -n /tmp/__%s", v->name);
-	vsb_printf(vsb, " -a '%s' -T %s", v->accept, v->telnet);
-	vsb_printf(vsb, " -P /tmp/__%s/varnishd.pid", v->name);
-	vsb_printf(vsb, " %s", v->args);
+	vsb_printf(vsb, " ./varnishd -d -d -n %s", v->workdir);
+	vsb_printf(vsb, " -p auto_restart=off");
+	vsb_printf(vsb, " -p syslog_cli_traffic=off");
+	vsb_printf(vsb, " -a '%s'", "127.0.0.1:0");
+	vsb_printf(vsb, " -S %s/_S", v->workdir);
+	vsb_printf(vsb, " -M %s:%s", abuf, pbuf);
+	vsb_printf(vsb, " -P %s/varnishd.pid", v->workdir);
+	vsb_printf(vsb, " %s", vsb_data(v->args));
 	vsb_finish(vsb);
 	AZ(vsb_overflowed(vsb));
 	vtc_log(v->vl, 3, "CMD: %s", vsb_data(vsb));
@@ -228,7 +286,7 @@ varnish_launch(struct varnish *v)
 		AZ(close(v->fds[2]));
 		AZ(close(v->fds[3]));
 		for (i = 3; i <getdtablesize(); i++)
-			close(i);
+			(void)close(i);
 		AZ(execl("/bin/sh", "/bin/sh", "-c", vsb_data(vsb), NULL));
 		exit(1);
 	}
@@ -239,26 +297,60 @@ varnish_launch(struct varnish *v)
 	vsb_delete(vsb);
 	AZ(pthread_create(&v->tp, NULL, varnish_thread, v));
 
-	vtc_log(v->vl, 3, "opening CLI connection");
-	for (i = 0; i < 30; i++) {
-		(void)usleep(200000);
-		v->cli_fd = VSS_open(v->telnet);
-		if (v->cli_fd >= 0)
-			break;
-	}
-	if (v->cli_fd < 0) {
-		vtc_log(v->vl, 0, "FAIL no CLI connection");
+	/* Wait for the varnish to call home */
+	fd.fd = v->cli_fd;
+	fd.events = POLLIN;
+	i = poll(&fd, 1, 6000);
+	if (i != 1) {
+		AZ(close(v->fds[1]));
 		(void)kill(v->pid, SIGKILL);
-		exit (1);
+		vtc_log(v->vl, 0, "FAIL timeout waiting for CLI connection");
+		return;
 	}
+	nfd = accept(v->cli_fd, NULL, NULL);
+	if (nfd < 0) {
+		AZ(close(v->fds[1]));
+		(void)kill(v->pid, SIGKILL);
+		vtc_log(v->vl, 0, "FAIL no CLI connection accepted");
+		return;
+	}
+
+	AZ(close(v->cli_fd));
+	v->cli_fd = nfd;
+	nfd = -1;
+
 	vtc_log(v->vl, 3, "CLI connection fd = %d", v->cli_fd);
 	assert(v->cli_fd >= 0);
-	vsb = vsb_newauto();
-	vsb_printf(vsb, "/tmp/__%s", v->name);
-	vsb_finish(vsb);
-	AZ(vsb_overflowed(vsb));
-	v->stats = VSL_OpenStats(vsb_data(vsb));
-	vsb_delete(vsb);
+
+
+	/* Receive the banner or auth response */
+	u = varnish_ask_cli(v, NULL, &r);
+	if (vtc_error)
+		return;
+	if (u != CLIS_AUTH)
+		vtc_log(v->vl, 0, "CLI auth demand expected: %u %s", u, r);
+
+	bprintf(abuf, "%s/_S", v->workdir);
+	nfd = open(abuf, O_RDONLY);
+	assert(nfd >= 0);
+
+	assert(sizeof abuf >= CLI_AUTH_RESPONSE_LEN + 6);
+	strcpy(abuf, "auth ");
+	CLI_response(nfd, r, abuf + 5);
+	AZ(close(nfd));
+	free(r);
+	strcat(abuf, "\n");
+
+	u = varnish_ask_cli(v, abuf, &r);
+	if (vtc_error)
+		return;
+	if (u != CLIS_OK)
+		vtc_log(v->vl, 0, "CLI auth command failed: %u %s", u, r);
+	free(r);
+
+	if (v->stats != NULL)
+		VSL_Close();
+	v->stats = VSL_OpenStats(v->workdir);
 }
 
 /**********************************************************************
@@ -269,14 +361,43 @@ static void
 varnish_start(struct varnish *v)
 {
 	enum cli_status_e u;
+	char *resp, *h, *p, *q;
 
 	if (v->cli_fd < 0)
 		varnish_launch(v);
+	if (vtc_error)
+		return;
 	vtc_log(v->vl, 2, "Start");
-	u = varnish_ask_cli(v, "start", NULL);
-	assert(u == CLIS_OK);
-	u = varnish_ask_cli(v, "debug.xid 1000", NULL);
-	assert(u == CLIS_OK);
+	u = varnish_ask_cli(v, "start", &resp);
+	if (vtc_error)
+		return;
+	if (u != CLIS_OK)
+		vtc_log(v->vl, 0, "CLI start command failed: %u %s", u, resp);
+	free(resp);
+	u = varnish_ask_cli(v, "debug.xid 1000", &resp);
+	if (vtc_error)
+		return;
+	if (u != CLIS_OK)
+		vtc_log(v->vl, 0, "CLI debug.xid command failed: %u %s",
+		    u, resp);
+	free(resp);
+	u = varnish_ask_cli(v, "debug.listen_address", &resp);
+	if (vtc_error)
+		return;
+	if (u != CLIS_OK)
+		vtc_log(v->vl, 0,
+		    "CLI debug.listen_address command failed: %u %s", u, resp);
+	h = resp;
+	p = strchr(h, ' ');
+	AN(p);
+	*p++ = '\0';
+	q = strchr(p, '\n');
+	AN(q);
+	*q = '\0';
+	vtc_log(v->vl, 2, "Listen on %s %s", h, p);
+	macro_def(v->vl, v->name, "addr", "%s", h);
+	macro_def(v->vl, v->name, "port", "%s", p);
+	macro_def(v->vl, v->name, "sock", "%s:%s", h, p);
 }
 
 /**********************************************************************
@@ -290,6 +411,11 @@ varnish_stop(struct varnish *v)
 
 	if (v->cli_fd < 0)
 		varnish_launch(v);
+	if (vtc_error)
+		return;
+	macro_def(v->vl, v->name, "addr", NULL);
+	macro_def(v->vl, v->name, "port", NULL);
+	macro_def(v->vl, v->name, "sock", NULL);
 	vtc_log(v->vl, 2, "Stop");
 	(void)varnish_ask_cli(v, "stop", NULL);
 	while (1) {
@@ -297,7 +423,8 @@ varnish_stop(struct varnish *v)
 		if (!strcmp(r, "Child in state stopped"))
 			break;
 		free(r);
-		sleep (1);
+		(void)sleep (1);
+		/* XXX: should fail eventually */
 	}
 }
 
@@ -313,6 +440,8 @@ varnish_wait(struct varnish *v)
 
 	if (v->cli_fd < 0)
 		return;
+	if (vtc_error)
+		(void)sleep(1);	/* give panic messages a chance */
 	varnish_stop(v);
 	vtc_log(v->vl, 2, "Wait");
 	AZ(close(v->cli_fd));
@@ -324,11 +453,16 @@ varnish_wait(struct varnish *v)
 	AZ(close(v->fds[0]));
 	r = wait4(v->pid, &status, 0, NULL);
 	vtc_log(v->vl, 2, "R %d Status: %04x", r, status);
-	if (WIFEXITED(status) && WEXITSTATUS(status) == 2)
+	if (WIFEXITED(status) && WEXITSTATUS(status) == 0)
 		return;
+#ifdef WCOREDUMP
 	vtc_log(v->vl, 0, "Bad exit code: %04x sig %x exit %x core %x",
 	    status, WTERMSIG(status), WEXITSTATUS(status),
 	    WCOREDUMP(status));
+#else
+	vtc_log(v->vl, 0, "Bad exit code: %04x sig %x exit %x",
+	    status, WTERMSIG(status), WEXITSTATUS(status));
+#endif
 }
 
 /**********************************************************************
@@ -342,9 +476,11 @@ varnish_cli(struct varnish *v, const char *cli, unsigned exp)
 
 	if (v->cli_fd < 0)
 		varnish_launch(v);
+	if (vtc_error)
+		return;
 	u = varnish_ask_cli(v, cli, NULL);
 	vtc_log(v->vl, 2, "CLI %03u <%s>", u, cli);
-	if (exp != 0 && exp != u)
+	if (exp != 0 && exp != (unsigned)u)
 		vtc_log(v->vl, 0, "FAIL CLI response %u expected %u", u, exp);
 }
 
@@ -360,6 +496,8 @@ varnish_vcl(struct varnish *v, const char *vcl, enum cli_status_e expect)
 
 	if (v->cli_fd < 0)
 		varnish_launch(v);
+	if (vtc_error)
+		return;
 	vsb = vsb_newauto();
 	AN(vsb);
 
@@ -400,6 +538,8 @@ varnish_vclbackend(struct varnish *v, const char *vcl)
 
 	if (v->cli_fd < 0)
 		varnish_launch(v);
+	if (vtc_error)
+		return;
 	vsb = vsb_newauto();
 	AN(vsb);
 
@@ -422,6 +562,8 @@ varnish_vclbackend(struct varnish *v, const char *vcl)
 	AZ(vsb_overflowed(vsb));
 
 	u = varnish_ask_cli(v, vsb_data(vsb), NULL);
+	if (u != CLIS_OK)
+		vtc_log(v->vl, 0, "FAIL VCL does not compile");
 	assert(u == CLIS_OK);
 	vsb_clear(vsb);
 	vsb_printf(vsb, "vcl.use vcl%d", v->vcl_nbr);
@@ -446,10 +588,10 @@ varnish_expect(const struct varnish *v, char * const *av) {
 
 	good = 0;
 
-	for (i = 0; i < 10; i++, usleep(100000)) {
+	for (i = 0; i < 10; i++, (void)usleep(100000)) {
 
 
-#define MAC_STAT(n, t, f, d)					\
+#define MAC_STAT(n, t, l, f, d)					\
 		if (!strcmp(av[0], #n)) {			\
 			val = v->stats->n;			\
 		} else
@@ -475,12 +617,13 @@ varnish_expect(const struct varnish *v, char * const *av) {
 		if (good)
 			break;
 	}
-	if (good)
+	if (good) {
 		vtc_log(v->vl, 2, "as expected: %s (%ju) %s %s",
 		    av[0], val, av[1], av[2]);
-	else
-		vtc_log(v->vl, 0, "Not true: %s (%ju) %s %s (%ju)",
-		    av[0], val, av[1], av[2], ref);
+		return;
+	}
+	vtc_log(v->vl, 0, "Not true: %s (%ju) %s %s (%ju)",
+	    av[0], val, av[1], av[2], ref);
 }
 
 /**********************************************************************
@@ -518,21 +661,13 @@ cmd_varnish(CMD_ARGS)
 	av++;
 
 	for (; *av != NULL; av++) {
-		if (!strcmp(*av, "-telnet")) {
-			AN(av[1]);
-			v->telnet = av[1];
-			av++;
-			continue;
-		}
-		if (!strcmp(*av, "-accept")) {
-			AN(av[1]);
-			v->accept = av[1];
-			av++;
-			continue;
-		}
+		if (vtc_error)
+			break;
 		if (!strcmp(*av, "-arg")) {
 			AN(av[1]);
-			v->args = av[1];
+			AZ(v->pid);
+			vsb_cat(v->args, " ");
+			vsb_cat(v->args, av[1]);
 			av++;
 			continue;
 		}
@@ -544,7 +679,7 @@ cmd_varnish(CMD_ARGS)
 		}
 		if (!strcmp(*av, "-cliok")) {
 			AN(av[1]);
-			varnish_cli(v, av[1], CLIS_OK);
+			varnish_cli(v, av[1], (unsigned)CLIS_OK);
 			av++;
 			continue;
 		}
@@ -555,10 +690,6 @@ cmd_varnish(CMD_ARGS)
 			av += 2;
 			continue;
 		}
-		if (!strcmp(*av, "-launch")) {
-			varnish_launch(v);
-			continue;
-		}
 		if (!strcmp(*av, "-start")) {
 			varnish_start(v);
 			continue;
@@ -596,6 +727,5 @@ cmd_varnish(CMD_ARGS)
 			continue;
 		}
 		vtc_log(v->vl, 0, "Unknown varnish argument: %s", *av);
-		exit (1);
 	}
 }
