@@ -3,15 +3,25 @@
  *
  * This file is in the public domain.
  *
- * $Id: miniobj.h 1996 2007-09-23 13:22:11Z des $
+ * $Id: miniobj.h 2470 2008-02-15 10:28:00Z phk $
  */
 
+#define ALLOC_OBJ(to, type_magic)					\
+	do {								\
+		(to) = calloc(sizeof *(to), 1);				\
+		if ((to) != NULL)					\
+			(to)->magic = (type_magic);			\
+	} while (0)
+
 #define FREE_OBJ(to)							\
 	do {								\
 		(to)->magic = (0);					\
 		free(to);						\
 	} while (0)
 
+#define VALID_OBJ(ptr, type_magic)					\
+	((ptr) != NULL && (ptr)->magic == (type_magic))
+
 #define CHECK_OBJ(ptr, type_magic)					\
 	do {								\
 		assert((ptr)->magic == type_magic);			\
@@ -23,6 +33,12 @@
 		assert((ptr)->magic == type_magic);			\
 	} while (0)
 
+#define CHECK_OBJ_ORNULL(ptr, type_magic)				\
+	do {								\
+		if ((ptr) != NULL)					\
+			assert((ptr)->magic == type_magic);		\
+	} while (0)
+
 #define CAST_OBJ(to, from, type_magic)					\
 	do {								\
 		(to) = (from);						\
