@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006-2008 Linpro AS
+ * Copyright (c) 2006-2009 Linpro AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -26,24 +26,26 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: vct.h 3580 2009-02-05 08:45:25Z tfheen $
+ * $Id: vct.h 4432 2010-01-07 11:28:24Z phk $
  */
 
 /* from libvarnish/vct.c */
 
-#define VCT_SP		(1<<0)
-#define VCT_CRLF	(1<<1)
-#define VCT_LWS		(VCT_CRLF | VCT_SP)
-#define VCT_CTL		(1<<2)
-#define VCT_ALPHA	(1<<3)
-#define VCT_SEPARATOR	(1<<4)
-#define VCT_DIGIT	(1<<5)
-#define VCT_HEX		(1<<6)
+#define VCT_SP			(1<<0)
+#define VCT_CRLF		(1<<1)
+#define VCT_LWS			(VCT_CRLF | VCT_SP)
+#define VCT_CTL			(1<<2)
+#define VCT_ALPHA		(1<<3)
+#define VCT_SEPARATOR		(1<<4)
+#define VCT_DIGIT		(1<<5)
+#define VCT_HEX			(1<<6)
+#define VCT_XMLNAMESTART	(1<<7)
+#define VCT_XMLNAME		(1<<8)
 
-extern unsigned char vct_typtab[256];
+extern const uint16_t vct_typtab[256];
 
 static inline int
-vct_is(unsigned char x, unsigned char y)
+vct_is(unsigned char x, uint16_t y)
 {
 
 	return (vct_typtab[x] & (y));
@@ -56,6 +58,8 @@ vct_is(unsigned char x, unsigned char y)
 #define vct_isalpha(x) vct_is(x, VCT_ALPHA)
 #define vct_issep(x) vct_is(x, VCT_SEPARATOR)
 #define vct_issepctl(x) vct_is(x, VCT_SEPARATOR | VCT_CTL)
+#define vct_isxmlnamestart(x) vct_is(x, VCT_XMLNAMESTART)
+#define vct_isxmlname(x) vct_is(x, VCT_XMLNAMESTART | VCT_XMLNAME)
 
 /* NB: VCT always operate in ASCII, don't replace 0x0d with \r etc. */
 #define vct_skipcrlf(p) (p[0] == 0x0d && p[1] == 0x0a ? 2 : 1)
