@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006-2008 Linpro AS
+ * Copyright (c) 2006-2009 Linpro AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -25,12 +25,13 @@
  * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
- *
- * $Id: vev.c 3079 2008-08-11 09:49:41Z tfheen $
  */
 
 #include "config.h"
 
+#include "svnid.h"
+SVNID("$Id: vev.c 4235 2009-09-11 13:06:15Z phk $")
+
 #include <stdio.h>
 #include <errno.h>
 #include <poll.h>
@@ -150,7 +151,7 @@ vev_get_sig(int sig)
 	if (sig < vev_nsig)
 		return (0);
 
-	os = calloc(sizeof *os, (sig + 1));
+	os = calloc(sizeof *os, (sig + 1L));
 	if (os == NULL)
 		return (ENOMEM);
 
