@@ -23,14 +23,15 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: vlu.c 3981 2009-03-23 08:47:37Z tfheen $
- *
  * Functions for assembling a bytestream into text-lines and calling
  * a function on each.
  */
 
 #include "config.h"
 
+#include "svnid.h"
+SVNID("$Id: vlu.c 4358 2009-11-19 19:03:25Z phk $")
+
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
@@ -46,7 +47,7 @@ struct vlu {
 	unsigned	bufl;
 	unsigned	bufp;
 	void		*priv;
-	int 		telnet;
+	int		telnet;
 	vlu_f	*func;
 };
 
@@ -62,12 +63,12 @@ VLU_New(void *priv, vlu_f *func, unsigned bufsize)
 		l->func = func;
 		l->priv = priv;
 		l->bufl = bufsize - 1;
-		l->buf = malloc(l->bufl + 1);
+		l->telnet = -1;
+		l->buf = malloc(l->bufl + 1L);
 		if (l->buf == NULL) {
 			FREE_OBJ(l);
 			l = NULL;
 		}
-		l->telnet = -1;
 	}
 	return (l);
 }
@@ -119,7 +120,7 @@ vlu_dotelnet(struct vlu *l, char *p)
 		/* Return WONT for these */
 		memcpy(tno, p, 3);
 		tno[1] = (char)252;
-		write(l->telnet, tno, 3);
+		(void)write(l->telnet, tno, 3);
 		i = 3;
 		break;
 	default:
@@ -144,7 +145,7 @@ LineUpProcess(struct vlu *l)
 	for (p = l->buf; *p != '\0'; p = q) {
 		/* Find first CR or NL */
 		for (q = p; *q != '\0'; q++) {
-			while (l->telnet >= 0 && *q == (char)255) 
+			while (l->telnet >= 0 && *q == (char)255)
 				if (vlu_dotelnet(l, q))
 					return (0);
 			if (*q == '\n' || *q == '\r')
@@ -193,3 +194,28 @@ VLU_File(FILE *f, struct vlu *l)
 	l->bufp = strlen(l->buf);
 	return (LineUpProcess(l));
 }
+
+int
+VLU_Data(const void *ptr, int len, struct vlu *l)
+{
+	const char *p;
+	int i = 0;
+
+	p = ptr;
+	CHECK_OBJ_NOTNULL(l, LINEUP_MAGIC);
+	if (len < 0)
+		len = strlen(p);
+	while (len > 0) {
+		i = len;
+		if (i > l->bufl - l->bufp)
+			i = l->bufl - l->bufp;
+		memcpy(l->buf + l->bufp, p, i);
+		l->bufp += i;
+		p += i;
+		len -= i;
+		i = LineUpProcess(l);
+		if (i)
+			break;
+	}
+	return (i);
+}
