@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006-2008 Linpro AS
+ * Copyright (c) 2006-2009 Linpro AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -25,12 +25,13 @@
  * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
- *
- * $Id: vcc_acl.c 3730 2009-02-11 07:02:43Z tfheen $
  */
 
 #include "config.h"
 
+#include "svnid.h"
+SVNID("$Id: vcc_acl.c 4459 2010-01-14 11:59:57Z phk $")
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <stddef.h>
@@ -39,7 +40,6 @@
 #include <sys/socket.h>
 #include <netdb.h>
 #include <netinet/in.h>
-#include <arpa/inet.h>
 
 #include "vsb.h"
 #include "vrt.h"
@@ -67,7 +67,7 @@ struct acl_e {
 		else if ((b) < (a))					\
 			return (1);					\
 	} while (0)
-		
+
 static int
 vcl_acl_cmp(struct acl_e *ae1, struct acl_e *ae2)
 {
@@ -250,14 +250,18 @@ vcc_acl_try_netnotation(struct tokenlist *tl, struct acl_e *ae)
 {
 	unsigned char b[4];
 	int i, j, k;
+	unsigned u;
 	const char *p;
 
 	memset(b, 0, sizeof b);
 	p = ae->t_addr->dec;
 	for (i = 0; i < 4; i++) {
-		j = sscanf(p, "%hhu%n", &b[i], &k);
+		j = sscanf(p, "%u%n", &u, &k);
 		if (j != 1)
 			return (0);
+		if (u & ~0xff)
+			return (0);
+		b[i] = (unsigned char)u;
 		if (p[k] == '\0')
 			break;
 		if (p[k] != '.')
@@ -451,7 +455,7 @@ void
 vcc_Cond_Ip(const struct var *vp, struct tokenlist *tl)
 {
 	unsigned tcond;
-	char *acln;
+	char acln[32];
 
 	switch (tl->t->tok) {
 	case '~':
@@ -468,13 +472,11 @@ vcc_Cond_Ip(const struct var *vp, struct tokenlist *tl)
 		VTAILQ_INIT(&tl->acl);
 		tcond = tl->t->tok;
 		vcc_NextToken(tl);
-		asprintf(&acln, "%u", tl->cnt);
-		assert(acln != NULL);
+		bprintf(acln, "%u", tl->cnt);
 		vcc_acl_entry(tl);
 		vcc_acl_emit(tl, acln, 1);
 		Fb(tl, 1, "%smatch_acl_anon_%s(sp, %s)\n",
 		    (tcond == T_NEQ ? "!" : ""), acln, vp->rname);
-		free(acln);
 		break;
 	default:
 		vsb_printf(tl->sb, "Invalid condition ");
@@ -490,7 +492,7 @@ void
 vcc_Acl(struct tokenlist *tl)
 {
 	struct token *an;
-	char *acln;
+	char acln[1024];
 
 	vcc_NextToken(tl);
 	VTAILQ_INIT(&tl->acl);
@@ -500,8 +502,7 @@ vcc_Acl(struct tokenlist *tl)
 	vcc_NextToken(tl);
 
 	vcc_AddDef(tl, an, R_ACL);
-	asprintf(&acln, "%.*s", PF(an));
-	assert(acln != NULL);
+	bprintf(acln, "%.*s", PF(an));
 
 	ExpectErr(tl, '{');
 	vcc_NextToken(tl);
@@ -516,6 +517,4 @@ vcc_Acl(struct tokenlist *tl)
 	vcc_NextToken(tl);
 
 	vcc_acl_emit(tl, acln, 0);
-
-	free(acln);
 }
