@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006-2008 Linpro AS
+ * Copyright (c) 2006-2009 Linpro AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -26,11 +26,15 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: vcc_action.c 3725 2009-02-10 15:01:28Z tfheen $
+ * This file parses the real action of the VCL code, the procedure
+ * statements which do the actual work.
  */
 
 #include "config.h"
 
+#include "svnid.h"
+SVNID("$Id: vcc_action.c 4626 2010-03-17 09:38:47Z phk $")
+
 #include <stdio.h>
 #include <string.h>
 
@@ -42,33 +46,6 @@
 
 /*--------------------------------------------------------------------*/
 
-static void
-parse_action(struct tokenlist *tl)
-{
-	int retval = 0;
-
-	Expect(tl, ID);
-
-#define VCL_RET_MAC(l, U)						\
-	do {								\
-		if (vcc_IdIs(tl->t, #l)) {				\
-			Fb(tl, 1, "VRT_done(sp, VCL_RET_" #U ");\n");	\
-			vcc_ProcAction(tl->curproc, VCL_RET_##U, tl->t);\
-			retval = 1;					\
-		}							\
-	} while (0);
-#include "vcl_returns.h"
-#undef VCL_RET_MAC
-	if (!retval) {
-		vsb_printf(tl->sb, "Expected action name.\n");
-		vcc_ErrWhere(tl, tl->t);
-		ERRCHK(tl);
-	}
-	vcc_NextToken(tl);
-}
-
-/*--------------------------------------------------------------------*/
-
 static void
 parse_restart(struct tokenlist *tl)
 {
@@ -185,7 +162,6 @@ parse_set(struct tokenlist *tl)
 	switch (vp->fmt) {
 	case INT:
 	case SIZE:
-	case RATE:
 	case TIME:
 	case RTIME:
 	case FLOAT:
@@ -207,8 +183,6 @@ parse_set(struct tokenlist *tl)
 				vcc_RTimeVal(tl);
 			else if (vp->fmt == SIZE)
 				vcc_SizeVal(tl);
-			else if (vp->fmt == RATE)
-				vcc_RateVal(tl);
 			else if (vp->fmt == FLOAT)
 				Fb(tl, 0, "%g", vcc_DoubleVal(tl));
 			else if (vp->fmt == INT) {
@@ -253,7 +227,7 @@ parse_set(struct tokenlist *tl)
 		vcc_ExpectCid(tl);
 		ERRCHK(tl);
 		vcc_AddRef(tl, tl->t, R_BACKEND);
-		Fb(tl, 0, "VGC_backend_%.*s", PF(tl->t));
+		Fb(tl, 0, "VGCDIR(_%.*s)", PF(tl->t));
 		vcc_NextToken(tl);
 		Fb(tl, 0, ");\n");
 		break;
@@ -271,7 +245,6 @@ parse_set(struct tokenlist *tl)
 		 * variable, so that varnishd can preallocate the worst case
 		 * number of slots for composing the hash string.
 		 */
-		tl->nhashcount++;
 		break;
 	case STRING:
 		if (tl->t->tok != '=') {
@@ -352,9 +325,9 @@ static const struct purge_var {
 	const char	*name;
 	unsigned	flag;
 } purge_var[] = {
-#define PVAR(a, b, c)   { a, b },
+#define PVAR(a, b, c)   { (a), (b) },
 #include "purge_vars.h"
-#undef PVAR     
+#undef PVAR
         { 0, 0 }
 };
 
@@ -385,7 +358,7 @@ parse_purge(struct tokenlist *tl)
 				vcc_ErrWhere(tl, tl->t);
 				return;
 			}
-			if (pv->flag &&
+			if ((pv->flag & PVAR_HTTP) &&
 			    tl->t->b + strlen(pv->name) >= tl->t->e) {
 				vsb_printf(tl->sb, "Missing header name.");
 				vcc_ErrWhere(tl, tl->t);
@@ -460,26 +433,6 @@ parse_purge_url(struct tokenlist *tl)
 
 /*--------------------------------------------------------------------*/
 
-static void
-parse_purge_hash(struct tokenlist *tl)
-{
-
-	vcc_NextToken(tl);
-	Expect(tl, '(');
-	vcc_NextToken(tl);
-
-	Fb(tl, 1, "VRT_purge(sp, \"obj.hash\", \"~\", ");
-	if (!vcc_StringVal(tl)) {
-		vcc_ExpectedStringval(tl);
-		return;
-	}
-	Expect(tl, ')');
-	vcc_NextToken(tl);
-	Fb(tl, 0, ", 0);\n");
-}
-
-/*--------------------------------------------------------------------*/
-
 static void
 parse_esi(struct tokenlist *tl)
 {
@@ -511,13 +464,29 @@ parse_panic(struct tokenlist *tl)
 static void
 parse_return(struct tokenlist *tl)
 {
+	int retval = 0;
+
 	vcc_NextToken(tl);
 	Expect(tl, '(');
 	vcc_NextToken(tl);
 	Expect(tl, ID);
 
-	parse_action(tl);
-	ERRCHK(tl);
+#define VCL_RET_MAC(l, U)						\
+	do {								\
+		if (vcc_IdIs(tl->t, #l)) {				\
+			Fb(tl, 1, "VRT_done(sp, VCL_RET_" #U ");\n");	\
+			vcc_ProcAction(tl->curproc, VCL_RET_##U, tl->t);\
+			retval = 1;					\
+		}							\
+	} while (0);
+#include "vcl_returns.h"
+#undef VCL_RET_MAC
+	if (!retval) {
+		vsb_printf(tl->sb, "Expected return action name.\n");
+		vcc_ErrWhere(tl, tl->t);
+		ERRCHK(tl);
+	}
+	vcc_NextToken(tl);
 	Expect(tl, ')');
 	vcc_NextToken(tl);
 }
@@ -542,6 +511,17 @@ parse_synthetic(struct tokenlist *tl)
 
 /*--------------------------------------------------------------------*/
 
+static void
+parse_new_syntax(struct tokenlist *tl)
+{
+
+	vsb_printf(tl->sb, "Please change \"%.*s\" to \"return(%.*s)\".\n",
+	    PF(tl->t), PF(tl->t));
+	vcc_ErrWhere(tl, tl->t);
+}
+
+/*--------------------------------------------------------------------*/
+
 typedef void action_f(struct tokenlist *tl);
 
 static struct action_table {
@@ -550,7 +530,9 @@ static struct action_table {
 } action_table[] = {
 	{ "restart",		parse_restart },
 	{ "error",		parse_error },
-#define VCL_RET_MAC(l, U) { #l, parse_action },
+
+#define VCL_RET_MAC(l, U)						\
+	{ #l,			parse_new_syntax },
 #include "vcl_returns.h"
 #undef VCL_RET_MAC
 
@@ -559,7 +541,6 @@ static struct action_table {
 	{ "esi",		parse_esi },
 	{ "panic",		parse_panic },
 	{ "purge",		parse_purge },
-	{ "purge_hash",		parse_purge_hash },
 	{ "purge_url",		parse_purge_url },
 	{ "remove",		parse_unset }, /* backward compatibility */
 	{ "set",		parse_set },
@@ -569,21 +550,19 @@ static struct action_table {
 	{ NULL,			NULL }
 };
 
-void
+int
 vcc_ParseAction(struct tokenlist *tl)
 {
 	struct token *at;
 	struct action_table *atp;
 
 	at = tl->t;
-	if (at->tok == ID) {
-		for(atp = action_table; atp->name != NULL; atp++) {
-			if (vcc_IdIs(at, atp->name)) {
-				atp->func(tl);
-				return;
-			}
+	assert (at->tok == ID);
+	for(atp = action_table; atp->name != NULL; atp++) {
+		if (vcc_IdIs(at, atp->name)) {
+			atp->func(tl);
+			return(1);
 		}
 	}
-	vsb_printf(tl->sb, "Expected action, 'if' or '}'\n");
-	vcc_ErrWhere(tl, at);
+	return (0);
 }
