@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006-2008 Linpro AS
+ * Copyright (c) 2006-2009 Linpro AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -26,8 +26,6 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: vcc_backend.c 3616 2009-02-05 11:43:20Z tfheen $
- *
  * A necessary explanation of a convoluted policy:
  *
  * In VCL we have backends and directors.
@@ -52,9 +50,13 @@
 
 #include "config.h"
 
+#include "svnid.h"
+SVNID("$Id: vcc_backend.c 4611 2010-03-16 08:52:37Z phk $")
+
 #include <sys/types.h>
 #include <sys/socket.h>
 
+#include <limits.h>
 #include <netdb.h>
 #include <stdio.h>
 #include <stdarg.h>
@@ -68,8 +70,8 @@
 
 struct host {
 	VTAILQ_ENTRY(host)      list;
-	int	                hnum;
 	struct token            *name;
+	char			*vgcname;
 };
 
 static const char *
@@ -98,7 +100,7 @@ static void
 Emit_Sockaddr(struct tokenlist *tl, const struct token *t_host,
     const char *port)
 {
-	struct addrinfo *res, *res0, hint;
+	struct addrinfo *res, *res0, *res1, hint;
 	int n4, n6, len, error, retval;
 	const char *emit, *multiple;
 	unsigned char *u;
@@ -136,9 +138,9 @@ Emit_Sockaddr(struct tokenlist *tl, const struct token *t_host,
 			    "Please specify which exact address "
 			    "you want to use, we found these:\n",
 			    PF(t_host), multiple);
-			for (res = res0; res; res = res->ai_next) {
-				error = getnameinfo(res->ai_addr,
-				    res->ai_addrlen, hbuf, sizeof hbuf,
+			for (res1 = res0; res1 != NULL; res1 = res1->ai_next) {
+				error = getnameinfo(res1->ai_addr,
+				    res1->ai_addrlen, hbuf, sizeof hbuf,
 				    NULL, 0, NI_NUMERICHOST);
 				AZ(error);
 				vsb_printf(tl->sb, "\t%s\n", hbuf);
@@ -188,17 +190,19 @@ Emit_Sockaddr(struct tokenlist *tl, const struct token *t_host,
  */
 
 static void
-vcc_EmitBeIdent(struct vsb *v, const struct token *name,
-    const struct token *qual, int serial, const struct token *first,
-    const struct token *last)
+vcc_EmitBeIdent(const struct tokenlist *tl, struct vsb *v,
+    int serial, const struct token *first, const struct token *last)
 {
 
-	AN(name);
-	AN(qual);
 	assert(first != last);
 	vsb_printf(v, "\t.ident =");
-	vsb_printf(v, "\n\t    \"%.*s %.*s [%d] \"",
-	    PF(qual), PF(name), serial);
+	if (serial >= 0) {
+		vsb_printf(v, "\n\t    \"%.*s %.*s [%d] \"",
+		    PF(tl->t_policy), PF(tl->t_dir), serial);
+	} else {
+		vsb_printf(v, "\n\t    \"%.*s %.*s \"",
+		    PF(tl->t_policy), PF(tl->t_dir));
+	}
 	while (1) {
 		if (first->dec != NULL)
 			vsb_printf(v, "\n\t    \"\\\"\" %.*s \"\\\" \"",
@@ -213,105 +217,6 @@ vcc_EmitBeIdent(struct vsb *v, const struct token *name,
 	vsb_printf(v, ",\n");
 }
 
-/*--------------------------------------------------------------------
- * Helper functions to complain about duplicate and missing fields
- *
- * XXX: idea: add groups to check for exclusivity, such that
- * XXX:    ("!foo", "?bar", "!{", "this", "that", "}", NULL)
- * XXX: means exactly one of "this" or "that", and
- * XXX:    ("!foo", "?bar", "?{", "this", "that", "}", NULL)
- * XXX: means at most one of "this" or "that".
- */
-
-struct fld_spec {
-	const char	*name;
-	struct token	*found;
-};
-
-void
-vcc_ResetFldSpec(struct fld_spec *f)
-{
-
-	for (; f->name != NULL; f++)
-		f->found = NULL;
-}
-
-struct fld_spec *
-vcc_FldSpec(struct tokenlist *tl, const char *first, ...)
-{
-	struct fld_spec f[100], *r;
-	int n = 0;
-	va_list ap;
-	const char *p;
-
-	f[n++].name = first;
-	va_start(ap, first);
-	while (1) {
-		p = va_arg(ap, const char *);
-		if (p == NULL)
-			break;
-		f[n++].name = p;
-		assert(n < 100);
-	}
-	va_end(ap);
-	f[n++].name = NULL;
-
-	vcc_ResetFldSpec(f);
-
-	r = TlAlloc(tl, sizeof *r * n);
-	memcpy(r, f, n * sizeof *r);
-	return (r);
-}
-
-void
-vcc_IsField(struct tokenlist *tl, struct token **t, struct fld_spec *fs)
-{
-	struct token *t_field;
-
-	ExpectErr(tl, '.');
-	vcc_NextToken(tl);
-	ExpectErr(tl, ID);
-	t_field = tl->t;
-	*t = t_field;
-	vcc_NextToken(tl);
-	ExpectErr(tl, '=');
-	vcc_NextToken(tl);
-
-	for (; fs->name != NULL; fs++) {
-		if (!vcc_IdIs(t_field, fs->name + 1))
-			continue;
-		if (fs->found == NULL) {
-			fs->found = t_field;
-			return;
-		}
-		vsb_printf(tl->sb, "Field ");
-		vcc_ErrToken(tl, t_field);
-		vsb_printf(tl->sb, " redefined at:\n");
-		vcc_ErrWhere(tl, t_field);
-		vsb_printf(tl->sb, "\nFirst defined at:\n");
-		vcc_ErrWhere(tl, fs->found);
-		return;
-	}
-	vsb_printf(tl->sb, "Unknown field: ");
-	vcc_ErrToken(tl, t_field);
-	vsb_printf(tl->sb, " at\n");
-	vcc_ErrWhere(tl, t_field);
-	return;
-}
-
-void
-vcc_FieldsOk(struct tokenlist *tl, const struct fld_spec *fs)
-{
-
-	for (; fs->name != NULL; fs++) {
-		if (*fs->name == '!' && fs->found == NULL) {
-			vsb_printf(tl->sb,
-			    "Mandatory field '%s' missing.\n", fs->name + 1);
-			tl->err = 1;
-		}
-	}
-}
-
 /*--------------------------------------------------------------------
  * Parse a backend probe specification
  */
@@ -340,15 +245,18 @@ vcc_ParseProbe(struct tokenlist *tl)
 	struct fld_spec *fs;
 	struct token *t_field;
 	struct token *t_did = NULL, *t_window = NULL, *t_threshold = NULL;
-	unsigned window, threshold;
+	struct token *t_initial = NULL;
+	unsigned window, threshold, initial, status;
 
 	fs = vcc_FldSpec(tl,
 	    "?url",
 	    "?request",
+	    "?expected_response",
 	    "?timeout",
 	    "?interval",
 	    "?window",
 	    "?threshold",
+	    "?initial",
 	    NULL);
 
 	ExpectErr(tl, '{');
@@ -356,6 +264,8 @@ vcc_ParseProbe(struct tokenlist *tl)
 
 	window = 0;
 	threshold = 0;
+	initial = 0;
+	status = 0;
 	Fb(tl, 0, "\t.probe = {\n");
 	while (tl->t->tok != '}') {
 
@@ -396,6 +306,22 @@ vcc_ParseProbe(struct tokenlist *tl)
 			window = vcc_UintVal(tl);
 			vcc_NextToken(tl);
 			ERRCHK(tl);
+		} else if (vcc_IdIs(t_field, "initial")) {
+			t_initial = tl->t;
+			initial = vcc_UintVal(tl);
+			vcc_NextToken(tl);
+			ERRCHK(tl);
+		} else if (vcc_IdIs(t_field, "expected_response")) {
+			status = vcc_UintVal(tl);
+			if (status < 100 || status > 999) {
+				vsb_printf(tl->sb,
+				    "Must specify .status with exactly three "
+				    " digits (100 <= x <= 999)\n");
+				vcc_ErrWhere(tl, tl->t);
+				return;
+			}
+			vcc_NextToken(tl);
+			ERRCHK(tl);
 		} else if (vcc_IdIs(t_field, "threshold")) {
 			t_threshold = tl->t;
 			threshold = vcc_UintVal(tl);
@@ -441,8 +367,14 @@ vcc_ParseProbe(struct tokenlist *tl)
 			vcc_ErrWhere(tl, t_window);
 		}
 		Fb(tl, 0, "\t\t.window = %u,\n", window);
-		Fb(tl, 0, "\t\t.threshold = %u\n", threshold);
+		Fb(tl, 0, "\t\t.threshold = %u,\n", threshold);
 	}
+	if (t_initial != NULL)
+		Fb(tl, 0, "\t\t.initial = %u,\n", initial);
+	else
+		Fb(tl, 0, "\t\t.initial = ~0U,\n", initial);
+	if (status > 0)
+		Fb(tl, 0, "\t\t.exp_status = %u,\n", status);
 	Fb(tl, 0, "\t},\n");
 	ExpectErr(tl, '}');
 	vcc_NextToken(tl);
@@ -455,19 +387,21 @@ vcc_ParseProbe(struct tokenlist *tl)
  */
 
 static void
-vcc_ParseHostDef(struct tokenlist *tl, int *nbh, const struct token *name,
-    const struct token *qual, int serial)
+vcc_ParseHostDef(struct tokenlist *tl, int serial, const char *vgcname)
 {
 	struct token *t_field;
 	struct token *t_first;
 	struct token *t_host = NULL;
 	struct token *t_port = NULL;
 	struct token *t_hosthdr = NULL;
+	unsigned saint = UINT_MAX;
 	const char *ep;
 	struct fld_spec *fs;
 	struct vsb *vsb;
 	unsigned u;
 
+	Fh(tl, 1, "\n#define VGC_backend_%s %d\n", vgcname, tl->ndirector);
+
 	fs = vcc_FldSpec(tl,
 	    "!host",
 	    "?port",
@@ -477,6 +411,7 @@ vcc_ParseHostDef(struct tokenlist *tl, int *nbh, const struct token *name,
 	    "?between_bytes_timeout",
 	    "?probe",
 	    "?max_connections",
+	    "?saintmode_threshold",
 	    NULL);
 	t_first = tl->t;
 
@@ -487,11 +422,11 @@ vcc_ParseHostDef(struct tokenlist *tl, int *nbh, const struct token *name,
 	AN(vsb);
 	tl->fb = vsb;
 
-	*nbh = tl->nbackend_host++;
-	Fb(tl, 0, "\nstatic const struct vrt_backend bh_%d = {\n", *nbh);
+	Fb(tl, 0, "\nstatic const struct vrt_backend vgc_dir_priv_%s = {\n",
+	    vgcname);
 
-	Fb(tl, 0, "\t.vcl_name = \"%.*s", PF(name));
-	if (serial)
+	Fb(tl, 0, "\t.vcl_name = \"%.*s", PF(tl->t_dir));
+	if (serial >= 0)
 		Fb(tl, 0, "[%d]", serial);
 	Fb(tl, 0, "\",\n");
 
@@ -560,6 +495,23 @@ vcc_ParseHostDef(struct tokenlist *tl, int *nbh, const struct token *name,
 			ExpectErr(tl, ';');
 			vcc_NextToken(tl);
 			Fb(tl, 0, "\t.max_connections = %u,\n", u);
+		} else if (vcc_IdIs(t_field, "saintmode_threshold")) {
+			u = vcc_UintVal(tl);
+			/* UINT_MAX == magic number to mark as unset, so
+			 * not allowed here.
+			 */
+			if (u == UINT_MAX) {
+				vsb_printf(tl->sb,
+				    "Value outside allowed range: ");
+				vcc_ErrToken(tl, tl->t);
+				vsb_printf(tl->sb, " at\n");
+				vcc_ErrWhere(tl, tl->t);
+			}
+			vcc_NextToken(tl);
+			ERRCHK(tl);
+			saint = u;
+			ExpectErr(tl, ';');
+			vcc_NextToken(tl);
 		} else if (vcc_IdIs(t_field, "probe")) {
 			vcc_ParseProbe(tl);
 			ERRCHK(tl);
@@ -600,7 +552,7 @@ vcc_ParseHostDef(struct tokenlist *tl, int *nbh, const struct token *name,
 	ExpectErr(tl, '}');
 
 	/* We have parsed it all, emit the ident string */
-	vcc_EmitBeIdent(tl->fb, name, qual, serial, t_first, tl->t);
+	vcc_EmitBeIdent(tl, tl->fb, serial, t_first, tl->t);
 
 	/* Emit the hosthdr field, fall back to .host if not specified */
 	Fb(tl, 0, "\t.hosthdr = ");
@@ -610,6 +562,8 @@ vcc_ParseHostDef(struct tokenlist *tl, int *nbh, const struct token *name,
 		EncToken(tl->fb, t_host);
 	Fb(tl, 0, ",\n");
 
+	Fb(tl, 0, "\t.saintmode_threshold = %d,\n",saint);
+
 	/* Close the struct */
 	Fb(tl, 0, "};\n");
 
@@ -620,6 +574,11 @@ vcc_ParseHostDef(struct tokenlist *tl, int *nbh, const struct token *name,
 	AZ(vsb_overflowed(vsb));
 	Fh(tl, 0, "%s", vsb_data(vsb));
 	vsb_delete(vsb);
+
+	Fi(tl, 0, "\tVRT_init_dir(cli, VCL_conf.director, \"simple\",\n"
+	    "\t    VGC_backend_%s, &vgc_dir_priv_%s);\n", vgcname, vgcname);
+	Ff(tl, 0, "\tVRT_fini_dir(cli, VGCDIR(%s));\n", vgcname);
+	tl->ndirector++;
 }
 
 /*--------------------------------------------------------------------
@@ -635,12 +594,14 @@ vcc_ParseHostDef(struct tokenlist *tl, int *nbh, const struct token *name,
  */
 
 void
-vcc_ParseBackendHost(struct tokenlist *tl, int *nbh, const struct token *name,
-    const struct token *qual, int serial)
+vcc_ParseBackendHost(struct tokenlist *tl, int serial, char **nm)
 {
 	struct host *h;
 	struct token *t;
+	char vgcname[BUFSIZ];
 
+	AN(nm);
+	*nm = NULL;
 	if (tl->t->tok == ID) {
 		VTAILQ_FOREACH(h, &tl->hosts, list) {
 			if (vcc_Teq(h->name, tl->t))
@@ -657,15 +618,22 @@ vcc_ParseBackendHost(struct tokenlist *tl, int *nbh, const struct token *name,
 		vcc_NextToken(tl);
 		ExpectErr(tl, ';');
 		vcc_NextToken(tl);
-		*nbh = h->hnum;
+		*nm = h->vgcname;
 	} else if (tl->t->tok == '{') {
 		t = tl->t;
-		vcc_ParseHostDef(tl, nbh, name, qual, serial);
+
+		sprintf(vgcname, "%.*s_%d", PF(tl->t_dir), serial);
+
+		Ff(tl, 0, "\tVRT_fini_dir(cli, VGCDIR(_%.*s));\n",
+		    PF(tl->t_dir));
+		vcc_ParseHostDef(tl, serial, vgcname);
 		if (tl->err) {
 			vsb_printf(tl->sb,
 			    "\nIn backend host specification starting at:\n");
 			vcc_ErrWhere(tl, t);
 		}
+		*nm = strdup(vgcname);	 /* XXX */
+
 		return;
 	} else {
 		vsb_printf(tl->sb,
@@ -683,30 +651,22 @@ vcc_ParseBackendHost(struct tokenlist *tl, int *nbh, const struct token *name,
  */
 
 static void
-vcc_ParseSimpleDirector(struct tokenlist *tl, const struct token *t_first,
-    struct token *t_dir)
+vcc_ParseSimpleDirector(struct tokenlist *tl)
 {
 	struct host *h;
+	char vgcname[BUFSIZ];
 
 	h = TlAlloc(tl, sizeof *h);
-	h->name = t_dir;
+	h->name = tl->t_dir;
+	vcc_AddDef(tl, tl->t_dir, R_BACKEND);
+	sprintf(vgcname, "_%.*s", PF(h->name));
+	h->vgcname = TlAlloc(tl, strlen(vgcname) + 1);
+	strcpy(h->vgcname, vgcname);
 
-	vcc_ParseHostDef(tl, &h->hnum, h->name, t_first, 0);
+	vcc_ParseHostDef(tl, -1, vgcname);
 	ERRCHK(tl);
 
 	VTAILQ_INSERT_TAIL(&tl->hosts, h, list);
-
-	Fi(tl, 0,
-	    "\tVRT_init_dir_simple(cli, &VGC_backend_%.*s , &sbe_%.*s);\n",
-	    PF(h->name), PF(h->name));
-	Ff(tl, 0, "\tVRT_fini_dir(cli, VGC_backend_%.*s);\n", PF(h->name));
-
-	Fc(tl, 0, "\nstatic const struct vrt_dir_simple sbe_%.*s = {\n",
-	    PF(h->name));
-	Fc(tl, 0, "\t.name = \"%.*s\",\n", PF(h->name));
-	Fc(tl, 0, "\t.host = &bh_%d,\n", h->hnum);
-	Fc(tl, 0, "};\n");
-
 }
 
 /*--------------------------------------------------------------------
@@ -717,7 +677,9 @@ static const struct dirlist {
 	const char	*name;
 	parsedirector_f	*func;
 } dirlist[] = {
+	{ "hash",		vcc_ParseRandomDirector },
 	{ "random",		vcc_ParseRandomDirector },
+	{ "client",		vcc_ParseRandomDirector },
 	{ "round-robin",	vcc_ParseRoundRobinDirector },
 	{ NULL,		NULL }
 };
@@ -725,7 +687,7 @@ static const struct dirlist {
 void
 vcc_ParseDirector(struct tokenlist *tl)
 {
-	struct token *t_dir, *t_first, *t_policy;
+	struct token *t_first;
 	struct dirlist const *dl;
 
 	t_first = tl->t;
@@ -733,38 +695,46 @@ vcc_ParseDirector(struct tokenlist *tl)
 
 	vcc_ExpectCid(tl);		/* ID: name */
 	ERRCHK(tl);
-	t_dir = tl->t;
+	tl->t_dir = tl->t;
 	vcc_NextToken(tl);
 
-	Fh(tl, 1, "\n#define VGC_backend_%.*s (VCL_conf.director[%d])\n",
-	    PF(t_dir), tl->ndirector);
-	vcc_AddDef(tl, t_dir, R_BACKEND);
-	tl->ndirector++;
 
 	if (vcc_IdIs(t_first, "backend")) {
-		vcc_ParseSimpleDirector(tl, t_first, t_dir);
+		tl->t_policy = t_first;
+		vcc_ParseSimpleDirector(tl);
 	} else {
+		Fh(tl, 1, "\n#define VGC_backend__%.*s %d\n",
+		    PF(tl->t_dir), tl->ndirector);
+		vcc_AddDef(tl, tl->t_dir, R_BACKEND);
+		tl->ndirector++;
 		ExpectErr(tl, ID);		/* ID: policy */
-		t_policy = tl->t;
+		tl->t_policy = tl->t;
 		vcc_NextToken(tl);
 
 		for (dl = dirlist; dl->name != NULL; dl++)
-			if (vcc_IdIs(t_policy, dl->name))
+			if (vcc_IdIs(tl->t_policy, dl->name))
 				break;
 		if (dl->name == NULL) {
 			vsb_printf(tl->sb, "Unknown director policy: ");
-			vcc_ErrToken(tl, t_policy);
+			vcc_ErrToken(tl, tl->t_policy);
 			vsb_printf(tl->sb, " at\n");
-			vcc_ErrWhere(tl, t_policy);
+			vcc_ErrWhere(tl, tl->t_policy);
 			return;
 		}
 		ExpectErr(tl, '{');
 		vcc_NextToken(tl);
-		dl->func(tl, t_policy, t_dir);
+		dl->func(tl);
 		if (!tl->err) {
 			ExpectErr(tl, '}');
 			vcc_NextToken(tl);
 		}
+		Fi(tl, 0,
+		    "\tVRT_init_dir(cli, VCL_conf.director, \"%.*s\",\n",
+		    PF(tl->t_policy));
+		Fi(tl, 0, "\t    VGC_backend__%.*s, &vgc_dir_priv_%.*s);\n",
+		    PF(tl->t_dir), PF(tl->t_dir));
+
+
 	}
 	if (tl->err) {
 		vsb_printf(tl->sb,
@@ -772,4 +742,6 @@ vcc_ParseDirector(struct tokenlist *tl)
 		vcc_ErrWhere(tl, t_first);
 		return;
 	}
+	tl->t_policy = NULL;
+	tl->t_dir = NULL;
 }
