@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006-2008 Linpro AS
+ * Copyright (c) 2006-2009 Linpro AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -25,14 +25,9 @@
  * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
- *
- * $Id: vcc_compile.c 3678 2009-02-06 14:27:47Z tfheen $
  */
 
 /*
- * XXX:
- *	generate interface structure
- *
  * XXX:
  *	Better error messages, throughout.
  *	>It also accured to me that we could link the errors to the error
@@ -53,20 +48,17 @@
  *	>Read more about this type of error:
  *	>http://varnish/doc/error.html#Unknown%20variable
  *
- * XXX:
- *	Create proper tmp filenames for .h, .c and .o
- *
- * XXX:
- *	and all the rest...
  */
 
 #include "config.h"
 
+#include "svnid.h"
+SVNID("$Id: vcc_compile.c 4616 2010-03-16 09:59:08Z phk $")
+
 #include <sys/stat.h>
 
 #include <ctype.h>
 #include <errno.h>
-#include <fcntl.h>
 #include <stdio.h>
 #include <stdarg.h>
 #include <stdlib.h>
@@ -96,8 +88,8 @@ static const char *vcc_default_vcl_b, *vcc_default_vcl_e;
 
 /*--------------------------------------------------------------------*/
 
-void
-TlFree(struct tokenlist *tl, void *p)
+static void
+TlDoFree(struct tokenlist *tl, void *p)
 {
 	struct membit *mb;
 
@@ -115,7 +107,7 @@ TlAlloc(struct tokenlist *tl, unsigned len)
 
 	p = calloc(len, 1);
 	assert(p != NULL);
-	TlFree(tl, p);
+	TlDoFree(tl, p);
 	return (p);
 }
 
@@ -134,7 +126,7 @@ IsMethod(const struct token *t)
 }
 
 /*--------------------------------------------------------------------
- * Printf output to the two vsbs, possibly indented
+ * Printf output to the vsbs, possibly indented
  */
 
 void
@@ -357,7 +349,6 @@ EmitStruct(const struct tokenlist *tl)
 	Fc(tl, 0, "\t.nsrc = %u,\n", tl->nsources);
 	Fc(tl, 0, "\t.srcname = srcname,\n");
 	Fc(tl, 0, "\t.srcbody = srcbody,\n");
-	Fc(tl, 0, "\t.nhashcount = %u,\n", tl->nhashcount);
 #define VCL_MET_MAC(l,u,b) \
 	Fc(tl, 0, "\t." #l "_func = VGC_function_vcl_" #l ",\n");
 #include "vcl_returns.h"
@@ -553,6 +544,9 @@ vcc_CompileSource(struct vsb *sb, struct source *sp)
 	Fh(tl, 0, "\n/* ---===### VCC generated below here ###===---*/\n");
 	Fh(tl, 0, "\nextern const struct VCL_conf VCL_conf;\n");
 
+	/* Macro for accessing directors */
+	Fh(tl, 0, "#define VGCDIR(n) VCL_conf.director[VGC_backend_##n]\n");
+
 	/* Register and lex the main source */
 	VTAILQ_INSERT_TAIL(&tl->sources, sp, list);
 	sp->idx = tl->nsources++;
