@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006-2008 Linpro AS
+ * Copyright (c) 2006-2009 Linpro AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -26,7 +26,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: vcc_compile.h 3676 2009-02-06 14:18:21Z tfheen $
+ * $Id: vcc_compile.h 4617 2010-03-16 10:23:15Z phk $
  */
 
 #include "vqueue.h"
@@ -81,7 +81,6 @@ struct tokenlist {
 	VTAILQ_HEAD(, ref)	refs;
 	struct vsb		*sb;
 	int			err;
-	int			nbackend_host;
 	int			ndirector;
 	VTAILQ_HEAD(, proc)	procs;
 	struct proc		*curproc;
@@ -89,8 +88,10 @@ struct tokenlist {
 
 	VTAILQ_HEAD(, acl_e)	acl;
 
+	struct token		*t_dir;
+	struct token		*t_policy;
+
 	unsigned		recnt;
-	unsigned		nhashcount;
 	unsigned		nsockaddr;
 };
 
@@ -100,7 +101,6 @@ enum var_type {
 	INT,
 	FLOAT,
 	SIZE,
-	RATE,
 	TIME,
 	RTIME,
 	STRING,
@@ -154,16 +154,14 @@ void vcc_Acl(struct tokenlist *tl);
 void vcc_Cond_Ip(const struct var *vp, struct tokenlist *tl);
 
 /* vcc_action.c */
-void vcc_ParseAction(struct tokenlist *tl);
+int vcc_ParseAction(struct tokenlist *tl);
 
 /* vcc_backend.c */
 struct fld_spec;
-typedef void parsedirector_f(struct tokenlist *tl,
-    const struct token *t_policy, const struct token *t_dir);
+typedef void parsedirector_f(struct tokenlist *tl);
 
 void vcc_ParseDirector(struct tokenlist *tl);
-void vcc_ParseBackendHost(struct tokenlist *tl, int *nbr,
-    const struct token *name, const struct token *qual, int serial);
+void vcc_ParseBackendHost(struct tokenlist *tl, int serial, char **nm);
 struct fld_spec * vcc_FldSpec(struct tokenlist *tl, const char *first, ...);
 void vcc_ResetFldSpec(struct fld_spec *f);
 void vcc_IsField(struct tokenlist *tl, struct token **t, struct fld_spec *fs);
@@ -171,6 +169,13 @@ void vcc_FieldsOk(struct tokenlist *tl, const struct fld_spec *fs);
 
 /* vcc_compile.c */
 extern struct method method_tab[];
+/*
+ * H -> Header, before the C code
+ * C -> C-code
+ * B -> Body of function, ends up in C once function is completed
+ * I -> Initializer function
+ * F -> Finish function
+ */
 void Fh(const struct tokenlist *tl, int indent, const char *fmt, ...);
 void Fc(const struct tokenlist *tl, int indent, const char *fmt, ...);
 void Fb(const struct tokenlist *tl, int indent, const char *fmt, ...);
@@ -178,7 +183,6 @@ void Fi(const struct tokenlist *tl, int indent, const char *fmt, ...);
 void Ff(const struct tokenlist *tl, int indent, const char *fmt, ...);
 void EncToken(struct vsb *sb, const struct token *t);
 int IsMethod(const struct token *t);
-void TlFree(struct tokenlist *tl, void *p);
 void *TlAlloc(struct tokenlist *tl, unsigned len);
 
 /* vcc_dir_random.c */
@@ -192,7 +196,6 @@ extern struct var vcc_vars[];
 
 /* vcc_parse.c */
 void vcc_Parse(struct tokenlist *tl);
-void vcc_RateVal(struct tokenlist *tl);
 void vcc_RTimeVal(struct tokenlist *tl);
 void vcc_TimeVal(struct tokenlist *tl);
 void vcc_SizeVal(struct tokenlist *tl);
@@ -200,7 +203,7 @@ unsigned vcc_UintVal(struct tokenlist *tl);
 double vcc_DoubleVal(struct tokenlist *tl);
 
 /* vcc_string.c */
-char *vcc_regexp(struct tokenlist *tl, int sub);
+char *vcc_regexp(struct tokenlist *tl);
 int vcc_StringVal(struct tokenlist *tl);
 void vcc_ExpectedStringval(struct tokenlist *tl);
 
@@ -240,4 +243,3 @@ int vcc_CheckUses(struct tokenlist *tl);
 #define ErrInternal(tl) vcc__ErrInternal(tl, __func__, __LINE__)
 #define Expect(a, b) vcc__Expect(a, b, __LINE__)
 #define ExpectErr(a, b) do { vcc__Expect(a, b, __LINE__); ERRCHK(a);} while (0)
-
