@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006-2008 Linpro AS
+ * Copyright (c) 2006-2009 Linpro AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -25,12 +25,13 @@
  * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
- *
- * $Id: vcc_parse.c 3718 2009-02-10 14:25:49Z tfheen $
  */
 
 #include "config.h"
 
+#include "svnid.h"
+SVNID("$Id: vcc_parse.c 4617 2010-03-16 10:23:15Z phk $")
+
 #include <stdio.h>
 #include <string.h>
 
@@ -120,23 +121,6 @@ SizeUnit(struct tokenlist *tl)
 	return (sc);
 }
 
-/*--------------------------------------------------------------------
- * Recognize and convert units of rate as { space '/' time }
- */
-
-static double
-RateUnit(struct tokenlist *tl)
-{
-	double sc;
-
-	assert(tl->t->tok == ID);
-	sc = SizeUnit(tl);
-	Expect(tl, '/');
-	vcc_NextToken(tl);
-	sc /= TimeUnit(tl);
-	return (sc);
-}
-
 /*--------------------------------------------------------------------
  * Recognize and convert { CNUM } to unsigned value
  */
@@ -227,18 +211,6 @@ vcc_SizeVal(struct tokenlist *tl)
 	Fb(tl, 0, "(%g * %g)", v, sc);
 }
 
-void
-vcc_RateVal(struct tokenlist *tl)
-{
-	double v, sc;
-
-	v = vcc_DoubleVal(tl);
-	ERRCHK(tl);
-	ExpectErr(tl, ID);
-	sc = RateUnit(tl);
-	Fb(tl, 0, "(%g * %g)", v, sc);
-}
-
 /*--------------------------------------------------------------------*/
 
 static void
@@ -253,7 +225,7 @@ Cond_String(const struct var *vp, struct tokenlist *tl)
 		     tl->t->tok == '~' ? "" : "!");
 		vcc_NextToken(tl);
 		ExpectErr(tl, CSTR);
-		p = vcc_regexp(tl, 0);
+		p = vcc_regexp(tl);
 		ERRCHK(tl);
 		vcc_NextToken(tl);
 		Fb(tl, 1, "%s, %s)\n", vp->rname, p);
@@ -263,10 +235,11 @@ Cond_String(const struct var *vp, struct tokenlist *tl)
 		Fb(tl, 1, "%sVRT_strcmp(%s, ",
 		    tl->t->tok == T_EQ ? "!" : "", vp->rname);
 		vcc_NextToken(tl);
-		ExpectErr(tl, CSTR);
-		EncToken(tl->fb, tl->t);
+		if (!vcc_StringVal(tl)) {
+			vcc_ExpectedStringval(tl);
+			break;
+		}
 		Fb(tl, 0, ")\n");
-		vcc_NextToken(tl);
 		break;
 	default:
 		Fb(tl, 1, "%s != (void*)0\n", vp->rname);
@@ -292,6 +265,9 @@ Cond_Int(const struct var *vp, struct tokenlist *tl)
 		case TIME:
 			vcc_TimeVal(tl);
 			break;
+		case RTIME:
+			vcc_RTimeVal(tl);
+			break;
 		case INT:
 			ExpectErr(tl, CNUM);
 			Fb(tl, 0, "%.*s ", PF(tl->t));
@@ -349,7 +325,7 @@ Cond_Backend(const struct var *vp, struct tokenlist *tl)
 	vcc_ExpectCid(tl);
 	ERRCHK(tl);
 	vcc_AddRef(tl, tl->t, R_BACKEND);
-	Fb(tl, 1, "VGC_backend_%.*s\n", PF(tl->t));
+	Fb(tl, 1, "VGCDIR(_%.*s)\n", PF(tl->t));
 	vcc_NextToken(tl);
 }
 
@@ -382,6 +358,7 @@ Cond_2(struct tokenlist *tl)
 		case IP:	L(tl, vcc_Cond_Ip(vp, tl)); break;
 		case STRING:	L(tl, Cond_String(vp, tl)); break;
 		case TIME:	L(tl, Cond_Int(vp, tl)); break;
+		case RTIME:	L(tl, Cond_Int(vp, tl)); break;
 		case BACKEND:	L(tl, Cond_Backend(vp, tl)); break;
 		default:
 			vsb_printf(tl->sb,
@@ -490,6 +467,7 @@ IfStmt(struct tokenlist *tl)
 static void
 Compound(struct tokenlist *tl)
 {
+	int i;
 
 	ExpectErr(tl, '{');
 	Fb(tl, 1, "{\n");
@@ -521,12 +499,21 @@ Compound(struct tokenlist *tl)
 			    "End of input while in compound statement\n");
 			tl->err = 1;
 			return;
-		default:
-			vcc_ParseAction(tl);
+		case ID:
+			i = vcc_ParseAction(tl);
 			ERRCHK(tl);
-			ExpectErr(tl, ';');
-			vcc_NextToken(tl);
-			break;
+			if (i) {
+				ExpectErr(tl, ';');
+				vcc_NextToken(tl);
+				break;
+			}
+			/* FALLTHROUGH */
+		default:
+			/* We deliberately do not mention inline C */
+			vsb_printf(tl->sb,
+			    "Expected an action, 'if', '{' or '}'\n");
+			vcc_ErrWhere(tl, tl->t);
+			return;
 		}
 	}
 }
@@ -582,8 +569,10 @@ Function(struct tokenlist *tl)
 
 /*--------------------------------------------------------------------
  * Top level of parser, recognize:
+ * 	Inline C-code
+ *	ACL definitions
  *	Function definitions
- *	Backend definitions
+ *	Backend & Director definitions
  *	End of input
  */
 
@@ -626,6 +615,7 @@ vcc_Parse(struct tokenlist *tl)
 				break;
 			/* FALLTHROUGH */
 		default:
+			/* We deliberately do not mention inline-C */
 			vsb_printf(tl->sb, "Expected one of\n\t");
 			for (tp = toplev; tp->name != NULL; tp++) {
 				if (tp[1].name == NULL)
