@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006-2008 Linpro AS
+ * Copyright (c) 2006-2009 Linpro AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -25,12 +25,13 @@
  * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
- *
- * $Id: vcc_var.c 3586 2009-02-05 09:17:38Z tfheen $
  */
 
 #include "config.h"
 
+#include "svnid.h"
+SVNID("$Id: vcc_var.c 4459 2010-01-14 11:59:57Z phk $")
+
 #include <stdio.h>
 #include <string.h>
 
@@ -47,7 +48,8 @@ HeaderVar(struct tokenlist *tl, const struct token *t, const struct var *vh)
 {
 	char *p;
 	struct var *v;
-	int i;
+	int i, l;
+	char buf[258];
 
 	(void)tl;
 
@@ -63,16 +65,22 @@ HeaderVar(struct tokenlist *tl, const struct token *t, const struct var *vh)
 	v->fmt = STRING;
 	v->hdr = vh->hdr;
 	v->methods = vh->methods;
-	asprintf(&p, "VRT_GetHdr(sp, %s, \"\\%03o%s:\")", v->hdr,
-	    (unsigned)(strlen(v->name + vh->len) + 1), v->name + vh->len);
-	AN(p);
-	TlFree(tl, p);
+	l = strlen(v->name + vh->len) + 1;
+
+	bprintf(buf, "VRT_GetHdr(sp, %s, \"\\%03o%s:\")",
+	    v->hdr, (unsigned)l, v->name + vh->len);
+	i = strlen(buf);
+	p = TlAlloc(tl, i + 1);
+	memcpy(p, buf, i + 1);
 	v->rname = p;
-	asprintf(&p, "VRT_SetHdr(sp, %s, \"\\%03o%s:\", ", v->hdr,
-	    (unsigned)(strlen(v->name + vh->len) + 1), v->name + vh->len);
-	AN(p);
-	TlFree(tl, p);
+
+	bprintf(buf, "VRT_SetHdr(sp, %s, \"\\%03o%s:\", ",
+	    v->hdr, (unsigned)l, v->name + vh->len);
+	i = strlen(buf);
+	p =  TlAlloc(tl, i + 1);
+	memcpy(p, buf, i + 1);
 	v->lname = p;
+
 	return (v);
 }
 
