@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006-2008 Linpro AS
+ * Copyright (c) 2006-2009 Linpro AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -26,8 +26,6 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: vcc_xref.c 3676 2009-02-06 14:18:21Z tfheen $
- *
  * This file contains code for two cross-reference or consistency checks.
  *
  * The first check is simply that all functions, acls and backends are
@@ -41,6 +39,9 @@
 
 #include "config.h"
 
+#include "svnid.h"
+SVNID("$Id: vcc_xref.c 4358 2009-11-19 19:03:25Z phk $")
+
 #include <stdio.h>
 
 #include "vsb.h"
@@ -268,7 +269,7 @@ vcc_CheckActionRecurse(struct tokenlist *tl, struct proc *p, unsigned bitmap)
 	u = p->ret_bitmap & ~bitmap;
 	if (u) {
 /*lint -save -e525 -e539 */
-#define VCL_RET_MAC(l, U) 						\
+#define VCL_RET_MAC(l, U)						\
 		if (u & (1 << (VCL_RET_##U))) {				\
 			vsb_printf(tl->sb, "Invalid return \"" #l "\"\n");\
 			vcc_ErrWhere(tl, p->return_tok[VCL_RET_##U]);	\
